@@ -1,5 +1,5 @@
 /******************************************************************************
-** Copyright (c) 2009-2016, Intel Corporation                                **
+** Copyright (c) 2009-2017, Intel Corporation                                **
 ** All rights reserved.                                                      **
 **                                                                           **
 ** Redistribution and use in source and binary forms, with or without        **
@@ -31,12 +31,16 @@
 #ifndef LIBXSMM_TIMER_H
 #define LIBXSMM_TIMER_H
 
-#include <libxsmm.h>
+#include "libxsmm_macros.h"
 
 
-/** Returns the current clock tick using a platform-specific resolution. */
-LIBXSMM_EXTERN_C LIBXSMM_RETARGETABLE unsigned long long libxsmm_timer_tick(void);
-/** Returns the duration between two clock ticks (seconds). */
-LIBXSMM_EXTERN_C LIBXSMM_RETARGETABLE double libxsmm_timer_duration(unsigned long long tick0, unsigned long long tick1);
+/** Returns the current clock tick of a monotonic timer source; uses a platform-specific resolution. */
+LIBXSMM_API unsigned long long libxsmm_timer_tick(void);
+
+/** Returns the current tick of a (monotonic) platform-specific counter; not necessarily CPU cycles. */
+LIBXSMM_API unsigned long long libxsmm_timer_xtick(void);
+
+/** Returns the duration (in seconds) between two values received by libxsmm_timer_tick. */
+LIBXSMM_API double libxsmm_timer_duration(unsigned long long tick0, unsigned long long tick1);
 
 #endif /*LIBXSMM_TIMER_H*/
