@@ -1,5 +1,5 @@
 !*****************************************************************************!
-!* Copyright (c) 2015-2016, Intel Corporation                                *!
+!* Copyright (c) 2015-2017, Intel Corporation                                *!
 !* All rights reserved.                                                      *!
 !*                                                                           *!
 !* Redistribution and use in source and binary forms, with or without        *!
@@ -35,7 +35,7 @@ MODULE STREAM_UPDATE_KERNELS
 
   INTERFACE
     SUBROUTINE stream_update_helmholtz( i_g1, i_g2, i_g3, i_tm1, i_tm2, i_tm3, &
-                                        i_a, i_b, io_c, i_h1, i_h2, i_length ) & 
+                                        i_a, i_b, io_c, i_h1, i_h2, i_length ) &
                                       BIND(C, name='stream_update_helmholtz')
       IMPORT :: C_DOUBLE, C_INT
       REAL(KIND=C_DOUBLE), DIMENSION(*), INTENT(IN)    :: i_g1
@@ -53,7 +53,7 @@ MODULE STREAM_UPDATE_KERNELS
     END SUBROUTINE
 
     SUBROUTINE stream_update_helmholtz_no_h2( i_g1, i_g2, i_g3, i_tm1, i_tm2, i_tm3, &
-                                              io_c, i_h1, i_length ) & 
+                                              io_c, i_h1, i_length ) &
                                             BIND(C, name='stream_update_helmholtz_no_h2')
       IMPORT :: C_DOUBLE, C_INT
       REAL(KIND=C_DOUBLE), DIMENSION(*), INTENT(IN)    :: i_g1
@@ -67,7 +67,25 @@ MODULE STREAM_UPDATE_KERNELS
       INTEGER(C_INT),      VALUE,        INTENT(IN)    :: i_length
     END SUBROUTINE
 
-    SUBROUTINE stream_vector_compscale( i_a, i_b, io_c, i_length ) & 
+    SUBROUTINE stream_update_var_helmholtz( i_g1, i_g2, i_g3, i_tm1, i_tm2, i_tm3, &
+                                            i_a, i_b, io_c, i_h1, i_h2, i_length ) &
+                                          BIND(C, name='stream_update_var_helmholtz')
+      IMPORT :: C_DOUBLE, C_INT
+      REAL(KIND=C_DOUBLE), DIMENSION(*), INTENT(IN)    :: i_g1
+      REAL(KIND=C_DOUBLE), DIMENSION(*), INTENT(IN)    :: i_g2
+      REAL(KIND=C_DOUBLE), DIMENSION(*), INTENT(IN)    :: i_g3
+      REAL(KIND=C_DOUBLE), DIMENSION(*), INTENT(IN)    :: i_tm1
+      REAL(KIND=C_DOUBLE), DIMENSION(*), INTENT(IN)    :: i_tm2
+      REAL(KIND=C_DOUBLE), DIMENSION(*), INTENT(IN)    :: i_tm3
+      REAL(KIND=C_DOUBLE), DIMENSION(*), INTENT(IN)    :: i_a
+      REAL(KIND=C_DOUBLE), DIMENSION(*), INTENT(IN)    :: i_b
+      REAL(KIND=C_DOUBLE), DIMENSION(*), INTENT(INOUT) :: io_c
+      REAL(KIND=C_DOUBLE), DIMENSION(*), INTENT(IN)    :: i_h1
+      REAL(KIND=C_DOUBLE), DIMENSION(*), INTENT(IN)    :: i_h2
+      INTEGER(C_INT),      VALUE,        INTENT(IN)    :: i_length
+    END SUBROUTINE
+
+    SUBROUTINE stream_vector_compscale( i_a, i_b, io_c, i_length ) &
                                             BIND(C, name='stream_vector_compscale')
       IMPORT :: C_DOUBLE, C_INT
       REAL(KIND=C_DOUBLE), DIMENSION(*), INTENT(IN)    :: i_a
@@ -76,7 +94,7 @@ MODULE STREAM_UPDATE_KERNELS
       INTEGER(C_INT),      VALUE,        INTENT(IN)    :: i_length
     END SUBROUTINE
 
-    SUBROUTINE stream_vector_copy( i_a, io_c, i_length ) & 
+    SUBROUTINE stream_vector_copy( i_a, io_c, i_length ) &
                                             BIND(C, name='stream_vector_copy')
       IMPORT :: C_DOUBLE, C_INT
       REAL(KIND=C_DOUBLE), DIMENSION(*), INTENT(IN)    :: i_a
@@ -84,7 +102,7 @@ MODULE STREAM_UPDATE_KERNELS
       INTEGER(C_INT),      VALUE,        INTENT(IN)    :: i_length
     END SUBROUTINE
 
-    SUBROUTINE stream_vector_set( i_scalar, io_c, i_length ) & 
+    SUBROUTINE stream_vector_set( i_scalar, io_c, i_length ) &
                                             BIND(C, name='stream_vector_set')
       IMPORT :: C_DOUBLE, C_INT
       REAL(KIND=C_DOUBLE), VALUE,        INTENT(IN)    :: i_scalar
