@@ -1,5 +1,5 @@
 /******************************************************************************
-** Copyright (c) 2015-2016, Intel Corporation                                **
+** Copyright (c) 2015-2017, Intel Corporation                                **
 ** All rights reserved.                                                      **
 **                                                                           **
 ** Redistribution and use in source and binary forms, with or without        **
@@ -29,7 +29,6 @@
 /* Hans Pabst (Intel Corp.)
 ******************************************************************************/
 #include <libxsmm.h>
-#include <libxsmm_timer.h>
 
 #if defined(LIBXSMM_OFFLOAD_TARGET)
 # pragma offload_attribute(push,target(LIBXSMM_OFFLOAD_TARGET))
@@ -51,31 +50,40 @@
 # pragma offload_attribute(pop)
 #endif
 
-#define MAX_SIZE (80 * 80)
+#if !defined(REAL_TYPE)
+# define REAL_TYPE double
+#endif
+
+#define MAX_SIZE (LIBXSMM_MAX_MNK / LIBXSMM_AVG_K)
 
 
-template<int Seed>
-struct LIBXSMM_RETARGETABLE init {
-  template<typename T> init(T *LIBXSMM_RESTRICT dst, double scale, int nrows, int ncols, int n = 0, int ld = 0) {
-    const int ldx = 0 == ld ? ncols : ld;
-    const int minval = n + Seed, addval = (nrows - 1) * ldx + (ncols - 1);
-    const int maxval = std::max(std::abs(minval), addval);
-    const double norm = 0 != maxval ? (scale / maxval) : scale;
-    for (int i = 0; i < nrows; ++i) {
-      for (int j = 0; j < ncols; ++j) {
-        const double value = static_cast<double>(i * ldx + j + minval);
-        dst[i*ldx+j] = static_cast<T>(norm * (value - 0.5 * addval));
-      }
+LIBXSMM_INLINE LIBXSMM_RETARGETABLE void init(int seed, REAL_TYPE *LIBXSMM_RESTRICT dst,
+  libxsmm_blasint nrows, libxsmm_blasint ncols, libxsmm_blasint ld, double scale)
+{
+  const double seed1 = scale * (seed + 1);
+  libxsmm_blasint i;
+#if defined(_OPENMP)
+# pragma omp parallel for private(i)
+#endif
+  for (i = 0; i < ncols; ++i) {
+    libxsmm_blasint j = 0;
+    for (; j < nrows; ++j) {
+      const libxsmm_blasint k = i * ld + j;
+      dst[k] = (REAL_TYPE)(seed1 / (k + 1));
+    }
+    for (; j < ld; ++j) {
+      const libxsmm_blasint k = i * ld + j;
+      dst[k] = (REAL_TYPE)seed;
     }
   }
-};
+}
 
 
 int main(int argc, char* argv[])
 {
   int result = EXIT_SUCCESS;
   try {
-    typedef double T;
+    typedef REAL_TYPE T;
     const int m = 1 < argc ? std::atoi(argv[1]) : 23;
     const int n = 2 < argc ? std::atoi(argv[2]) : m;
     const int k = 3 < argc ? std::atoi(argv[3]) : m;
@@ -88,7 +96,7 @@ int main(int argc, char* argv[])
 
     struct raii { // avoid std::vector (first-touch init. causes NUMA issue)
       T *a, *b, *c;
-      raii(int asize, int bsize, int csize): a(new T[asize]), b(new T[bsize]), c(new T[csize]) {}
+      raii(int asize_, int bsize_, int csize_): a(new T[asize_]), b(new T[bsize_]), c(new T[csize_]) {}
       ~raii() { delete[] a; delete[] b; delete[] c; }
     } buffer(s * asize + aspace - 1, s * bsize + aspace - 1, s * csize + aspace - 1);
     T *const a = LIBXSMM_ALIGN2(buffer.a, LIBXSMM_ALIGNMENT);
@@ -99,24 +107,23 @@ int main(int argc, char* argv[])
 #   pragma omp parallel for
 #endif
     for (int i = 0; i < s; ++i) {
-      init<42>(a + i * asize, scale, m, k, i);
-      init<24>(b + i * bsize, scale, k, n, i);
-      init<22>(c + i * csize, scale, m, n, i);
+      init(42 + i, a + i * asize, m, k, m, scale);
+      init(24 + i, b + i * bsize, k, n, k, scale);
+      init(22 + i, c + i * csize, m, n, m, scale);
     }
 
 #if defined(LIBXSMM_OFFLOAD_TARGET)
 #   pragma offload target(LIBXSMM_OFFLOAD_TARGET) in(a: length(s * asize)) in(b: length(s * bsize)) inout(c: length(s * csize))
 #endif
     {
-#if defined(MKL_ENABLE_AVX512_MIC)
-      mkl_enable_instructions(MKL_ENABLE_AVX512_MIC);
+#if defined(MKL_ENABLE_AVX512)
+      mkl_enable_instructions(MKL_ENABLE_AVX512);
 #endif
       // initialize LIBXSMM
       libxsmm_init();
 
-      fprintf(stdout, "m=%i n=%i k=%i (%s, %s) size=%i memory=%.f MB\n\n", m, n, k,
-        0 != LIBXSMM_ROW_MAJOR ? "row-major" : "column-major", 8 == sizeof(T) ? "DP" : "SP",
-        s, 1.0 * (s * (asize + bsize + csize) * sizeof(T)) / (1 << 20));
+      fprintf(stdout, "m=%i n=%i k=%i size=%i memory=%.1f MB (%s)\n\n", m, n, k, s,
+        1.0 * (s * (asize + bsize + csize) * sizeof(T)) / (1 << 20), 8 == sizeof(T) ? "DP" : "SP");
 
       { // batched
         fprintf(stdout, "Batched (A,B,C)...\n");
