@@ -1,5 +1,5 @@
 /******************************************************************************
-** Copyright (c) 2015-2016, Intel Corporation                                **
+** Copyright (c) 2015-2017, Intel Corporation                                **
 ** All rights reserved.                                                      **
 **                                                                           **
 ** Redistribution and use in source and binary forms, with or without        **
@@ -26,18 +26,21 @@
 ** NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS        **
 ** SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.              **
 ******************************************************************************/
-/* Alexander Heinecke (Intel Corp.)
+/* Alexander Heinecke (Intel Corp.), Hans Pabst (Intel Corp.)
 ******************************************************************************/
 #include "generator_common.h"
-#include <libxsmm_macros.h>
+#include <libxsmm_sync.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <stdarg.h>
 #include <string.h>
 
+#if !defined(GENERATOR_COMMON_MAX_ERROR_LENGTH)
+# define GENERATOR_COMMON_MAX_ERROR_LENGTH 511
+#endif
 
-static char libxsmm_global_error_message[512];
 
+LIBXSMM_INTERNAL_API_DEFINITION
 void libxsmm_strncpy( char*                  o_dest,
                       const char*            i_src,
                       unsigned int           i_dest_length,
@@ -52,6 +55,7 @@ void libxsmm_strncpy( char*                  o_dest,
   strcpy( o_dest, i_src );
 }
 
+LIBXSMM_INTERNAL_API_DEFINITION
 void libxsmm_append_code_as_string( libxsmm_generated_code* io_generated_code,
                                     const char*             i_code_to_append,
                                     const int               i_append_length ) {
@@ -109,6 +113,7 @@ void libxsmm_append_code_as_string( libxsmm_generated_code* io_generated_code,
   io_generated_code->buffer_size = (io_generated_code->code_size) + 1;
 }
 
+LIBXSMM_INTERNAL_API_DEFINITION
 void libxsmm_close_function( libxsmm_generated_code* io_generated_code ) {
   if ( io_generated_code->code_type == 0 ) {
     char l_new_code[512];
@@ -118,6 +123,7 @@ void libxsmm_close_function( libxsmm_generated_code* io_generated_code ) {
   }
 }
 
+LIBXSMM_INTERNAL_API_DEFINITION
 unsigned int libxsmm_check_x86_gp_reg_name_callee_save( const unsigned int i_gp_reg_number ) {
   if ( (i_gp_reg_number == LIBXSMM_X86_GP_REG_RBX) ||
        (i_gp_reg_number == LIBXSMM_X86_GP_REG_RBP) ||
@@ -131,6 +137,7 @@ unsigned int libxsmm_check_x86_gp_reg_name_callee_save( const unsigned int i_gp_
   }
 }
 
+LIBXSMM_INTERNAL_API_DEFINITION
 void libxsmm_get_x86_gp_reg_name( const unsigned int i_gp_reg_number,
                                   char*              o_gp_reg_name,
                                   const int          i_gp_reg_name_max_length ) {
@@ -189,6 +196,7 @@ void libxsmm_get_x86_gp_reg_name( const unsigned int i_gp_reg_number,
   }
 }
 
+LIBXSMM_INTERNAL_API_DEFINITION
 void libxsmm_get_x86_instr_name( const unsigned int i_instr_number,
                                  char*              o_instr_name,
                                  const int          i_instr_name_max_length ) {
@@ -221,6 +229,18 @@ void libxsmm_get_x86_instr_name( const unsigned int i_instr_number,
     case LIBXSMM_X86_INSTR_VMOVSS:
       libxsmm_strncpy(o_instr_name, "vmovss", i_instr_name_max_length, 6 );
       break;
+    case LIBXSMM_X86_INSTR_VPBROADCASTB:
+      libxsmm_strncpy(o_instr_name, "vpbroadcastb", i_instr_name_max_length, 12 );
+      break;
+    case LIBXSMM_X86_INSTR_VPBROADCASTW:
+      libxsmm_strncpy(o_instr_name, "vpbroadcastw", i_instr_name_max_length, 12 );
+      break;
+    case LIBXSMM_X86_INSTR_VPBROADCASTD:
+      libxsmm_strncpy(o_instr_name, "vpbroadcastd", i_instr_name_max_length, 12 );
+      break;
+    case LIBXSMM_X86_INSTR_VPBROADCASTQ:
+      libxsmm_strncpy(o_instr_name, "vpbroadcastq", i_instr_name_max_length, 12 );
+      break;
     /* SSE vector moves */
     case LIBXSMM_X86_INSTR_MOVAPD:
       libxsmm_strncpy(o_instr_name, "movapd", i_instr_name_max_length, 6 );
@@ -277,6 +297,32 @@ void libxsmm_get_x86_instr_name( const unsigned int i_instr_number,
     case LIBXSMM_X86_INSTR_VPREFETCH0:
       libxsmm_strncpy(o_instr_name, "vprefetch0", i_instr_name_max_length, 10 );
       break;
+    /* Gather/scatter single precision */
+    case LIBXSMM_X86_INSTR_VGATHERDPS:
+      libxsmm_strncpy(o_instr_name, "vgatherdps", i_instr_name_max_length, 10 );
+      break;
+    case LIBXSMM_X86_INSTR_VGATHERQPS:
+      libxsmm_strncpy(o_instr_name, "vgatherqps", i_instr_name_max_length, 10 );
+      break;
+    case LIBXSMM_X86_INSTR_VSCATTERDPS:
+      libxsmm_strncpy(o_instr_name, "vscatterdps", i_instr_name_max_length, 11 );
+      break;
+    case LIBXSMM_X86_INSTR_VSCATTERQPS:
+      libxsmm_strncpy(o_instr_name, "vscatterqps", i_instr_name_max_length, 11 );
+      break;
+    /* Gather/scatter double precision */
+    case LIBXSMM_X86_INSTR_VGATHERDPD:
+      libxsmm_strncpy(o_instr_name, "vgatherdpd", i_instr_name_max_length, 10 );
+      break;
+    case LIBXSMM_X86_INSTR_VGATHERQPD:
+      libxsmm_strncpy(o_instr_name, "vgatherqpd", i_instr_name_max_length, 10 );
+      break;
+    case LIBXSMM_X86_INSTR_VSCATTERDPD:
+      libxsmm_strncpy(o_instr_name, "vscatterdpd", i_instr_name_max_length, 11 );
+      break;
+    case LIBXSMM_X86_INSTR_VSCATTERQPD:
+      libxsmm_strncpy(o_instr_name, "vscatterqpd", i_instr_name_max_length, 11 );
+      break;
     /* AVX double precision */
     case LIBXSMM_X86_INSTR_VXORPD:
       libxsmm_strncpy(o_instr_name, "vxorpd", i_instr_name_max_length, 6 );
@@ -357,6 +403,24 @@ void libxsmm_get_x86_instr_name( const unsigned int i_instr_number,
     case LIBXSMM_X86_INSTR_VPXORD:
       libxsmm_strncpy(o_instr_name, "vpxord", i_instr_name_max_length, 6 );
       break;
+    case LIBXSMM_X86_INSTR_VPADDB:
+      libxsmm_strncpy(o_instr_name, "vpaddb", i_instr_name_max_length, 6 );
+      break;
+    case LIBXSMM_X86_INSTR_VPADDW:
+      libxsmm_strncpy(o_instr_name, "vpaddw", i_instr_name_max_length, 6 );
+      break;
+    case LIBXSMM_X86_INSTR_VPADDD:
+      libxsmm_strncpy(o_instr_name, "vpaddd", i_instr_name_max_length, 6 );
+      break;
+    case LIBXSMM_X86_INSTR_VPADDQ:
+      libxsmm_strncpy(o_instr_name, "vpaddq", i_instr_name_max_length, 6 );
+      break;
+    case LIBXSMM_X86_INSTR_VPMADDWD:
+      libxsmm_strncpy(o_instr_name, "vpmaddwd", i_instr_name_max_length, 8 );
+      break;
+    case LIBXSMM_X86_INSTR_VPMADDUBSW:
+      libxsmm_strncpy(o_instr_name, "vpmaddubsw", i_instr_name_max_length, 10 );
+      break;
     /* GP instructions */
     case LIBXSMM_X86_INSTR_ADDQ:
       libxsmm_strncpy(o_instr_name, "addq", i_instr_name_max_length, 4 );
@@ -391,6 +455,9 @@ void libxsmm_get_x86_instr_name( const unsigned int i_instr_number,
     case LIBXSMM_X86_INSTR_KMOVW:
       libxsmm_strncpy(o_instr_name, "kmovw", i_instr_name_max_length, 5 );
       break;
+    case LIBXSMM_X86_INSTR_KXNORW:
+      libxsmm_strncpy(o_instr_name, "kxnorw", i_instr_name_max_length, 6 );
+      break;
     /* default, we didn't had a match */
     default:
       fprintf(stderr, " LIBXSMM ERROR: libxsmm_get_x86_64_instr_name i_instr_number (%u) is out of range!\n", i_instr_number);
@@ -398,6 +465,7 @@ void libxsmm_get_x86_instr_name( const unsigned int i_instr_number,
   }
 }
 
+LIBXSMM_INTERNAL_API_DEFINITION
 unsigned int libxsmm_is_x86_vec_instr_single_precision( const unsigned int i_instr_number ) {
   unsigned int l_return = 0;
 
@@ -479,6 +547,32 @@ unsigned int libxsmm_is_x86_vec_instr_single_precision( const unsigned int i_ins
     case LIBXSMM_X86_INSTR_VPACKSTOREHPS:
       l_return = 1;
       break;
+    /* Gather/Scatter single precision */
+    case LIBXSMM_X86_INSTR_VGATHERDPS:
+      l_return = 1;
+      break;
+    case LIBXSMM_X86_INSTR_VGATHERQPS:
+      l_return = 1;
+      break;
+    case LIBXSMM_X86_INSTR_VSCATTERDPS:
+      l_return = 1;
+      break;
+    case LIBXSMM_X86_INSTR_VSCATTERQPS:
+      l_return = 1;
+      break;
+    /* Gather/Scatter double precision */
+    case LIBXSMM_X86_INSTR_VGATHERDPD:
+      l_return = 0;
+      break;
+    case LIBXSMM_X86_INSTR_VGATHERQPD:
+      l_return = 0;
+      break;
+    case LIBXSMM_X86_INSTR_VSCATTERDPD:
+      l_return = 0;
+      break;
+    case LIBXSMM_X86_INSTR_VSCATTERQPD:
+      l_return = 0;
+      break;
     /* AVX double precision */
     case LIBXSMM_X86_INSTR_VXORPD:
       l_return = 0;
@@ -564,6 +658,7 @@ unsigned int libxsmm_is_x86_vec_instr_single_precision( const unsigned int i_ins
   return l_return;
 }
 
+LIBXSMM_INTERNAL_API_DEFINITION
 void libxsmm_reset_x86_gp_reg_mapping( libxsmm_gp_reg_mapping* io_gp_reg_mapping ) {
   io_gp_reg_mapping->gp_reg_a = LIBXSMM_X86_GP_REG_UNDEF;
   io_gp_reg_mapping->gp_reg_b = LIBXSMM_X86_GP_REG_UNDEF;
@@ -581,6 +676,7 @@ void libxsmm_reset_x86_gp_reg_mapping( libxsmm_gp_reg_mapping* io_gp_reg_mapping
   io_gp_reg_mapping->gp_reg_help_5 = LIBXSMM_X86_GP_REG_UNDEF;
 }
 
+LIBXSMM_INTERNAL_API_DEFINITION
 void libxsmm_reset_loop_label_tracker( libxsmm_loop_label_tracker* io_loop_label_tracker ) {
   unsigned int l_i;
   for ( l_i = 0; l_i < 32; l_i++) {
@@ -589,6 +685,7 @@ void libxsmm_reset_loop_label_tracker( libxsmm_loop_label_tracker* io_loop_label
   io_loop_label_tracker->label_count = 0;
 }
 
+LIBXSMM_INTERNAL_API_DEFINITION
 void libxsmm_mmfunction_signature( libxsmm_generated_code*         io_generated_code,
                                   const char*                     i_routine_name,
                                   const libxsmm_gemm_descriptor* i_xgemm_desc ) {
@@ -620,6 +717,7 @@ void libxsmm_mmfunction_signature( libxsmm_generated_code*         io_generated_
   libxsmm_append_code_as_string( io_generated_code, l_new_code, l_code_length );
 }
 
+LIBXSMM_INTERNAL_API_DEFINITION
 void libxsmm_generator_isa_check_header( libxsmm_generated_code* io_generated_code,
                                          const char*             i_arch ) {
   if ( io_generated_code->code_type == 0 ) {
@@ -671,6 +769,7 @@ void libxsmm_generator_isa_check_header( libxsmm_generated_code* io_generated_co
   }
 }
 
+LIBXSMM_INTERNAL_API_DEFINITION
 void libxsmm_generator_isa_check_footer( libxsmm_generated_code* io_generated_code,
                                          const char*             i_arch ) {
   if ( io_generated_code->code_type == 0 ) {
@@ -701,6 +800,7 @@ void libxsmm_generator_isa_check_footer( libxsmm_generated_code* io_generated_co
   }
 }
 
+LIBXSMM_INTERNAL_API_DEFINITION
 void libxsmm_handle_error( libxsmm_generated_code* io_generated_code,
                            const unsigned int      i_error_code ) {
   io_generated_code->last_error = i_error_code;
@@ -709,125 +809,202 @@ void libxsmm_handle_error( libxsmm_generated_code* io_generated_code,
 #endif
 }
 
+LIBXSMM_INTERNAL_API_DEFINITION
 const char* libxsmm_strerror(unsigned int i_error_code) {
-  int l_max_error_length = 511;
+  static LIBXSMM_TLS char error_message[GENERATOR_COMMON_MAX_ERROR_LENGTH+1];
 
   switch (i_error_code) {
     case LIBXSMM_ERR_GENERAL:
-      LIBXSMM_SNPRINTF( libxsmm_global_error_message, l_max_error_length, " LIBXSMM ERROR: a general error occured!\n" );
+      LIBXSMM_SNPRINTF( error_message, GENERATOR_COMMON_MAX_ERROR_LENGTH, " LIBXSMM ERROR: a general error occured!\n" );
       break;
     case LIBXSMM_ERR_ARCH_PREC:
-      LIBXSMM_SNPRINTF( libxsmm_global_error_message, l_max_error_length, " LIBXSMM ERROR: unknown architecture and precision!\n" );
+      LIBXSMM_SNPRINTF( error_message, GENERATOR_COMMON_MAX_ERROR_LENGTH, " LIBXSMM ERROR: unknown architecture and precision!\n" );
       break;
     case LIBXSMM_ERR_ARCH:
-      LIBXSMM_SNPRINTF( libxsmm_global_error_message, l_max_error_length, " LIBXSMM ERROR: unknown architecture!\n" );
+      LIBXSMM_SNPRINTF( error_message, GENERATOR_COMMON_MAX_ERROR_LENGTH, " LIBXSMM ERROR: unknown architecture!\n" );
       break;
     case LIBXSMM_ERR_LDA:
-      LIBXSMM_SNPRINTF( libxsmm_global_error_message, l_max_error_length, " LIBXSMM ERROR: lda needs to be greater than or equal to m!\n" );
+      LIBXSMM_SNPRINTF( error_message, GENERATOR_COMMON_MAX_ERROR_LENGTH, " LIBXSMM ERROR: lda needs to be greater than or equal to m!\n" );
       break;
     case LIBXSMM_ERR_LDB:
-      LIBXSMM_SNPRINTF( libxsmm_global_error_message, l_max_error_length, " LIBXSMM ERROR: ldb needs to be greater than or equal to k!\n" );
+      LIBXSMM_SNPRINTF( error_message, GENERATOR_COMMON_MAX_ERROR_LENGTH, " LIBXSMM ERROR: ldb needs to be greater than or equal to k!\n" );
       break;
     case LIBXSMM_ERR_LDC:
-      LIBXSMM_SNPRINTF( libxsmm_global_error_message, l_max_error_length, " LIBXSMM ERROR: ldc needs to be greater than or equal to m!\n" );
+      LIBXSMM_SNPRINTF( error_message, GENERATOR_COMMON_MAX_ERROR_LENGTH, " LIBXSMM ERROR: ldc needs to be greater than or equal to m!\n" );
       break;
     case LIBXSMM_ERR_SPGEMM_GEN:
-      LIBXSMM_SNPRINTF( libxsmm_global_error_message, l_max_error_length, " LIBXSMM ERROR: could not determine which sparse code generation variant is requested!\n" );
+      LIBXSMM_SNPRINTF( error_message, GENERATOR_COMMON_MAX_ERROR_LENGTH, " LIBXSMM ERROR: could not determine which sparse code generation variant is requested!\n" );
       break;
     case LIBXSMM_ERR_CSC_INPUT:
-      LIBXSMM_SNPRINTF( libxsmm_global_error_message, l_max_error_length, " LIBXSMM ERROR: could not open the specified CSC input file!\n" );
+      LIBXSMM_SNPRINTF( error_message, GENERATOR_COMMON_MAX_ERROR_LENGTH, " LIBXSMM ERROR: could not open the specified CSC input file!\n" );
       break;
     case LIBXSMM_ERR_CSC_READ_LEN:
-      LIBXSMM_SNPRINTF( libxsmm_global_error_message, l_max_error_length, " LIBXSMM ERROR: exceeded predefined line-length when reading line of CSC file!\n" );
+      LIBXSMM_SNPRINTF( error_message, GENERATOR_COMMON_MAX_ERROR_LENGTH, " LIBXSMM ERROR: exceeded predefined line-length when reading line of CSC file!\n" );
       break;
     case LIBXSMM_ERR_CSC_READ_DESC:
-      LIBXSMM_SNPRINTF( libxsmm_global_error_message, l_max_error_length, " LIBXSMM ERROR: error when reading descriptor of CSC file!\n" );
+      LIBXSMM_SNPRINTF( error_message, GENERATOR_COMMON_MAX_ERROR_LENGTH, " LIBXSMM ERROR: error when reading descriptor of CSC file!\n" );
       break;
     case LIBXSMM_ERR_CSC_READ_ELEMS:
-      LIBXSMM_SNPRINTF( libxsmm_global_error_message, l_max_error_length, " LIBXSMM ERROR: error when reading line of CSC file!\n" );
+      LIBXSMM_SNPRINTF( error_message, GENERATOR_COMMON_MAX_ERROR_LENGTH, " LIBXSMM ERROR: error when reading line of CSC file!\n" );
       break;
     case LIBXSMM_ERR_CSC_LEN:
-      LIBXSMM_SNPRINTF( libxsmm_global_error_message, l_max_error_length, " LIBXSMM ERROR: number of elements read differs from number of elements specified in CSC file!\n" );
+      LIBXSMM_SNPRINTF( error_message, GENERATOR_COMMON_MAX_ERROR_LENGTH, " LIBXSMM ERROR: number of elements read differs from number of elements specified in CSC file!\n" );
       break;
     case LIBXSMM_ERR_N_BLOCK:
-      LIBXSMM_SNPRINTF( libxsmm_global_error_message, l_max_error_length, " LIBXSMM ERROR: invalid N blocking in microkernel!\n" );
+      LIBXSMM_SNPRINTF( error_message, GENERATOR_COMMON_MAX_ERROR_LENGTH, " LIBXSMM ERROR: invalid N blocking in microkernel!\n" );
       break;
     case LIBXSMM_ERR_M_BLOCK:
-      LIBXSMM_SNPRINTF( libxsmm_global_error_message, l_max_error_length, " LIBXSMM ERROR: invalid M blocking in microkernel!\n" );
+      LIBXSMM_SNPRINTF( error_message, GENERATOR_COMMON_MAX_ERROR_LENGTH, " LIBXSMM ERROR: invalid M blocking in microkernel!\n" );
+      break;
+    case LIBXSMM_ERR_K_BLOCK:
+      LIBXSMM_SNPRINTF( error_message, GENERATOR_COMMON_MAX_ERROR_LENGTH, " LIBXSMM ERROR: invalid K blocking in microkernel!\n" );
       break;
     case LIBXSMM_ERR_NO_IMCI:
-      LIBXSMM_SNPRINTF( libxsmm_global_error_message, l_max_error_length, " LIBXSMM ERROR: IMCI architecture requested but called for a different one!\n" );
+      LIBXSMM_SNPRINTF( error_message, GENERATOR_COMMON_MAX_ERROR_LENGTH, " LIBXSMM ERROR: IMCI architecture requested but called for a different one!\n" );
       break;
     case LIBXSMM_ERR_REG_BLOCK:
-      LIBXSMM_SNPRINTF( libxsmm_global_error_message, l_max_error_length, " LIBXSMM ERROR: invalid MxN register blocking was specified!\n" );
+      LIBXSMM_SNPRINTF( error_message, GENERATOR_COMMON_MAX_ERROR_LENGTH, " LIBXSMM ERROR: invalid MxN register blocking was specified!\n" );
       break;
     case LIBXSMM_ERR_VEC_MOVE_IMCI:
-      LIBXSMM_SNPRINTF( libxsmm_global_error_message, l_max_error_length, " LIBXSMM ERROR: invalid vec move instruction for IMCI instruction replacement!\n" );
+      LIBXSMM_SNPRINTF( error_message, GENERATOR_COMMON_MAX_ERROR_LENGTH, " LIBXSMM ERROR: invalid vec move instruction for IMCI instruction replacement!\n" );
       break;
     case LIBXSMM_ERR_APPEND_STR:
-      LIBXSMM_SNPRINTF( libxsmm_global_error_message, l_max_error_length, " LIBXSMM ERROR: append code as string was called for generation mode which does not support this!\n" );
+      LIBXSMM_SNPRINTF( error_message, GENERATOR_COMMON_MAX_ERROR_LENGTH, " LIBXSMM ERROR: append code as string was called for generation mode which does not support this!\n" );
       break;
     case LIBXSMM_ERR_ALLOC:
-      LIBXSMM_SNPRINTF( libxsmm_global_error_message, l_max_error_length, " LIBXSMM ERROR: memory allocation failed!\n" );
+      LIBXSMM_SNPRINTF( error_message, GENERATOR_COMMON_MAX_ERROR_LENGTH, " LIBXSMM ERROR: memory allocation failed!\n" );
       break;
     case LIBXSMM_ERR_NO_IMCI_AVX512_BCAST:
-      LIBXSMM_SNPRINTF( libxsmm_global_error_message, l_max_error_length, " LIBXSMM ERROR: fused memory broadcast is not supported on other platforms than AVX512/IMCI!\n" );
+      LIBXSMM_SNPRINTF( error_message, GENERATOR_COMMON_MAX_ERROR_LENGTH, " LIBXSMM ERROR: fused memory broadcast is not supported on other platforms than AVX512/IMCI!\n" );
       break;
     case LIBXSMM_ERR_CALLEE_SAVE_A:
-      LIBXSMM_SNPRINTF( libxsmm_global_error_message, l_max_error_length, " LIBXSMM ERROR: reg_a cannot be callee save, since input, please use either rdi, rsi, rdx, rcx, r8, r9 for this value!\n" );
+      LIBXSMM_SNPRINTF( error_message, GENERATOR_COMMON_MAX_ERROR_LENGTH, " LIBXSMM ERROR: reg_a cannot be callee save, since input, please use either rdi, rsi, rdx, rcx, r8, r9 for this value!\n" );
       break;
     case LIBXSMM_ERR_CALLEE_SAVE_B:
-      LIBXSMM_SNPRINTF( libxsmm_global_error_message, l_max_error_length, " LIBXSMM ERROR: reg_b cannot be callee save, since input, please use either rdi, rsi, rdx, rcx, r8, r9 for this value!\n" );
+      LIBXSMM_SNPRINTF( error_message, GENERATOR_COMMON_MAX_ERROR_LENGTH, " LIBXSMM ERROR: reg_b cannot be callee save, since input, please use either rdi, rsi, rdx, rcx, r8, r9 for this value!\n" );
       break;
     case LIBXSMM_ERR_CALLEE_SAVE_C:
-      LIBXSMM_SNPRINTF( libxsmm_global_error_message, l_max_error_length, " LIBXSMM ERROR: reg_c cannot be callee save, since input, please use either rdi, rsi, rdx, rcx, r8, r9 for this value!\n" );
+      LIBXSMM_SNPRINTF( error_message, GENERATOR_COMMON_MAX_ERROR_LENGTH, " LIBXSMM ERROR: reg_c cannot be callee save, since input, please use either rdi, rsi, rdx, rcx, r8, r9 for this value!\n" );
       break;
     case LIBXSMM_ERR_CALLEE_SAVE_A_PREF:
-      LIBXSMM_SNPRINTF( libxsmm_global_error_message, l_max_error_length, " LIBXSMM ERROR: reg_a_prefetch cannot be callee save, since input, please use either rdi, rsi, rdx, rcx, r8, r9 for this value!\n" );
+      LIBXSMM_SNPRINTF( error_message, GENERATOR_COMMON_MAX_ERROR_LENGTH, " LIBXSMM ERROR: reg_a_prefetch cannot be callee save, since input, please use either rdi, rsi, rdx, rcx, r8, r9 for this value!\n" );
       break;
     case LIBXSMM_ERR_CALLEE_SAVE_B_PREF:
-      LIBXSMM_SNPRINTF( libxsmm_global_error_message, l_max_error_length, " LIBXSMM ERROR: reg_b_prefetch cannot be callee save, since input, please use either rdi, rsi, rdx, rcx, r8, r9 for this value!\n" );
+      LIBXSMM_SNPRINTF( error_message, GENERATOR_COMMON_MAX_ERROR_LENGTH, " LIBXSMM ERROR: reg_b_prefetch cannot be callee save, since input, please use either rdi, rsi, rdx, rcx, r8, r9 for this value!\n" );
       break;
     case LIBXSMM_ERR_NO_INDEX_SCALE_ADDR:
-      LIBXSMM_SNPRINTF( libxsmm_global_error_message, l_max_error_length, " LIBXSMM ERROR: Index + Scale addressing mode is currently not implemented!\n");
+      LIBXSMM_SNPRINTF( error_message, GENERATOR_COMMON_MAX_ERROR_LENGTH, " LIBXSMM ERROR: Index + Scale addressing mode is currently not implemented!\n");
       break;
     case LIBXSMM_ERR_UNSUPPORTED_JUMP:
-      LIBXSMM_SNPRINTF( libxsmm_global_error_message, l_max_error_length, " LIBXSMM ERROR: Unsupported jump instruction requested!\n");
+      LIBXSMM_SNPRINTF( error_message, GENERATOR_COMMON_MAX_ERROR_LENGTH, " LIBXSMM ERROR: Unsupported jump instruction requested!\n");
       break;
     case LIBXSMM_ERR_NO_JMPLBL_AVAIL:
-      LIBXSMM_SNPRINTF( libxsmm_global_error_message, l_max_error_length, " LIBXSMM ERROR: No destination jump label is available!\n");
+      LIBXSMM_SNPRINTF( error_message, GENERATOR_COMMON_MAX_ERROR_LENGTH, " LIBXSMM ERROR: No destination jump label is available!\n");
       break;
     case LIBXSMM_ERR_EXCEED_JMPLBL:
-      LIBXSMM_SNPRINTF( libxsmm_global_error_message, l_max_error_length, " LIBXSMM ERROR: too many nested loops, exceeding loop label tracker!\n");
+      LIBXSMM_SNPRINTF( error_message, GENERATOR_COMMON_MAX_ERROR_LENGTH, " LIBXSMM ERROR: too many nested loops, exceeding loop label tracker!\n");
       break;
     case LIBXSMM_ERR_CSC_ALLOC_DATA:
-      LIBXSMM_SNPRINTF( libxsmm_global_error_message, l_max_error_length, " LIBXSMM ERROR: could not allocate temporay memory for reading CSC file!\n");
+      LIBXSMM_SNPRINTF( error_message, GENERATOR_COMMON_MAX_ERROR_LENGTH, " LIBXSMM ERROR: could not allocate temporay memory for reading CSC file!\n");
       break;
     case LIBXSMM_ERR_CSR_ALLOC_DATA:
-      LIBXSMM_SNPRINTF( libxsmm_global_error_message, l_max_error_length, " LIBXSMM ERROR: could not allocate temporay memory for reading CSR file!\n");
+      LIBXSMM_SNPRINTF( error_message, GENERATOR_COMMON_MAX_ERROR_LENGTH, " LIBXSMM ERROR: could not allocate temporay memory for reading CSR file!\n");
       break;
     case LIBXSMM_ERR_CSR_INPUT:
-      LIBXSMM_SNPRINTF( libxsmm_global_error_message, l_max_error_length, " LIBXSMM ERROR: could not open the specified CSR input file!\n" );
+      LIBXSMM_SNPRINTF( error_message, GENERATOR_COMMON_MAX_ERROR_LENGTH, " LIBXSMM ERROR: could not open the specified CSR input file!\n" );
       break;
     case LIBXSMM_ERR_CSR_READ_LEN:
-      LIBXSMM_SNPRINTF( libxsmm_global_error_message, l_max_error_length, " LIBXSMM ERROR: exceeded predefined line-length when reading line of CSR file!\n" );
+      LIBXSMM_SNPRINTF( error_message, GENERATOR_COMMON_MAX_ERROR_LENGTH, " LIBXSMM ERROR: exceeded predefined line-length when reading line of CSR file!\n" );
       break;
     case LIBXSMM_ERR_CSR_READ_DESC:
-      LIBXSMM_SNPRINTF( libxsmm_global_error_message, l_max_error_length, " LIBXSMM ERROR: error when reading descriptor of CSR file!\n" );
+      LIBXSMM_SNPRINTF( error_message, GENERATOR_COMMON_MAX_ERROR_LENGTH, " LIBXSMM ERROR: error when reading descriptor of CSR file!\n" );
       break;
     case LIBXSMM_ERR_CSR_READ_ELEMS:
-      LIBXSMM_SNPRINTF( libxsmm_global_error_message, l_max_error_length, " LIBXSMM ERROR: error when reading line of CSR file!\n" );
+      LIBXSMM_SNPRINTF( error_message, GENERATOR_COMMON_MAX_ERROR_LENGTH, " LIBXSMM ERROR: error when reading line of CSR file!\n" );
       break;
     case LIBXSMM_ERR_CSR_LEN:
-      LIBXSMM_SNPRINTF( libxsmm_global_error_message, l_max_error_length, " LIBXSMM ERROR: number of elements read differs from number of elements specified in CSR file!\n" );
+      LIBXSMM_SNPRINTF( error_message, GENERATOR_COMMON_MAX_ERROR_LENGTH, " LIBXSMM ERROR: number of elements read differs from number of elements specified in CSR file!\n" );
+      break;
+    case LIBXSMM_ERR_UNSUP_CONV_FORMAT:
+      LIBXSMM_SNPRINTF( error_message, GENERATOR_COMMON_MAX_ERROR_LENGTH, " LIBXSMM ERROR: invalid activation or filter format was detected during convolution kernel generation!\n" );
+      break;
+    case LIBXSMM_ERR_INVALID_KW_UNROLL:
+      LIBXSMM_SNPRINTF( error_message, GENERATOR_COMMON_MAX_ERROR_LENGTH, " LIBXSMM ERROR: invalid kw unrull was detected during convolution kernel generation!\n" );
+      break;
+    case LIBXSMM_ERR_INVALID_KH_UNROLL:
+      LIBXSMM_SNPRINTF( error_message, GENERATOR_COMMON_MAX_ERROR_LENGTH, " LIBXSMM ERROR: invalid kh unrull was detected during convolution kernel generation!\n" );
+      break;
+    case LIBXSMM_ERR_INVALID_OFW_UNROLL:
+      LIBXSMM_SNPRINTF( error_message, GENERATOR_COMMON_MAX_ERROR_LENGTH, " LIBXSMM ERROR: invalid ofw unrull was detected during convolution kernel generation!\n" );
+      break;
+    case LIBXSMM_ERR_INVALID_OFH_UNROLL:
+      LIBXSMM_SNPRINTF( error_message, GENERATOR_COMMON_MAX_ERROR_LENGTH, " LIBXSMM ERROR: invalid ofh unrull was detected during convolution kernel generation!\n" );
+      break;
+    case LIBXSMM_ERR_INVALID_CONV_ACC:
+      LIBXSMM_SNPRINTF( error_message, GENERATOR_COMMON_MAX_ERROR_LENGTH, " LIBXSMM ERROR: invalid size of accumulator was detected during convolution kernel generation!\n" );
+      break;
+    case LIBXSMM_ERR_UNSUP_ARCH:
+      LIBXSMM_SNPRINTF( error_message, GENERATOR_COMMON_MAX_ERROR_LENGTH, " LIBXSMM ERROR: unsupported arch for the selected module was specified!\n" );
+      break;
+    case LIBXSMM_ERR_CONV_OFM_VEC:
+      LIBXSMM_SNPRINTF( error_message, GENERATOR_COMMON_MAX_ERROR_LENGTH, " LIBXSMM ERROR: forward conv/weight updated vectorization failed, OFM blocking is not divisible by VLEN!\n" );
+      break;
+    case LIBXSMM_ERR_CONV_IFM_VEC:
+      LIBXSMM_SNPRINTF( error_message, GENERATOR_COMMON_MAX_ERROR_LENGTH, " LIBXSMM ERROR: backward conv vectorization failed, IFM blocking is not divisible by VLEN!\n" );
+      break;
+    case LIBXSMM_ERR_CONV_CONT_STRIDE:
+      LIBXSMM_SNPRINTF( error_message, GENERATOR_COMMON_MAX_ERROR_LENGTH, " LIBXSMM ERROR: backward conv vectorization failed, stride_h/w need to be 1!\n" );
+      break;
+    case LIBXSMM_ERR_UNSUP_DATATYPE:
+      LIBXSMM_SNPRINTF( error_message, GENERATOR_COMMON_MAX_ERROR_LENGTH, " LIBXSMM ERROR: unsupported datatype was requested!\n" );
+      break;
+    case LIBXSMM_ERR_UNSUP_DT_FORMAT:
+      LIBXSMM_SNPRINTF( error_message, GENERATOR_COMMON_MAX_ERROR_LENGTH, " LIBXSMM ERROR: unsupported datatype and format combination was requested!\n" );
+      break;
+    case LIBXSMM_ERR_INVALID_GEMM_CONFIG:
+      LIBXSMM_SNPRINTF( error_message, GENERATOR_COMMON_MAX_ERROR_LENGTH, " LIBXSMM ERROR: invalid GEMM config in setup detected!\n" );
       break;
     /* default, we didn't don't know what happend */
     default:
-      LIBXSMM_SNPRINTF( libxsmm_global_error_message, l_max_error_length, " LIBXSMM ERROR: an unknown error occured!\n" );
+      LIBXSMM_SNPRINTF( error_message, GENERATOR_COMMON_MAX_ERROR_LENGTH, " LIBXSMM ERROR: an unknown error occured!\n" );
       break;
   }
 
-  return libxsmm_global_error_message;
+  return error_message;
 }
 
+LIBXSMM_INTERNAL_API_DEFINITION
+void libxsmm_convfunction_signature_fp32( libxsmm_generated_code*         io_generated_code,
+                                          const char*                     i_routine_name     ) {
+  char l_new_code[512];
+  int l_max_code_length = 511;
+  int l_code_length = 0;
 
+  if ( io_generated_code->code_type > 1 ) {
+    return;
+  } else if ( io_generated_code->code_type == 1 ) {
+    l_code_length = LIBXSMM_SNPRINTF(l_new_code, l_max_code_length, ".global %s\n.type %s, @function\n%s:\n", i_routine_name, i_routine_name, i_routine_name);
+  } else {
+    l_code_length = LIBXSMM_SNPRINTF(l_new_code, l_max_code_length, "void %s(const float* inputptr, const float* weightptr, float* outputptr, const float* inputpfptr, const float* weightpfptr, const float* outputpfptr) {\n", i_routine_name);
+  }
+
+  libxsmm_append_code_as_string( io_generated_code, l_new_code, l_code_length );
+}
+
+LIBXSMM_INTERNAL_API_DEFINITION
+void libxsmm_convfunction_signature_int16( libxsmm_generated_code*         io_generated_code,
+                                           const char*                     i_routine_name     ) {
+  char l_new_code[512];
+  int l_max_code_length = 511;
+  int l_code_length = 0;
+
+  if ( io_generated_code->code_type > 1 ) {
+    return;
+  } else if ( io_generated_code->code_type == 1 ) {
+    l_code_length = LIBXSMM_SNPRINTF(l_new_code, l_max_code_length, ".global %s\n.type %s, @function\n%s:\n", i_routine_name, i_routine_name, i_routine_name);
+  } else {
+    l_code_length = LIBXSMM_SNPRINTF(l_new_code, l_max_code_length, "void %s(const short* inputptr, const short* weightptr, int* outputptr, const short* inputpfptr, const short* weightpfptr, const int* outputpfptr) {\n", i_routine_name);
+  }
+
+  libxsmm_append_code_as_string( io_generated_code, l_new_code, l_code_length );
+}
