@@ -1,5 +1,5 @@
 /******************************************************************************
-** Copyright (c) 2015-2016, Intel Corporation                                **
+** Copyright (c) 2015-2017, Intel Corporation                                **
 ** All rights reserved.                                                      **
 **                                                                           **
 ** Redistribution and use in source and binary forms, with or without        **
@@ -79,6 +79,17 @@
 #define LIBXSMM_X86_INSTR_VMOVDDUP       10006
 #define LIBXSMM_X86_INSTR_VMOVSD         10007
 #define LIBXSMM_X86_INSTR_VMOVSS         10008
+#define LIBXSMM_X86_INSTR_VPBROADCASTB   10025
+#define LIBXSMM_X86_INSTR_VPBROADCASTW   10026
+#define LIBXSMM_X86_INSTR_VPBROADCASTD   10027
+#define LIBXSMM_X86_INSTR_VPBROADCASTQ   10028
+#define LIBXSMM_X86_INSTR_VMOVDQA32      10029
+#define LIBXSMM_X86_INSTR_VMOVDQA64      10030
+#define LIBXSMM_X86_INSTR_VMOVDQU8       10031
+#define LIBXSMM_X86_INSTR_VMOVDQU16      10032
+#define LIBXSMM_X86_INSTR_VMOVDQU32      10033
+#define LIBXSMM_X86_INSTR_VMOVDQU64      10034
+
 /* SSE */
 #define LIBXSMM_X86_INSTR_MOVAPD         10009
 #define LIBXSMM_X86_INSTR_MOVUPD         10010
@@ -98,6 +109,16 @@
 #define LIBXSMM_X86_INSTR_VPACKSTORELPS  10023
 #define LIBXSMM_X86_INSTR_VPACKSTOREHPS  10024
 
+/* Gather/Scatter instructions */
+#define LIBXSMM_X86_INSTR_VGATHERDPS     11000
+#define LIBXSMM_X86_INSTR_VGATHERDPD     11001
+#define LIBXSMM_X86_INSTR_VGATHERQPS     11002
+#define LIBXSMM_X86_INSTR_VGATHERQPD     11003
+#define LIBXSMM_X86_INSTR_VSCATTERDPS    11004
+#define LIBXSMM_X86_INSTR_VSCATTERDPD    11005
+#define LIBXSMM_X86_INSTR_VSCATTERQPS    11006
+#define LIBXSMM_X86_INSTR_VSCATTERQPD    11007
+
 /* Vector compute instructions */
 /* AVX1,AVX2,AVX512 */
 #define LIBXSMM_X86_INSTR_VXORPD         20000
@@ -147,6 +168,15 @@
 #define LIBXSMM_X86_INSTR_SUBSS          20043
 /* AVX512, IMCI Integer XOR as there is no FP */
 #define LIBXSMM_X86_INSTR_VPXORD         20044
+/* additional integer stuff */
+#define LIBXSMM_X86_INSTR_VPADDQ         20045
+#define LIBXSMM_X86_INSTR_VPADDD         20046
+#define LIBXSMM_X86_INSTR_VPADDW         20047
+#define LIBXSMM_X86_INSTR_VPADDB         20048
+#define LIBXSMM_X86_INSTR_VPMADDWD       20049
+#define LIBXSMM_X86_INSTR_VPMADDUBSW     20050
+#define LIBXSMM_X86_INSTR_VPADDSW        20051
+#define LIBXSMM_X86_INSTR_VPADDSB        20052
 
 /* GP instructions */
 #define LIBXSMM_X86_INSTR_ADDQ           30000
@@ -165,6 +195,9 @@
 #define LIBXSMM_X86_INSTR_KMOV           40000
 #define LIBXSMM_X86_INSTR_KMOVW          40001
 
+/* Mask compute instructions */
+#define LIBXSMM_X86_INSTR_KXNORW         45000
+
 /* define error codes */
 #define LIBXSMM_ERR_GENERAL              90000
 #define LIBXSMM_ERR_ARCH_PREC            90001
@@ -202,6 +235,20 @@
 #define LIBXSMM_ERR_CSR_READ_ELEMS       90033
 #define LIBXSMM_ERR_CSR_LEN              90034
 #define LIBXSMM_ERR_CSR_ALLOC_DATA       90035
+#define LIBXSMM_ERR_UNSUP_CONV_FORMAT    90036
+#define LIBXSMM_ERR_INVALID_KW_UNROLL    90037
+#define LIBXSMM_ERR_INVALID_KH_UNROLL    90038
+#define LIBXSMM_ERR_INVALID_OFW_UNROLL   90039
+#define LIBXSMM_ERR_INVALID_OFH_UNROLL   90040
+#define LIBXSMM_ERR_INVALID_CONV_ACC     90041
+#define LIBXSMM_ERR_UNSUP_ARCH           90042
+#define LIBXSMM_ERR_CONV_IFM_VEC         90043
+#define LIBXSMM_ERR_CONV_OFM_VEC         90044
+#define LIBXSMM_ERR_CONV_CONT_STRIDE     90045
+#define LIBXSMM_ERR_UNSUP_DATATYPE       90046
+#define LIBXSMM_ERR_UNSUP_DT_FORMAT      90047
+#define LIBXSMM_ERR_K_BLOCK              90048
+#define LIBXSMM_ERR_INVALID_GEMM_CONFIG  90049
 
 /* micro kernel config */
 typedef struct libxsmm_micro_kernel_config_struct {
@@ -233,6 +280,7 @@ typedef struct libxsmm_gp_reg_mapping_struct {
   unsigned int gp_reg_c;
   unsigned int gp_reg_a_prefetch;
   unsigned int gp_reg_b_prefetch;
+  unsigned int gp_reg_c_prefetch;
   unsigned int gp_reg_mloop;
   unsigned int gp_reg_nloop;
   unsigned int gp_reg_kloop;
@@ -244,6 +292,96 @@ typedef struct libxsmm_gp_reg_mapping_struct {
   unsigned int gp_reg_help_5;
 } libxsmm_gp_reg_mapping;
 
+/* convolution kernel config */
+typedef struct libxsmm_convolution_kernel_config_struct {
+  unsigned int instruction_set;
+  unsigned int vector_reg_count;
+  unsigned int vector_length_in;
+  unsigned int datatype_size_in;
+  unsigned int vector_length_out;
+  unsigned int datatype_size_out;
+  unsigned int vector_length_wt;
+  unsigned int datatype_size_wt;
+  unsigned int vmove_instruction;
+  unsigned int vbcst_instruction;
+  unsigned int vfma_instruction;
+  unsigned int vxor_instruction;
+  unsigned int vadd_instruction;
+  unsigned int prefetch_instruction;
+  unsigned int alu_add_instruction;
+  unsigned int alu_mul_instruction;
+  unsigned int alu_sub_instruction;
+  unsigned int alu_cmp_instruction;
+  unsigned int alu_jmp_instruction;
+  unsigned int alu_mov_instruction;
+  char vector_name;
+  unsigned int l_ld_ifm_act;
+  unsigned int l_ld_ofm_act;
+  unsigned int l_ld_ifm_fil;
+  unsigned int l_ld_ofm_fil;
+} libxsmm_convolution_kernel_config;
+
+/* struct for storing the current gp reg mapping for convolution */
+typedef struct libxsmm_convolution_forward_gp_reg_mapping_struct {
+  unsigned int gp_reg_input;
+  unsigned int gp_reg_weight;
+  unsigned int gp_reg_output;
+  unsigned int gp_reg_input_pf;
+  unsigned int gp_reg_weight_pf;
+  unsigned int gp_reg_output_pf;
+  unsigned int gp_reg_kw_loop;
+  unsigned int gp_reg_kh_loop;
+  unsigned int gp_reg_ifmInner_loop;
+  unsigned int gp_reg_help_0;
+  unsigned int gp_reg_help_1;
+  unsigned int gp_reg_help_2;
+  unsigned int gp_reg_help_3;
+  unsigned int gp_reg_help_4;
+  unsigned int gp_reg_help_5;
+  unsigned int gp_reg_help_6;
+} libxsmm_convolution_forward_gp_reg_mapping;
+
+/* struct for storing the current gp reg mapping for convolution */
+typedef struct libxsmm_convolution_backward_gp_reg_mapping_struct {
+  unsigned int gp_reg_input;
+  unsigned int gp_reg_weight;
+  unsigned int gp_reg_output;
+  unsigned int gp_reg_input_pf;
+  unsigned int gp_reg_weight_pf;
+  unsigned int gp_reg_output_pf;
+  unsigned int gp_reg_kw_loop;
+  unsigned int gp_reg_oi_loop;
+  unsigned int gp_reg_kh_loop;
+  unsigned int gp_reg_ofmInner_loop;
+  unsigned int gp_reg_help_0;
+  unsigned int gp_reg_help_1;
+  unsigned int gp_reg_help_2;
+  unsigned int gp_reg_help_3;
+  unsigned int gp_reg_help_4;
+  unsigned int gp_reg_help_5;
+  unsigned int gp_reg_help_6;
+} libxsmm_convolution_backward_gp_reg_mapping;
+
+/* struct for storing the current gp reg mapping for convolution */
+typedef struct libxsmm_convolution_weight_update_gp_reg_mapping_struct {
+  unsigned int gp_reg_input;
+  unsigned int gp_reg_weight;
+  unsigned int gp_reg_output;
+  unsigned int gp_reg_input_pf;
+  unsigned int gp_reg_weight_pf;
+  unsigned int gp_reg_output_pf;
+  unsigned int gp_reg_ifmInner_loop;
+  unsigned int gp_reg_oj_loop;
+  unsigned int gp_reg_oi_loop;
+  unsigned int gp_reg_help_0;
+  unsigned int gp_reg_help_1;
+  unsigned int gp_reg_help_2;
+  unsigned int gp_reg_help_3;
+  unsigned int gp_reg_help_4;
+  unsigned int gp_reg_help_5;
+  unsigned int gp_reg_help_6;
+} libxsmm_convolution_weight_update_gp_reg_mapping;
+
 /* struct for tracking local labels in assembly
    we don't allow overlapping loops */
 typedef struct libxsmm_loop_label_tracker_struct {
@@ -251,47 +389,69 @@ typedef struct libxsmm_loop_label_tracker_struct {
   unsigned int label_count;
 } libxsmm_loop_label_tracker;
 
+LIBXSMM_INTERNAL_API
 void libxsmm_reset_loop_label_tracker( libxsmm_loop_label_tracker* io_loop_label_tracker );
 
+LIBXSMM_INTERNAL_API
 void libxsmm_get_x86_gp_reg_name( const unsigned int i_gp_reg_number,
                                   char*              o_gp_reg_name,
                                   const int          i_gp_reg_name_max_length );
 
+LIBXSMM_INTERNAL_API
 unsigned int libxsmm_check_x86_gp_reg_name_callee_save( const unsigned int i_gp_reg_number );
 
+LIBXSMM_INTERNAL_API
 void libxsmm_get_x86_instr_name( const unsigned int i_instr_number,
                                  char*              o_instr_name,
                                  const int          i_instr_name_max_length );
 
+LIBXSMM_INTERNAL_API
 void libxsmm_reset_x86_gp_reg_mapping( libxsmm_gp_reg_mapping* io_gp_reg_mapping );
 
+LIBXSMM_INTERNAL_API
 unsigned int libxsmm_is_x86_vec_instr_single_precision( const unsigned int i_instr_number );
 
 /* some string manipulation helper needed to
    generated code */
+LIBXSMM_INTERNAL_API
 void libxsmm_append_code_as_string( libxsmm_generated_code* io_generated_code,
                                     const char*             i_code_to_append,
                                     const int               i_append_length );
 
+LIBXSMM_INTERNAL_API
 void libxsmm_close_function( libxsmm_generated_code* io_generated_code );
 
-void libxsmm_mmfunction_signature( libxsmm_generated_code*         io_generated_code,
-                                  const char*                     i_routine_name,
+LIBXSMM_INTERNAL_API
+void libxsmm_mmfunction_signature( libxsmm_generated_code*       io_generated_code,
+                                  const char*                    i_routine_name,
                                   const libxsmm_gemm_descriptor* i_xgemm_desc );
 
-void libxsmm_generator_isa_check_header( libxsmm_generated_code*  io_generated_code,
-                                         const char*              i_arch );
+LIBXSMM_INTERNAL_API
+void libxsmm_generator_isa_check_header( libxsmm_generated_code* io_generated_code,
+                                         const char*             i_arch );
 
-void libxsmm_generator_isa_check_footer( libxsmm_generated_code*  io_generated_code,
-                                         const char*              i_arch );
+LIBXSMM_INTERNAL_API
+void libxsmm_generator_isa_check_footer( libxsmm_generated_code* io_generated_code,
+                                         const char*             i_arch );
 
+LIBXSMM_INTERNAL_API
 void libxsmm_handle_error( libxsmm_generated_code* io_generated_code,
                            const unsigned int      i_error_code );
 
 
-void libxsmm_strncpy( char*                  o_dest,
-                      const char*            i_src,
-                      unsigned int           i_dest_length,
-                      unsigned int           i_src_length );
+LIBXSMM_INTERNAL_API
+void libxsmm_strncpy( char*        o_dest,
+                      const char*  i_src,
+                      unsigned int i_dest_length,
+                      unsigned int i_src_length );
+
+LIBXSMM_INTERNAL_API
+void libxsmm_convfunction_signature_fp32( libxsmm_generated_code*         io_generated_code,
+                                          const char*                     i_routine_name     );
+
+LIBXSMM_INTERNAL_API
+void libxsmm_convfunction_signature_int16( libxsmm_generated_code*         io_generated_code,
+                                           const char*                     i_routine_name     );
 
 #endif /* GENERATOR_COMMON_H */
+
