@@ -1,5 +1,5 @@
 /******************************************************************************
-** Copyright (c) 2015-2016, Intel Corporation                                **
+** Copyright (c) 2015-2017, Intel Corporation                                **
 ** All rights reserved.                                                      **
 **                                                                           **
 ** Redistribution and use in source and binary forms, with or without        **
@@ -37,14 +37,14 @@
 #include <libxsmm_macros.h>
 #include "generator_common.h"
 #include "generator_gemm_common.h"
-#include "generator_gemm_sse3_avx_avx2.h"
+#include "generator_gemm_sse3_avx_avx2_avx512.h"
 #include "generator_gemm_imci_avx512.h"
 #include "generator_gemm_noarch.h"
 
 /* @TODO change int based architecture value */
 void libxsmm_generator_gemm_kernel( libxsmm_generated_code*         io_generated_code,
-                                     const libxsmm_gemm_descriptor* i_xgemm_desc,
-                                     const char*                     i_arch ) {
+                                    const libxsmm_gemm_descriptor* i_xgemm_desc,
+                                    const char*                     i_arch ) {
   /* apply the alignement override */
   libxsmm_gemm_descriptor l_xgemm_desc_mod = *i_xgemm_desc;
   unsigned int l_vector_length = 1;
@@ -115,12 +115,19 @@ void libxsmm_generator_gemm_kernel( libxsmm_generated_code*         io_generated
        (strcmp(i_arch, "snb") == 0) ||
        (strcmp(i_arch, "hsw") == 0)    ) {
     /* call actual kernel generation with revised parameters */
-    libxsmm_generator_gemm_sse3_avx_avx2_kernel(io_generated_code, &l_xgemm_desc_mod, i_arch );
+    libxsmm_generator_gemm_sse3_avx_avx2_avx512_kernel(io_generated_code, &l_xgemm_desc_mod, i_arch );
   } else if ( (strcmp(i_arch, "knc") == 0) ||
-       (strcmp(i_arch, "knl") == 0) ||
-       (strcmp(i_arch, "skx") == 0)    ) {
+              (strcmp(i_arch, "knl") == 0)    ) {
     /* call actual kernel generation with revised parameters */
     libxsmm_generator_gemm_imci_avx512_kernel(io_generated_code, &l_xgemm_desc_mod, i_arch );
+  } else if ( (strcmp(i_arch, "skx") == 0) ) {
+    /* call actual kernel generation with revised parameters */
+    if ( (l_vector_length == 16  && (l_xgemm_desc_mod.m == 32 || l_xgemm_desc_mod.m == 48 || l_xgemm_desc_mod.m == 64)) ||
+         (l_vector_length == 8   && (l_xgemm_desc_mod.m == 16 || l_xgemm_desc_mod.m == 24 || l_xgemm_desc_mod.m == 32))    ) {
+      libxsmm_generator_gemm_sse3_avx_avx2_avx512_kernel(io_generated_code, &l_xgemm_desc_mod, i_arch );
+    } else {
+      libxsmm_generator_gemm_imci_avx512_kernel(io_generated_code, &l_xgemm_desc_mod, i_arch );
+    }
   } else if ( (strcmp(i_arch, "noarch") == 0) ) {
     /* call actual kernel generation with revised parameters */
     libxsmm_generator_gemm_noarch_kernel(io_generated_code, &l_xgemm_desc_mod, i_arch );
@@ -169,18 +176,18 @@ void libxsmm_generator_gemm_inlineasm(const char*                     i_file_out
   if ( l_generated_code.last_error != 0 ) {
     fprintf(stderr, "LIBXSMM ERROR there was an error generating code. Last known error is:\n%s\n",
       libxsmm_strerror(l_generated_code.last_error));
-    exit(-1);
+    return;
   }
 
   /* append code to source file */
   {
     FILE *const l_file_handle = fopen( i_file_out, "a" );
     if ( l_file_handle != NULL ) {
-      fputs( l_generated_code.generated_code, l_file_handle );
+      fputs( (const char*)l_generated_code.generated_code, l_file_handle );
       fclose( l_file_handle );
     } else {
       fprintf(stderr, "LIBXSMM ERROR libxsmm_generator_gemm_inlineasm could not write to into destination source file\n");
-      exit(-1);
+      return;
     }
   }
 
@@ -203,7 +210,7 @@ void libxsmm_generator_gemm_directasm(const char*                     i_file_out
   /* check if we are not noarch */
   if ( strcmp( i_arch, "noarch" ) == 0 ) {
     fprintf(stderr, "LIBXSMM ERROR, libxsmm_generator_gemm_direct: we cannot create ASM when noarch is specified!\n");
-    exit(-1);
+    return;
   }
 
   /* add signature to code string */
@@ -216,18 +223,18 @@ void libxsmm_generator_gemm_directasm(const char*                     i_file_out
   if ( l_generated_code.last_error != 0 ) {
     fprintf(stderr, "LIBXSMM ERROR there was an error generating code. Last known error is:\n%s\n",
       libxsmm_strerror(l_generated_code.last_error));
-    exit(-1);
+    return;
   }
 
   /* append code to source file */
   {
     FILE *const l_file_handle = fopen( i_file_out, "w" );
     if ( l_file_handle != NULL ) {
-      fputs( l_generated_code.generated_code, l_file_handle );
+      fputs( (const char*)l_generated_code.generated_code, l_file_handle );
       fclose( l_file_handle );
     } else {
       fprintf(stderr, "LIBXSMM ERROR, libxsmm_generator_gemm_direct: could not write to into destination source file!\n");
-      exit(-1);
+      return;
     }
   }
 
