@@ -1,5 +1,5 @@
 /******************************************************************************
-** Copyright (c) 2015-2016, Intel Corporation                                **
+** Copyright (c) 2015-2017, Intel Corporation                                **
 ** All rights reserved.                                                      **
 **                                                                           **
 ** Redistribution and use in source and binary forms, with or without        **
@@ -36,13 +36,13 @@
 #include <stdlib.h>
 #include <string.h>
 
-
+LIBXSMM_INTERNAL_API_DEFINITION
 void libxsmm_generator_gemm_avx512_microkernel( libxsmm_generated_code*             io_generated_code,
-                                                 const libxsmm_gp_reg_mapping*       i_gp_reg_mapping,
-                                                 const libxsmm_micro_kernel_config*  i_micro_kernel_config,
-                                                 const libxsmm_gemm_descriptor*      i_xgemm_desc,
-                                                 const unsigned int                  i_n_blocking,
-                                                 const unsigned int                  i_k_blocking )
+                                                 const libxsmm_gp_reg_mapping*      i_gp_reg_mapping,
+                                                 const libxsmm_micro_kernel_config* i_micro_kernel_config,
+                                                 const libxsmm_gemm_descriptor*     i_xgemm_desc,
+                                                 const unsigned int                 i_n_blocking,
+                                                 const unsigned int                 i_k_blocking )
 {
   unsigned int l_n;
   unsigned int l_k;
@@ -54,6 +54,7 @@ void libxsmm_generator_gemm_avx512_microkernel( libxsmm_generated_code*
   unsigned int l_k_b_updates = 0;
   unsigned int l_displacement_k_a = 0;
   unsigned int l_k_a_update = 0;
+  unsigned int l_n_accs = 0;
 
 #if !defined(NDEBUG)
   if ( i_n_blocking > 30 ) {
@@ -62,6 +63,15 @@ void libxsmm_generator_gemm_avx512_microkernel( libxsmm_generated_code*
   }
 #endif
 
+  /* compute number of n accumulators to hide FMA latencies */
+  if (i_n_blocking >= 12) {
+    l_n_accs = 1;
+  } else if (i_n_blocking >= 6) {
+    l_n_accs = 2;
+  } else {
+    l_n_accs = 4;
+  }
+
   /* Intialize helper registers for SIB addressing */
   /* helper 0: Index register holding ldb*datatype_size */
   libxsmm_x86_instruction_alu_imm( io_generated_code, i_micro_kernel_config->alu_mov_instruction,
@@ -77,7 +87,7 @@ void libxsmm_generator_gemm_avx512_microkernel( libxsmm_generated_code*
                                    i_gp_reg_mapping->gp_reg_help_3, i_micro_kernel_config->datatype_size * i_xgemm_desc->ldb * 7 );
 
   /* helper 4: B + 9*ldb, additional base address
-     helper 5: B + 18*ldb, additional base address 
+     helper 5: B + 18*ldb, additional base address
      helper 6: B + 27*ldb, additional base address, using the the prefetch b register, which was save to stack */
   if ( i_n_blocking > 9 ) {
     libxsmm_x86_instruction_alu_reg( io_generated_code, i_micro_kernel_config->alu_mov_instruction, i_gp_reg_mapping->gp_reg_b, i_gp_reg_mapping->gp_reg_help_4);
@@ -98,25 +108,38 @@ void libxsmm_generator_gemm_avx512_microkernel( libxsmm_generated_code*
   l_displacement_k_b = 0;
   l_displacement_k_a = 0;
 
+  /* xor additional accumulator, if needed */
+  for ( l_k = 1; l_k < l_n_accs; l_k++) {
+    for ( l_n = 0; l_n < i_n_blocking; l_n++) {
+      libxsmm_x86_instruction_vec_compute_reg( io_generated_code,
+                                           i_micro_kernel_config->instruction_set,
+                                           i_micro_kernel_config->vxor_instruction,
+                                           i_micro_kernel_config->vector_name,
+                                           i_micro_kernel_config->vector_reg_count - (i_n_blocking*(l_k+1)) + l_n,
+                                           i_micro_kernel_config->vector_reg_count - (i_n_blocking*(l_k+1)) + l_n,
+                                           i_micro_kernel_config->vector_reg_count - (i_n_blocking*(l_k+1)) + l_n );
+    }
+  }
+
   /* apply k blocking */
   for ( l_k = 0; l_k < i_k_blocking; l_k++ ) {
     /* advance b pointer if needed */
     if ( (l_k > 0) && ((l_k%128) == 0) ) {
-      libxsmm_x86_instruction_alu_imm( io_generated_code, i_micro_kernel_config->alu_add_instruction, 
+      libxsmm_x86_instruction_alu_imm( io_generated_code, i_micro_kernel_config->alu_add_instruction,
                                        i_gp_reg_mapping->gp_reg_b, 128*i_micro_kernel_config->datatype_size );
       /* advance the second base pointer only if it's needed */
       if ( i_n_blocking > 9 ) {
-        libxsmm_x86_instruction_alu_imm( io_generated_code, i_micro_kernel_config->alu_add_instruction, 
+        libxsmm_x86_instruction_alu_imm( io_generated_code, i_micro_kernel_config->alu_add_instruction,
                                          i_gp_reg_mapping->gp_reg_help_4, 128*i_micro_kernel_config->datatype_size );
       }
       /* advance the third base pointer only if it's needed */
       if ( i_n_blocking > 18 ) {
-        libxsmm_x86_instruction_alu_imm( io_generated_code, i_micro_kernel_config->alu_add_instruction, 
+        libxsmm_x86_instruction_alu_imm( io_generated_code, i_micro_kernel_config->alu_add_instruction,
                                          i_gp_reg_mapping->gp_reg_help_5, 128*i_micro_kernel_config->datatype_size );
       }
       /* advance the fourth base pointer only if it's needed */
       if ( i_n_blocking > 27 ) {
-        libxsmm_x86_instruction_alu_imm( io_generated_code, i_micro_kernel_config->alu_add_instruction, 
+        libxsmm_x86_instruction_alu_imm( io_generated_code, i_micro_kernel_config->alu_add_instruction,
                                          i_gp_reg_mapping->gp_reg_b_prefetch, 128*i_micro_kernel_config->datatype_size );
       }
 
@@ -221,7 +244,7 @@ void libxsmm_generator_gemm_avx512_microkernel( libxsmm_generated_code*
                                         i_gp_reg_mapping->gp_reg_a_prefetch,
                                         LIBXSMM_X86_GP_REG_UNDEF, 0,
                                         (i_xgemm_desc->lda * l_k * i_micro_kernel_config->datatype_size) );
-      if ( l_k == (i_k_blocking - 1) && (i_k_blocking != i_xgemm_desc->k) ) {
+      if ( l_k == (i_k_blocking - 1) && (i_k_blocking != (unsigned int)i_xgemm_desc->k) ) {
         libxsmm_x86_instruction_alu_imm( io_generated_code,
                                          i_micro_kernel_config->alu_add_instruction,
                                          i_gp_reg_mapping->gp_reg_a_prefetch,
@@ -230,7 +253,7 @@ void libxsmm_generator_gemm_avx512_microkernel( libxsmm_generated_code*
     }
 
     /* in last k-iteration: advance pointers */
-    if ( (l_k == (i_k_blocking - 1)) && (i_k_blocking != i_xgemm_desc->k) ) {
+    if ( (l_k == (i_k_blocking - 1)) && (i_k_blocking != (unsigned int)i_xgemm_desc->k) ) {
       libxsmm_x86_instruction_alu_imm( io_generated_code,
                                        i_micro_kernel_config->alu_add_instruction,
                                        i_gp_reg_mapping->gp_reg_a,
@@ -254,7 +277,7 @@ void libxsmm_generator_gemm_avx512_microkernel( libxsmm_generated_code*
       } else {
         l_b_reg = i_gp_reg_mapping->gp_reg_b;
       }
-      /* Select SIB */ 
+      /* Select SIB */
       if ( l_n % 9 == 0 ) {
         l_b_idx = LIBXSMM_X86_GP_REG_UNDEF;
         l_scale = 0;
@@ -285,7 +308,7 @@ void libxsmm_generator_gemm_avx512_microkernel( libxsmm_generated_code*
       } else {
         /* shouldn't happen.... */
       }
-        
+
 #if 1
       libxsmm_x86_instruction_vec_compute_mem( io_generated_code,
                                                i_micro_kernel_config->instruction_set,
@@ -297,7 +320,7 @@ void libxsmm_generator_gemm_avx512_microkernel( libxsmm_generated_code*
                                                l_disp,
                                                i_micro_kernel_config->vector_name,
                                                l_k%2,
-                                               i_micro_kernel_config->vector_reg_count - i_n_blocking + l_n );
+                                               i_micro_kernel_config->vector_reg_count - (i_n_blocking*((l_k%l_n_accs)+1)) + l_n );
 #else
       libxsmm_x86_instruction_vec_compute_mem( io_generated_code,
                                                i_micro_kernel_config->instruction_set,
@@ -309,7 +332,7 @@ void libxsmm_generator_gemm_avx512_microkernel( libxsmm_generated_code*
                                                (l_k * i_micro_kernel_config->datatype_size)+(i_xgemm_desc->ldb * i_micro_kernel_config->datatype_size * l_n),
                                                i_micro_kernel_config->vector_name,
                                                l_k%2,
-                                               i_micro_kernel_config->vector_reg_count - i_n_blocking + l_n );
+                                               i_micro_kernel_config->vector_reg_count - (i_n_blocking*((l_k%l_n_accs)+1)) + l_n );
 #endif
     }
     l_displacement_k_b++;
@@ -317,12 +340,12 @@ void libxsmm_generator_gemm_avx512_microkernel( libxsmm_generated_code*
 
   /* Adjust a pointer */
   if (l_k_a_update > 0) {
-    libxsmm_x86_instruction_alu_imm( io_generated_code, i_micro_kernel_config->alu_sub_instruction, i_gp_reg_mapping->gp_reg_a, 
+    libxsmm_x86_instruction_alu_imm( io_generated_code, i_micro_kernel_config->alu_sub_instruction, i_gp_reg_mapping->gp_reg_a,
                                      l_k_a_update );
   }
 
   /* advance pointers of B only when we are not fully unrolling K and taking care of intermediate advances */
-  if ( i_k_blocking < i_xgemm_desc->k ) {
+  if ( i_k_blocking < (unsigned int)i_xgemm_desc->k ) {
     /* advance pointers of B */
     libxsmm_x86_instruction_alu_imm( io_generated_code,
                                      i_micro_kernel_config->alu_add_instruction,
@@ -331,32 +354,46 @@ void libxsmm_generator_gemm_avx512_microkernel( libxsmm_generated_code*
   } else {
     /* we have to make sure that we are reseting the pointer to its original value in case a full unroll */
     if ( l_k_b_updates > 0 ) {
-      libxsmm_x86_instruction_alu_imm( io_generated_code, 
-                                       i_micro_kernel_config->alu_sub_instruction, 
+      libxsmm_x86_instruction_alu_imm( io_generated_code,
+                                       i_micro_kernel_config->alu_sub_instruction,
                                        i_gp_reg_mapping->gp_reg_b, 128*(i_micro_kernel_config->datatype_size)*l_k_b_updates );
     }
   }
+
+  /* add additional accumulators, if needed */
+  for ( l_k = 1; l_k < l_n_accs; l_k++) {
+    for ( l_n = 0; l_n < i_n_blocking; l_n++) {
+      libxsmm_x86_instruction_vec_compute_reg( io_generated_code,
+                                           i_micro_kernel_config->instruction_set,
+                                           i_micro_kernel_config->vadd_instruction,
+                                           i_micro_kernel_config->vector_name,
+                                           i_micro_kernel_config->vector_reg_count - (i_n_blocking*(l_k+1)) + l_n,
+                                           i_micro_kernel_config->vector_reg_count - i_n_blocking + l_n,
+                                           i_micro_kernel_config->vector_reg_count - i_n_blocking + l_n );
+    }
+  }
 }
 
 #if 0
+LIBXSMM_INTERNAL_API_DEFINITION
 void libxsmm_generator_gemm_avx512_microkernel_k_large( libxsmm_generated_code*             io_generated_code,
-                                                         const libxsmm_gp_reg_mapping*       i_gp_reg_mapping,
-                                                         const libxsmm_micro_kernel_config*  i_micro_kernel_config,
+                                                         const libxsmm_gp_reg_mapping*      i_gp_reg_mapping,
+                                                         const libxsmm_micro_kernel_config* i_micro_kernel_config,
                                                          const libxsmm_gemm_descriptor*     i_xgemm_desc,
-                                                         const unsigned int                  i_n_blocking,
-                                                         const unsigned int                  i_k_blocking )
+                                                         const unsigned int                 i_n_blocking,
+                                                         const unsigned int                 i_k_blocking )
 {
   unsigned int l_n;
   unsigned int l_k;
 
 #if !defined(NDEBUG)
   if ( i_n_blocking > 24 ) {
-    fprintf(stderr, "LIBXSMM ERROR, libxsmm_generator_gemm_avx512_microkernel_k_large: i_n_blocking needs to be 24 or smaller!\n");
-    exit(-1);
+    libxsmm_handle_error( io_generated_code, LIBXSMM_ERR_N_BLOCK );
+    return;
   }
   if ( i_k_blocking < 8 ) {
-    fprintf(stderr, "LIBXSMM ERROR, libxsmm_generator_gemm_avx512_microkernel_k_large: i_k_blocking needs to be at least 8!\n");
-    exit(-1);
+    libxsmm_handle_error( io_generated_code, LIBXSMM_ERR_K_BLOCK );
+    return;
   }
 #endif
 
@@ -517,11 +554,12 @@ void libxsmm_generator_gemm_avx512_microkernel_k_large( libxsmm_generated_code*
 }
 #endif
 
+LIBXSMM_INTERNAL_API_DEFINITION
 void libxsmm_generator_gemm_avx512_microkernel_k_large_n_nine( libxsmm_generated_code*             io_generated_code,
-                                                                const libxsmm_gp_reg_mapping*       i_gp_reg_mapping,
-                                                                const libxsmm_micro_kernel_config*  i_micro_kernel_config,
-                                                                const libxsmm_gemm_descriptor*      i_xgemm_desc,
-                                                                const unsigned int                  i_k_blocking )
+                                                                const libxsmm_gp_reg_mapping*      i_gp_reg_mapping,
+                                                                const libxsmm_micro_kernel_config* i_micro_kernel_config,
+                                                                const libxsmm_gemm_descriptor*     i_xgemm_desc,
+                                                                const unsigned int                 i_k_blocking )
 {
   unsigned int l_n;
   unsigned int l_k;
@@ -533,8 +571,8 @@ void libxsmm_generator_gemm_avx512_microkernel_k_large_n_nine( libxsmm_generated
 
 #if !defined(NDEBUG)
   if ( i_k_blocking < 8 ) {
-    fprintf(stderr, "LIBXSMM ERROR, libxsmm_generator_gemm_avx512_microkernel_k_large_n_nine: i_k_blocking needs to be at least 8!\n");
-    exit(-1);
+    libxsmm_handle_error( io_generated_code, LIBXSMM_ERR_K_BLOCK );
+    return;
   }
 #endif
 
@@ -571,7 +609,7 @@ void libxsmm_generator_gemm_avx512_microkernel_k_large_n_nine( libxsmm_generated
         l_displacement_k_b = 0;
         l_k_b_updates++;
       }
-    } else { 
+    } else {
       l_displacement_k_b = 0;
       l_k_b_updates = 0;
     }
@@ -606,7 +644,7 @@ void libxsmm_generator_gemm_avx512_microkernel_k_large_n_nine( libxsmm_generated
       } else {
         l_displacement_k_a++;
       }
-      
+
       libxsmm_x86_instruction_vec_move( io_generated_code,
                                     i_micro_kernel_config->instruction_set,
                                     i_micro_kernel_config->a_vmove_instruction,
@@ -866,7 +904,7 @@ void libxsmm_generator_gemm_avx512_microkernel_k_large_n_nine( libxsmm_generated
                                     i_gp_reg_mapping->gp_reg_a_prefetch,
                                     LIBXSMM_X86_GP_REG_UNDEF, 0,
                                     (i_xgemm_desc->lda * l_k * i_micro_kernel_config->datatype_size) );
-      if ( l_k == (i_k_blocking - 1) && (i_k_blocking != i_xgemm_desc->k) ) {
+      if ( l_k == (i_k_blocking - 1) && (i_k_blocking != (unsigned int)i_xgemm_desc->k) ) {
         libxsmm_x86_instruction_alu_imm( io_generated_code,
                                      i_micro_kernel_config->alu_add_instruction,
                                      i_gp_reg_mapping->gp_reg_a_prefetch,
@@ -890,7 +928,7 @@ void libxsmm_generator_gemm_avx512_microkernel_k_large_n_nine( libxsmm_generated
     }
 
     /* in last k-iteration: advance pointers */
-    if ( l_k == (i_k_blocking - 1) && (i_k_blocking != i_xgemm_desc->k) ) {
+    if ( l_k == (i_k_blocking - 1) && (i_k_blocking != (unsigned int)i_xgemm_desc->k) ) {
       libxsmm_x86_instruction_alu_imm( io_generated_code,
                                    i_micro_kernel_config->alu_add_instruction,
                                    i_gp_reg_mapping->gp_reg_a,
@@ -1044,12 +1082,12 @@ void libxsmm_generator_gemm_avx512_microkernel_k_large_n_nine( libxsmm_generated
   }
 
   if (l_k_b_updates > 0) {
-    libxsmm_x86_instruction_alu_imm( io_generated_code, i_micro_kernel_config->alu_sub_instruction, i_gp_reg_mapping->gp_reg_b, 
+    libxsmm_x86_instruction_alu_imm( io_generated_code, i_micro_kernel_config->alu_sub_instruction, i_gp_reg_mapping->gp_reg_b,
                                      128*(i_micro_kernel_config->datatype_size)*l_k_b_updates );
   }
 
   if (l_k_a_update > 0) {
-    libxsmm_x86_instruction_alu_imm( io_generated_code, i_micro_kernel_config->alu_sub_instruction, i_gp_reg_mapping->gp_reg_a, 
+    libxsmm_x86_instruction_alu_imm( io_generated_code, i_micro_kernel_config->alu_sub_instruction, i_gp_reg_mapping->gp_reg_a,
                                      l_k_a_update );
   }
 
@@ -1067,7 +1105,7 @@ void libxsmm_generator_gemm_avx512_microkernel_k_large_n_nine( libxsmm_generated
   }
 
   /* advance pointers of B only when we are not fully unrolling K*/
-  if ( i_k_blocking < i_xgemm_desc->k ) {
+  if ( i_k_blocking < (unsigned int)i_xgemm_desc->k ) {
     libxsmm_x86_instruction_alu_imm( io_generated_code,
                                  i_micro_kernel_config->alu_add_instruction,
                                  i_gp_reg_mapping->gp_reg_b,
@@ -1075,13 +1113,14 @@ void libxsmm_generator_gemm_avx512_microkernel_k_large_n_nine( libxsmm_generated
   }
 }
 
+LIBXSMM_INTERNAL_API_DEFINITION
 unsigned int libxsmm_generator_gemm_avx512_kernel_kloop( libxsmm_generated_code*            io_generated_code,
-                                                          libxsmm_loop_label_tracker*        io_loop_label_tracker,
-                                                          const libxsmm_gp_reg_mapping*      i_gp_reg_mapping,
-                                                          const libxsmm_micro_kernel_config* i_micro_kernel_config,
-                                                          const libxsmm_gemm_descriptor*    i_xgemm_desc,
-                                                          const char*                        i_arch,
-                                                          unsigned int                       i_n_blocking ) {
+                                                         libxsmm_loop_label_tracker*        io_loop_label_tracker,
+                                                         const libxsmm_gp_reg_mapping*      i_gp_reg_mapping,
+                                                         const libxsmm_micro_kernel_config* i_micro_kernel_config,
+                                                         const libxsmm_gemm_descriptor*     i_xgemm_desc,
+                                                         const char*                        i_arch,
+                                                         unsigned int                       i_n_blocking ) {
   /* l_k_blocking must be smaller than l_k_threshold */
   const unsigned int l_k_blocking = 8;
   const unsigned int l_k_threshold = 64;
@@ -1090,7 +1129,8 @@ unsigned int libxsmm_generator_gemm_avx512_kernel_kloop( libxsmm_generated_code*
   LIBXSMM_UNUSED(i_arch);
 
   if ( (l_k_blocking >= l_k_threshold) ) {
-    fprintf(stderr, "LIBXSMM AVX512 micro kenrel: l_k_blocking must be smaller than l_k_threshold\n");
+    libxsmm_handle_error( io_generated_code, LIBXSMM_ERR_K_BLOCK );
+    return 0;
   }
 
   /* Let's do something special for SeisSol high-order (N == 9 holds true) */
@@ -1101,7 +1141,7 @@ unsigned int libxsmm_generator_gemm_avx512_kernel_kloop( libxsmm_generated_code*
                                                                i_xgemm_desc,
                                                                i_xgemm_desc->k );
     l_k_unrolled = 1;
-  } else if ( i_xgemm_desc->k <= l_k_threshold ) {
+  } else if ( (unsigned int)i_xgemm_desc->k <= l_k_threshold ) {
     libxsmm_generator_gemm_avx512_microkernel( io_generated_code,
                                                 i_gp_reg_mapping,
                                                 i_micro_kernel_config,
@@ -1109,7 +1149,7 @@ unsigned int libxsmm_generator_gemm_avx512_kernel_kloop( libxsmm_generated_code*
                                                 i_n_blocking,
                                                 i_xgemm_desc->k);
     l_k_unrolled = 1;
-  } else if ( (i_xgemm_desc->k % l_k_blocking == 0) && (i_xgemm_desc->k > l_k_threshold) ) {
+  } else if ( (i_xgemm_desc->k % l_k_blocking == 0) && (l_k_threshold < (unsigned int)i_xgemm_desc->k) ) {
     libxsmm_generator_gemm_header_kloop( io_generated_code, io_loop_label_tracker, i_gp_reg_mapping, i_micro_kernel_config,
                                           i_micro_kernel_config->vector_length, l_k_blocking);
 
