@@ -1,5 +1,5 @@
 /******************************************************************************
-** Copyright (c) 2015-2016, Intel Corporation                                **
+** Copyright (c) 2015-2017, Intel Corporation                                **
 ** All rights reserved.                                                      **
 **                                                                           **
 ** Redistribution and use in source and binary forms, with or without        **
@@ -32,16 +32,17 @@
 #include "generator_gemm_common.h"
 #include "generator_common.h"
 #include "generator_x86_instructions.h"
-#include <libxsmm_macros.h>
+
+#include <libxsmm_intrinsics_x86.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 
-
+LIBXSMM_INTERNAL_API_DEFINITION
 void libxsmm_generator_gemm_init_micro_kernel_config_fullvector( libxsmm_micro_kernel_config*    io_micro_kernel_config,
                                                                   const libxsmm_gemm_descriptor* i_xgemm_desc,
-                                                                  const char*                     i_arch,
-                                                                  const unsigned int              i_use_masking_a_c ) {
+                                                                  const char*                    i_arch,
+                                                                  const unsigned int             i_use_masking_a_c ) {
   if( strcmp( i_arch, "wsm" ) == 0 ) {
     io_micro_kernel_config->instruction_set = LIBXSMM_X86_SSE3;
     io_micro_kernel_config->vector_reg_count = 16;
@@ -189,7 +190,7 @@ void libxsmm_generator_gemm_init_micro_kernel_config_fullvector( libxsmm_micro_k
       } else {
         io_micro_kernel_config->a_vmove_instruction = LIBXSMM_X86_INSTR_VMOVUPD;
       }
-      io_micro_kernel_config->b_vmove_instruction = LIBXSMM_X86_INSTR_UNDEF;
+      io_micro_kernel_config->b_vmove_instruction = LIBXSMM_X86_INSTR_VBROADCASTSD;
       io_micro_kernel_config->b_shuff_instruction = LIBXSMM_X86_INSTR_UNDEF;
       if ( (LIBXSMM_GEMM_FLAG_ALIGN_C & i_xgemm_desc->flags) != 0 ) {
         io_micro_kernel_config->c_vmove_instruction = LIBXSMM_X86_INSTR_VMOVAPD;
@@ -207,7 +208,7 @@ void libxsmm_generator_gemm_init_micro_kernel_config_fullvector( libxsmm_micro_k
       } else {
         io_micro_kernel_config->a_vmove_instruction = LIBXSMM_X86_INSTR_VMOVUPS;
       }
-      io_micro_kernel_config->b_vmove_instruction = LIBXSMM_X86_INSTR_UNDEF;
+      io_micro_kernel_config->b_vmove_instruction = LIBXSMM_X86_INSTR_VBROADCASTSS;
       io_micro_kernel_config->b_shuff_instruction = LIBXSMM_X86_INSTR_UNDEF;
       if ( (LIBXSMM_GEMM_FLAG_ALIGN_C & i_xgemm_desc->flags) != 0 ) {
         io_micro_kernel_config->c_vmove_instruction = LIBXSMM_X86_INSTR_VMOVAPS;
@@ -228,12 +229,15 @@ void libxsmm_generator_gemm_init_micro_kernel_config_fullvector( libxsmm_micro_k
   io_micro_kernel_config->alu_mov_instruction = LIBXSMM_X86_INSTR_MOVQ;
 }
 
+LIBXSMM_INTERNAL_API_DEFINITION
 void libxsmm_generator_gemm_init_micro_kernel_config_halfvector( libxsmm_micro_kernel_config*    io_micro_kernel_config,
                                                                   const libxsmm_gemm_descriptor* i_xgemm_desc,
-                                                                  const char*                     i_arch,
-                                                                  const unsigned int              i_use_masking_a_c ) {
+                                                                  const char*                    i_arch,
+                                                                  const unsigned int             i_use_masking_a_c ) {
   if( strcmp( i_arch, "wsm" ) == 0 ) {
-    fprintf(stderr, "LIBXSMM WARNING, ibxsmm_generator_gemm_init_micro_kernel_config_halfvector, redirecting to scalar, please fix the generation code!!!\n");
+#if !defined(NDEBUG)
+    fprintf(stderr, "LIBXSMM WARNING, libxsmm_generator_gemm_init_micro_kernel_config_halfvector, redirecting to scalar, please fix the generation code!!!\n");
+#endif
     libxsmm_generator_gemm_init_micro_kernel_config_scalar( io_micro_kernel_config, i_xgemm_desc, i_arch, i_use_masking_a_c );
   } else if( strcmp( i_arch, "snb" ) == 0 ) {
     io_micro_kernel_config->instruction_set = LIBXSMM_X86_AVX;
@@ -322,7 +326,9 @@ void libxsmm_generator_gemm_init_micro_kernel_config_halfvector( libxsmm_micro_k
   } else if ( (strcmp( i_arch, "knc" ) == 0) ||
               (strcmp( i_arch, "knl" ) == 0) ||
               (strcmp( i_arch, "skx" ) == 0)    ) {
-    fprintf(stderr, "LIBXSMM WARNING, ibxsmm_generator_gemm_init_micro_kernel_config_halfvector, IMCI/AVX512 redirecting to fullvector, please fix the generation code!!!\n");
+#if !defined(NDEBUG)
+    fprintf(stderr, "LIBXSMM WARNING, libxsmm_generator_gemm_init_micro_kernel_config_halfvector, IMCI/AVX512 redirecting to fullvector, please fix the generation code!!!\n");
+#endif
     libxsmm_generator_gemm_init_micro_kernel_config_fullvector( io_micro_kernel_config, i_xgemm_desc, i_arch, i_use_masking_a_c );
   } else {  }
 
@@ -334,10 +340,11 @@ void libxsmm_generator_gemm_init_micro_kernel_config_halfvector( libxsmm_micro_k
   io_micro_kernel_config->alu_mov_instruction = LIBXSMM_X86_INSTR_MOVQ;
 }
 
+LIBXSMM_INTERNAL_API_DEFINITION
 void libxsmm_generator_gemm_init_micro_kernel_config_scalar( libxsmm_micro_kernel_config*    io_micro_kernel_config,
                                                               const libxsmm_gemm_descriptor* i_xgemm_desc,
-                                                              const char*                     i_arch,
-                                                              const unsigned int              i_use_masking_a_c ) {
+                                                              const char*                    i_arch,
+                                                              const unsigned int             i_use_masking_a_c ) {
   if( strcmp( i_arch, "wsm" ) == 0 ) {
     io_micro_kernel_config->instruction_set = LIBXSMM_X86_SSE3;
     io_micro_kernel_config->vector_reg_count = 16;
@@ -419,8 +426,41 @@ void libxsmm_generator_gemm_init_micro_kernel_config_scalar( libxsmm_micro_kerne
   } else if ( (strcmp( i_arch, "knc" ) == 0) ||
               (strcmp( i_arch, "knl" ) == 0) ||
               (strcmp( i_arch, "skx" ) == 0)    ) {
-    fprintf(stderr, "LIBXSMM WARNING, ibxsmm_generator_gemm_init_micro_kernel_config_halfvector, IMCI/AVX512 redirecting to fullvector, please fix the generation code!!!\n");
-    libxsmm_generator_gemm_init_micro_kernel_config_fullvector( io_micro_kernel_config, i_xgemm_desc, i_arch, i_use_masking_a_c );
+    if ((strcmp( i_arch, "knc" ) == 0)) {
+      io_micro_kernel_config->instruction_set = LIBXSMM_X86_IMCI;
+#if !defined(NDEBUG)
+      fprintf(stderr, "LIBXSMM WARNING, libxsmm_generator_gemm_init_micro_kernel_config_scalar, IMCI redirecting to fullvector, please fix the generation code!!!\n");
+#endif
+      libxsmm_generator_gemm_init_micro_kernel_config_fullvector( io_micro_kernel_config, i_xgemm_desc, i_arch, i_use_masking_a_c );
+    } else if ((strcmp( i_arch, "knl" ) == 0)) {
+      io_micro_kernel_config->instruction_set = LIBXSMM_X86_AVX512_MIC;
+    } else {
+      io_micro_kernel_config->instruction_set = LIBXSMM_X86_AVX512_CORE;
+    }
+    io_micro_kernel_config->vector_reg_count = 16;
+    io_micro_kernel_config->use_masking_a_c = i_use_masking_a_c;
+    io_micro_kernel_config->vector_name = 'x';
+    if ( (LIBXSMM_GEMM_FLAG_F32PREC & i_xgemm_desc->flags) == 0 ) {
+      io_micro_kernel_config->vector_length = 1;
+      io_micro_kernel_config->datatype_size = 8;
+      io_micro_kernel_config->a_vmove_instruction = LIBXSMM_X86_INSTR_VMOVSD;
+      io_micro_kernel_config->b_vmove_instruction = LIBXSMM_X86_INSTR_VMOVSD;
+      io_micro_kernel_config->b_shuff_instruction = LIBXSMM_X86_INSTR_UNDEF;
+      io_micro_kernel_config->c_vmove_instruction = LIBXSMM_X86_INSTR_VMOVSD;
+      io_micro_kernel_config->vxor_instruction = LIBXSMM_X86_INSTR_VPXORD;
+      io_micro_kernel_config->vmul_instruction = LIBXSMM_X86_INSTR_VFMADD231SD;
+      io_micro_kernel_config->vadd_instruction = LIBXSMM_X86_INSTR_VADDSD;
+    } else {
+      io_micro_kernel_config->vector_length = 1;
+      io_micro_kernel_config->datatype_size = 4;
+      io_micro_kernel_config->a_vmove_instruction = LIBXSMM_X86_INSTR_VMOVSS;
+      io_micro_kernel_config->b_vmove_instruction = LIBXSMM_X86_INSTR_VMOVSS;
+      io_micro_kernel_config->b_shuff_instruction = LIBXSMM_X86_INSTR_UNDEF;
+      io_micro_kernel_config->c_vmove_instruction = LIBXSMM_X86_INSTR_VMOVSS;
+      io_micro_kernel_config->vxor_instruction = LIBXSMM_X86_INSTR_VPXORD;
+      io_micro_kernel_config->vmul_instruction = LIBXSMM_X86_INSTR_VFMADD231SS;
+      io_micro_kernel_config->vadd_instruction = LIBXSMM_X86_INSTR_VADDSS;
+    }
   } else {  }
 
   io_micro_kernel_config->prefetch_instruction = LIBXSMM_X86_INSTR_PREFETCHT1;
@@ -431,6 +471,7 @@ void libxsmm_generator_gemm_init_micro_kernel_config_scalar( libxsmm_micro_kerne
   io_micro_kernel_config->alu_mov_instruction = LIBXSMM_X86_INSTR_MOVQ;
 }
 
+LIBXSMM_INTERNAL_API_DEFINITION
 void libxsmm_generator_gemm_add_flop_counter( libxsmm_generated_code*         io_generated_code,
                                                const libxsmm_gemm_descriptor* i_xgemm_desc ) {
   if ( io_generated_code->code_type == 0 ) {
@@ -453,26 +494,28 @@ void libxsmm_generator_gemm_add_flop_counter( libxsmm_generated_code*         io
   }
 }
 
+LIBXSMM_INTERNAL_API_DEFINITION
 void libxsmm_generator_gemm_header_kloop( libxsmm_generated_code*             io_generated_code,
-                                           libxsmm_loop_label_tracker*         io_loop_label_tracker,
-                                           const libxsmm_gp_reg_mapping*       i_gp_reg_mapping,
-                                           const libxsmm_micro_kernel_config*  i_micro_kernel_config,
-                                           const unsigned int                  i_m_blocking,
-                                           const unsigned int                  i_k_blocking ) {
+                                           libxsmm_loop_label_tracker*        io_loop_label_tracker,
+                                           const libxsmm_gp_reg_mapping*      i_gp_reg_mapping,
+                                           const libxsmm_micro_kernel_config* i_micro_kernel_config,
+                                           const unsigned int                 i_m_blocking,
+                                           const unsigned int                 i_k_blocking ) {
   LIBXSMM_UNUSED(i_m_blocking);
   libxsmm_x86_instruction_alu_imm( io_generated_code, i_micro_kernel_config->alu_mov_instruction, i_gp_reg_mapping->gp_reg_kloop, 0);
   libxsmm_x86_instruction_register_jump_label( io_generated_code, io_loop_label_tracker );
   libxsmm_x86_instruction_alu_imm( io_generated_code, i_micro_kernel_config->alu_add_instruction, i_gp_reg_mapping->gp_reg_kloop, i_k_blocking);
 }
 
+LIBXSMM_INTERNAL_API_DEFINITION
 void libxsmm_generator_gemm_footer_kloop( libxsmm_generated_code*             io_generated_code,
-                                           libxsmm_loop_label_tracker*         io_loop_label_tracker,
-                                           const libxsmm_gp_reg_mapping*       i_gp_reg_mapping,
-                                           const libxsmm_micro_kernel_config*  i_micro_kernel_config,
+                                           libxsmm_loop_label_tracker*        io_loop_label_tracker,
+                                           const libxsmm_gp_reg_mapping*      i_gp_reg_mapping,
+                                           const libxsmm_micro_kernel_config* i_micro_kernel_config,
                                            const libxsmm_gemm_descriptor*     i_xgemm_desc,
-                                           const unsigned int                  i_m_blocking,
-                                           const unsigned int                  i_max_blocked_k,
-                                           const unsigned int                  i_kloop_complete ) {
+                                           const unsigned int                 i_m_blocking,
+                                           const unsigned int                 i_max_blocked_k,
+                                           const unsigned int                 i_kloop_complete ) {
   LIBXSMM_UNUSED(i_m_blocking);
   libxsmm_x86_instruction_alu_imm( io_generated_code, i_micro_kernel_config->alu_cmp_instruction, i_gp_reg_mapping->gp_reg_kloop, i_max_blocked_k );
   libxsmm_x86_instruction_jump_back_to_label( io_generated_code, i_micro_kernel_config->alu_jmp_instruction, io_loop_label_tracker );
@@ -482,26 +525,35 @@ void libxsmm_generator_gemm_footer_kloop( libxsmm_generated_code*             io
   }
 }
 
+LIBXSMM_INTERNAL_API_DEFINITION
 void libxsmm_generator_gemm_header_nloop( libxsmm_generated_code*             io_generated_code,
-                                           libxsmm_loop_label_tracker*         io_loop_label_tracker,
-                                           const libxsmm_gp_reg_mapping*       i_gp_reg_mapping,
-                                           const libxsmm_micro_kernel_config*  i_micro_kernel_config,
-                                           const unsigned int                  i_n_blocking) {
+                                           libxsmm_loop_label_tracker*        io_loop_label_tracker,
+                                           const libxsmm_gp_reg_mapping*      i_gp_reg_mapping,
+                                           const libxsmm_micro_kernel_config* i_micro_kernel_config,
+                                           const unsigned int                 i_n_blocking) {
   libxsmm_x86_instruction_register_jump_label( io_generated_code, io_loop_label_tracker );
   libxsmm_x86_instruction_alu_imm( io_generated_code, i_micro_kernel_config->alu_add_instruction, i_gp_reg_mapping->gp_reg_nloop, i_n_blocking );
   libxsmm_x86_instruction_alu_imm( io_generated_code, i_micro_kernel_config->alu_mov_instruction, i_gp_reg_mapping->gp_reg_mloop, 0 );
 }
 
 
+LIBXSMM_INTERNAL_API_DEFINITION
 void libxsmm_generator_gemm_footer_nloop( libxsmm_generated_code*             io_generated_code,
-                                           libxsmm_loop_label_tracker*         io_loop_label_tracker,
-                                           const libxsmm_gp_reg_mapping*       i_gp_reg_mapping,
-                                           const libxsmm_micro_kernel_config*  i_micro_kernel_config,
+                                           libxsmm_loop_label_tracker*        io_loop_label_tracker,
+                                           const libxsmm_gp_reg_mapping*      i_gp_reg_mapping,
+                                           const libxsmm_micro_kernel_config* i_micro_kernel_config,
                                            const libxsmm_gemm_descriptor*     i_xgemm_desc,
-                                           const unsigned int                  i_n_blocking,
-                                           const unsigned int                  i_n_done ) {
+                                           const unsigned int                 i_n_blocking,
+                                           const unsigned int                 i_n_done ) {
   libxsmm_x86_instruction_alu_imm( io_generated_code, i_micro_kernel_config->alu_add_instruction, i_gp_reg_mapping->gp_reg_c,
                                (i_n_blocking*(i_xgemm_desc->ldc)*(i_micro_kernel_config->datatype_size)) - ((i_xgemm_desc->m)*(i_micro_kernel_config->datatype_size)) );
+#if 0
+  if ( i_xgemm_desc->prefetch == LIBXSMM_PREFETCH_CL2 ||
+       i_xgemm_desc->prefetch == LIBXSMM_PREFETCH_AL2CL2BL2_VIA_C ) {
+    libxsmm_x86_instruction_alu_imm( io_generated_code, i_micro_kernel_config->alu_add_instruction, i_gp_reg_mapping->gp_reg_c_prefetch,
+                                     (i_n_blocking*(i_xgemm_desc->ldc)*(i_micro_kernel_config->datatype_size)) - ((i_xgemm_desc->m)*(i_micro_kernel_config->datatype_size)) );
+  }
+#endif
   libxsmm_x86_instruction_alu_imm( io_generated_code, i_micro_kernel_config->alu_add_instruction,
                                i_gp_reg_mapping->gp_reg_b, (i_n_blocking*(i_xgemm_desc->ldb)*(i_micro_kernel_config->datatype_size)) );
   libxsmm_x86_instruction_alu_imm( io_generated_code, i_micro_kernel_config->alu_sub_instruction,
@@ -510,27 +562,39 @@ void libxsmm_generator_gemm_footer_nloop( libxsmm_generated_code*             io
   libxsmm_x86_instruction_jump_back_to_label( io_generated_code, i_micro_kernel_config->alu_jmp_instruction, io_loop_label_tracker );
 }
 
-
+LIBXSMM_INTERNAL_API_DEFINITION
 void libxsmm_generator_gemm_header_mloop( libxsmm_generated_code*             io_generated_code,
-                                           libxsmm_loop_label_tracker*         io_loop_label_tracker,
-                                           const libxsmm_gp_reg_mapping*       i_gp_reg_mapping,
-                                           const libxsmm_micro_kernel_config*  i_micro_kernel_config,
-                                           const unsigned int                  i_m_blocking ) {
+                                           libxsmm_loop_label_tracker*        io_loop_label_tracker,
+                                           const libxsmm_gp_reg_mapping*      i_gp_reg_mapping,
+                                           const libxsmm_micro_kernel_config* i_micro_kernel_config,
+                                           const unsigned int                 i_m_blocking ) {
   libxsmm_x86_instruction_register_jump_label( io_generated_code, io_loop_label_tracker );
   libxsmm_x86_instruction_alu_imm( io_generated_code, i_micro_kernel_config->alu_add_instruction, i_gp_reg_mapping->gp_reg_mloop, i_m_blocking );
 }
 
+LIBXSMM_INTERNAL_API_DEFINITION
 void libxsmm_generator_gemm_footer_mloop( libxsmm_generated_code*             io_generated_code,
-                                           libxsmm_loop_label_tracker*         io_loop_label_tracker,
-                                           const libxsmm_gp_reg_mapping*       i_gp_reg_mapping,
-                                           const libxsmm_micro_kernel_config*  i_micro_kernel_config,
+                                           libxsmm_loop_label_tracker*        io_loop_label_tracker,
+                                           const libxsmm_gp_reg_mapping*      i_gp_reg_mapping,
+                                           const libxsmm_micro_kernel_config* i_micro_kernel_config,
                                            const libxsmm_gemm_descriptor*     i_xgemm_desc,
-                                           const unsigned int                  i_m_blocking,
-                                           const unsigned int                  i_m_done,
-                                           const unsigned int                  i_k_unrolled ) {
+                                           const unsigned int                 i_m_blocking,
+                                           const unsigned int                 i_m_done,
+                                           const unsigned int                 i_k_unrolled ) {
   /* advance C pointer */
   libxsmm_x86_instruction_alu_imm( io_generated_code, i_micro_kernel_config->alu_add_instruction,
                                i_gp_reg_mapping->gp_reg_c, i_m_blocking*(i_micro_kernel_config->datatype_size) );
+
+  /* C prefetch */
+#if 0
+  if ( i_xgemm_desc->prefetch == LIBXSMM_PREFETCH_CL2 ||
+       i_xgemm_desc->prefetch == LIBXSMM_PREFETCH_AL2CL2BL2_VIA_C ) {
+    libxsmm_x86_instruction_alu_imm( io_generated_code, i_micro_kernel_config->alu_add_instruction,
+                                     i_gp_reg_mapping->gp_reg_c_prefetch, i_m_blocking*(i_micro_kernel_config->datatype_size) );
+
+  }
+#endif
+
   /* B prefetch */
   if ( i_xgemm_desc->prefetch == LIBXSMM_PREFETCH_BL2_VIA_C ||
        i_xgemm_desc->prefetch == LIBXSMM_PREFETCH_AL2BL2_VIA_C ||
@@ -568,12 +632,13 @@ void libxsmm_generator_gemm_footer_mloop( libxsmm_generated_code*             io
   libxsmm_x86_instruction_jump_back_to_label( io_generated_code, i_micro_kernel_config->alu_jmp_instruction, io_loop_label_tracker );
 }
 
+LIBXSMM_INTERNAL_API_DEFINITION
 void libxsmm_generator_gemm_load_C( libxsmm_generated_code*             io_generated_code,
-                                     const libxsmm_gp_reg_mapping*       i_gp_reg_mapping,
-                                     const libxsmm_micro_kernel_config*  i_micro_kernel_config,
+                                     const libxsmm_gp_reg_mapping*      i_gp_reg_mapping,
+                                     const libxsmm_micro_kernel_config* i_micro_kernel_config,
                                      const libxsmm_gemm_descriptor*     i_xgemm_desc,
-                                     const unsigned int                  i_m_blocking,
-                                     const unsigned int                  i_n_blocking ) {
+                                     const unsigned int                 i_m_blocking,
+                                     const unsigned int                 i_n_blocking ) {
   /* deriving register blocking from kernel config */
   const unsigned int l_m_blocking = i_m_blocking/i_micro_kernel_config->vector_length;
   /* start register of accumulator */
@@ -596,11 +661,16 @@ void libxsmm_generator_gemm_load_C( libxsmm_generated_code*             io_gener
     }
   } else if (i_micro_kernel_config->instruction_set == LIBXSMM_X86_IMCI        ||
              i_micro_kernel_config->instruction_set == LIBXSMM_X86_AVX512_MIC  ||
-             i_micro_kernel_config->instruction_set == LIBXSMM_X86_AVX512_CORE    ) {
+             ( (i_micro_kernel_config->instruction_set == LIBXSMM_X86_AVX512_CORE) && (i_m_blocking == i_micro_kernel_config->vector_length) )  ) {
     if ( (i_n_blocking > 30) || (i_n_blocking < 1) || (i_m_blocking != i_micro_kernel_config->vector_length) ) {
       libxsmm_handle_error( io_generated_code, LIBXSMM_ERR_REG_BLOCK );
       return;
     }
+  } else if ( i_micro_kernel_config->instruction_set == LIBXSMM_X86_AVX512_CORE ) {
+    if ( (i_n_blocking > 6) || (i_n_blocking < 1) || (i_m_blocking < 1) ) {
+      libxsmm_handle_error( io_generated_code, LIBXSMM_ERR_REG_BLOCK );
+      return;
+    }
   } else {}
   if ( i_m_blocking % i_micro_kernel_config->vector_length != 0 ) {
     libxsmm_handle_error( io_generated_code, LIBXSMM_ERR_M_BLOCK );
@@ -622,6 +692,18 @@ void libxsmm_generator_gemm_load_C( libxsmm_generated_code*             io_gener
                                       i_micro_kernel_config->vector_name,
                                       l_vec_reg_acc_start + l_m + (l_m_blocking * l_n), i_micro_kernel_config->use_masking_a_c, 0 );
       }
+#if 0
+      if ( i_xgemm_desc->prefetch == LIBXSMM_PREFETCH_CL2 ||
+           i_xgemm_desc->prefetch == LIBXSMM_PREFETCH_AL2CL2BL2_VIA_C )  {
+        for (l_m = 0; l_m < l_m_blocking; l_m += l_m++ ) {
+          libxsmm_x86_instruction_prefetch( io_generated_code,
+                                            i_micro_kernel_config->prefetch_instruction,
+                                            i_gp_reg_mapping->gp_reg_c_prefetch,
+                                            LIBXSMM_X86_GP_REG_UNDEF, 0,
+                                            ((l_n * i_xgemm_desc->ldc) + (l_m * (i_micro_kernel_config->vector_length))) * (i_micro_kernel_config->datatype_size));
+        }
+      }
+#endif
     }
   } else {
     /* overwriting C, so let's xout accumulator */
@@ -635,16 +717,29 @@ void libxsmm_generator_gemm_load_C( libxsmm_generated_code*             io_gener
                                              l_vec_reg_acc_start + l_m + (l_m_blocking * l_n),
                                              l_vec_reg_acc_start + l_m + (l_m_blocking * l_n) );
       }
+#if 0
+      if ( i_xgemm_desc->prefetch == LIBXSMM_PREFETCH_CL2 ||
+           i_xgemm_desc->prefetch == LIBXSMM_PREFETCH_AL2CL2BL2_VIA_C )  {
+        for (l_m = 0; l_m < l_m_blocking; l_m += l_m++ ) {
+          libxsmm_x86_instruction_prefetch( io_generated_code,
+                                            i_micro_kernel_config->prefetch_instruction,
+                                            i_gp_reg_mapping->gp_reg_c_prefetch,
+                                            LIBXSMM_X86_GP_REG_UNDEF, 0,
+                                            ((l_n * i_xgemm_desc->ldc) + (l_m * (i_micro_kernel_config->vector_length))) * (i_micro_kernel_config->datatype_size));
+        }
+      }
+#endif
     }
   }
 }
 
+LIBXSMM_INTERNAL_API_DEFINITION
 void libxsmm_generator_gemm_store_C( libxsmm_generated_code*             io_generated_code,
-                                      const libxsmm_gp_reg_mapping*       i_gp_reg_mapping,
-                                      const libxsmm_micro_kernel_config*  i_micro_kernel_config,
+                                      const libxsmm_gp_reg_mapping*      i_gp_reg_mapping,
+                                      const libxsmm_micro_kernel_config* i_micro_kernel_config,
                                       const libxsmm_gemm_descriptor*     i_xgemm_desc,
-                                      const unsigned int                  i_m_blocking,
-                                      const unsigned int                  i_n_blocking )
+                                      const unsigned int                 i_m_blocking,
+                                      const unsigned int                 i_n_blocking )
 {
   /* deriving register blocking from kernel config */
   unsigned int l_m_blocking = i_m_blocking/i_micro_kernel_config->vector_length;
@@ -666,11 +761,16 @@ void libxsmm_generator_gemm_store_C( libxsmm_generated_code*             io_gene
     }
   } else if (i_micro_kernel_config->instruction_set == LIBXSMM_X86_IMCI        ||
              i_micro_kernel_config->instruction_set == LIBXSMM_X86_AVX512_MIC  ||
-             i_micro_kernel_config->instruction_set == LIBXSMM_X86_AVX512_CORE    ) {
+             ( (i_micro_kernel_config->instruction_set == LIBXSMM_X86_AVX512_CORE) && (i_m_blocking == i_micro_kernel_config->vector_length) )  ) {
     if ( (i_n_blocking > 30) || (i_n_blocking < 1) || (i_m_blocking != i_micro_kernel_config->vector_length) ) {
       libxsmm_handle_error( io_generated_code, LIBXSMM_ERR_REG_BLOCK );
       return;
     }
+  } else if ( i_micro_kernel_config->instruction_set == LIBXSMM_X86_AVX512_CORE ) {
+    if ( (i_n_blocking > 6) || (i_n_blocking < 1) || (i_m_blocking < 1) ) {
+      libxsmm_handle_error( io_generated_code, LIBXSMM_ERR_REG_BLOCK );
+      return;
+    }
   } else {}
   if ( i_m_blocking % i_micro_kernel_config->vector_length != 0 ) {
     libxsmm_handle_error( io_generated_code, LIBXSMM_ERR_M_BLOCK );
