@@ -1,5 +1,5 @@
 /******************************************************************************
-** Copyright (c) 2015-2016, Intel Corporation                                **
+** Copyright (c) 2015-2017, Intel Corporation                                **
 ** All rights reserved.                                                      **
 **                                                                           **
 ** Redistribution and use in source and binary forms, with or without        **
@@ -36,17 +36,17 @@
 #include "generator_x86_instructions.h"
 #include "generator_common.h"
 
-#include <libxsmm_macros.h>
+#include <libxsmm_intrinsics_x86.h>
 #include <string.h>
 #include <stdlib.h>
 #include <stdio.h>
 
-
-LIBXSMM_INLINE void libxsmm_generator_gemm_imci_avx512_kernel_initialize_mask( libxsmm_generated_code*            io_generated_code,
-                                                                 const libxsmm_gp_reg_mapping*      i_gp_reg_mapping,
-                                                                 const libxsmm_micro_kernel_config* i_micro_kernel_config,
-                                                                 const libxsmm_gemm_descriptor*    i_xgemm_desc,
-                                                                 unsigned int                       i_m_done ) {
+LIBXSMM_INLINE
+void libxsmm_generator_gemm_imci_avx512_kernel_initialize_mask( libxsmm_generated_code*            io_generated_code,
+                                                                const libxsmm_gp_reg_mapping*      i_gp_reg_mapping,
+                                                                const libxsmm_micro_kernel_config* i_micro_kernel_config,
+                                                                const libxsmm_gemm_descriptor*     i_xgemm_desc,
+                                                                unsigned int                       i_m_done ) {
   unsigned int l_mask;
 
   /* init full mask */
@@ -78,20 +78,21 @@ LIBXSMM_INLINE void libxsmm_generator_gemm_imci_avx512_kernel_initialize_mask( l
   } else {}
 }
 
-LIBXSMM_INLINE void libxsmm_generator_gemm_imci_avx512_kernel_mloop( libxsmm_generated_code*            io_generated_code,
-                                                       libxsmm_loop_label_tracker*        io_loop_label_tracker,
-                                                       const libxsmm_gp_reg_mapping*      i_gp_reg_mapping,
-                                                       const libxsmm_micro_kernel_config* i_micro_kernel_config,
-                                                       const libxsmm_gemm_descriptor*    i_xgemm_desc,
-                                                       const char*                        i_arch,
-                                                       unsigned int                       i_n_blocking ) {
+LIBXSMM_INLINE
+void libxsmm_generator_gemm_imci_avx512_kernel_mloop( libxsmm_generated_code*            io_generated_code,
+                                                      libxsmm_loop_label_tracker*        io_loop_label_tracker,
+                                                      const libxsmm_gp_reg_mapping*      i_gp_reg_mapping,
+                                                      const libxsmm_micro_kernel_config* i_micro_kernel_config,
+                                                      const libxsmm_gemm_descriptor*     i_xgemm_desc,
+                                                      const char*                        i_arch,
+                                                      unsigned int                       i_n_blocking ) {
   /* set function pointers for AVX512 and IMCI */
   unsigned int (*l_generator_microkernel_kloop)( libxsmm_generated_code*, libxsmm_loop_label_tracker*, const libxsmm_gp_reg_mapping*, const libxsmm_micro_kernel_config*,
-                                                 const libxsmm_gemm_descriptor*, const char*, unsigned int );
+                                                 const libxsmm_gemm_descriptor*, const char*, unsigned int ) = 0;
   void (*l_generator_load)( libxsmm_generated_code*, const libxsmm_gp_reg_mapping*, const libxsmm_micro_kernel_config*,
-                            const libxsmm_gemm_descriptor*, const unsigned int, const unsigned int );
+                            const libxsmm_gemm_descriptor*, const unsigned int, const unsigned int ) = 0;
   void (*l_generator_store)( libxsmm_generated_code*, const libxsmm_gp_reg_mapping*, const libxsmm_micro_kernel_config*,
-                            const libxsmm_gemm_descriptor*, const unsigned int, const unsigned int );
+                            const libxsmm_gemm_descriptor*, const unsigned int, const unsigned int ) = 0;
   unsigned int l_k_unrolled;
   unsigned int l_m_done;
 
@@ -108,8 +109,8 @@ LIBXSMM_INLINE void libxsmm_generator_gemm_imci_avx512_kernel_mloop( libxsmm_gen
     l_generator_load = libxsmm_generator_gemm_load_C_imci;
     l_generator_store = libxsmm_generator_gemm_store_C_imci;
   } else {
-    fprintf(stderr, "LIBXSMM ERROR libxsmm_generator_gemm_imci_avx512_kernel_mloop, cannot select microkernel\n");
-    exit(-1);
+    libxsmm_handle_error( io_generated_code, LIBXSMM_ERR_ARCH );
+    return;
   }
 
   /* we proceed as much as we can in vector length steps, remainder is handled uisng masking */
@@ -122,14 +123,14 @@ LIBXSMM_INLINE void libxsmm_generator_gemm_imci_avx512_kernel_mloop( libxsmm_gen
                       i_xgemm_desc, i_micro_kernel_config->vector_length, i_n_blocking );
 
     /* if we are generating for KNL && i_n_blocking is greater 26 && we prefetch via C -> push prefetch gpr */
-    if ( (i_n_blocking > 26)          && 
+    if ( (i_n_blocking > 26)          &&
          (strcmp(i_arch, "knc") != 0) &&
          (i_xgemm_desc->prefetch == LIBXSMM_PREFETCH_BL2_VIA_C ||
           i_xgemm_desc->prefetch == LIBXSMM_PREFETCH_AL2BL2_VIA_C ||
           i_xgemm_desc->prefetch == LIBXSMM_PREFETCH_AL2BL2_VIA_C_AHEAD ||
           i_xgemm_desc->prefetch == LIBXSMM_PREFETCH_AL2BL2_VIA_C_JPST ) ) {
       libxsmm_x86_instruction_push_reg( io_generated_code, i_gp_reg_mapping->gp_reg_b_prefetch );
-    }                             
+    }
 
     l_k_unrolled = l_generator_microkernel_kloop( io_generated_code,
                                                   io_loop_label_tracker,
@@ -140,14 +141,14 @@ LIBXSMM_INLINE void libxsmm_generator_gemm_imci_avx512_kernel_mloop( libxsmm_gen
                                                   i_n_blocking );
 
     /* if we are generating for KNL && i_n_blocking is greater 26 && we prefetch via C -> push prefetch gpr */
-    if ( (i_n_blocking > 26)          && 
+    if ( (i_n_blocking > 26)          &&
          (strcmp(i_arch, "knc") != 0) &&
          (i_xgemm_desc->prefetch == LIBXSMM_PREFETCH_BL2_VIA_C ||
           i_xgemm_desc->prefetch == LIBXSMM_PREFETCH_AL2BL2_VIA_C ||
           i_xgemm_desc->prefetch == LIBXSMM_PREFETCH_AL2BL2_VIA_C_AHEAD ||
           i_xgemm_desc->prefetch == LIBXSMM_PREFETCH_AL2BL2_VIA_C_JPST ) ) {
       libxsmm_x86_instruction_pop_reg( io_generated_code, i_gp_reg_mapping->gp_reg_b_prefetch );
-    }   
+    }
 
     l_generator_store( io_generated_code, i_gp_reg_mapping, i_micro_kernel_config,
                        i_xgemm_desc, i_micro_kernel_config->vector_length, i_n_blocking  );
@@ -156,7 +157,7 @@ LIBXSMM_INLINE void libxsmm_generator_gemm_imci_avx512_kernel_mloop( libxsmm_gen
   }
 
   /* Remainder Handling using Masking, we are using M loop counter register as GP register for the mask */
-  if ( l_m_done != i_xgemm_desc->m ) {
+  if ( l_m_done != (unsigned int)i_xgemm_desc->m ) {
     /* request masking support, @TODO performance penality here, as a new object is created */
     libxsmm_micro_kernel_config l_micro_kernel_config_mask;
     libxsmm_generator_gemm_init_micro_kernel_config_fullvector( &l_micro_kernel_config_mask, i_xgemm_desc, i_arch, 1 );
@@ -173,14 +174,14 @@ LIBXSMM_INLINE void libxsmm_generator_gemm_imci_avx512_kernel_mloop( libxsmm_gen
                       i_xgemm_desc, l_micro_kernel_config_mask.vector_length, i_n_blocking );
 
     /* if we are generating for KNL && i_n_blocking is greater 26 && we prefetch via C -> push prefetch gpr */
-    if ( (i_n_blocking > 26)          && 
+    if ( (i_n_blocking > 26)          &&
          (strcmp(i_arch, "knc") != 0) &&
          (i_xgemm_desc->prefetch == LIBXSMM_PREFETCH_BL2_VIA_C ||
           i_xgemm_desc->prefetch == LIBXSMM_PREFETCH_AL2BL2_VIA_C ||
           i_xgemm_desc->prefetch == LIBXSMM_PREFETCH_AL2BL2_VIA_C_AHEAD ||
           i_xgemm_desc->prefetch == LIBXSMM_PREFETCH_AL2BL2_VIA_C_JPST ) ) {
       libxsmm_x86_instruction_push_reg( io_generated_code, i_gp_reg_mapping->gp_reg_b_prefetch );
-    }  
+    }
 
     l_k_unrolled = l_generator_microkernel_kloop( io_generated_code,
                                                   io_loop_label_tracker,
@@ -191,14 +192,14 @@ LIBXSMM_INLINE void libxsmm_generator_gemm_imci_avx512_kernel_mloop( libxsmm_gen
                                                   i_n_blocking );
 
     /* if we are generating for KNL && i_n_blocking is greater 26 && we prefetch via C -> push prefetch gpr */
-    if ( (i_n_blocking > 26)          && 
+    if ( (i_n_blocking > 26)          &&
          (strcmp(i_arch, "knc") != 0) &&
          (i_xgemm_desc->prefetch == LIBXSMM_PREFETCH_BL2_VIA_C ||
           i_xgemm_desc->prefetch == LIBXSMM_PREFETCH_AL2BL2_VIA_C ||
           i_xgemm_desc->prefetch == LIBXSMM_PREFETCH_AL2BL2_VIA_C_AHEAD ||
           i_xgemm_desc->prefetch == LIBXSMM_PREFETCH_AL2BL2_VIA_C_JPST ) ) {
       libxsmm_x86_instruction_pop_reg( io_generated_code, i_gp_reg_mapping->gp_reg_b_prefetch );
-    }  
+    }
 
     l_generator_store( io_generated_code, i_gp_reg_mapping, &l_micro_kernel_config_mask,
                        i_xgemm_desc, l_micro_kernel_config_mask.vector_length, i_n_blocking  );
@@ -224,9 +225,10 @@ LIBXSMM_INLINE void libxsmm_generator_gemm_imci_avx512_kernel_mloop( libxsmm_gen
   }
 }
 
+LIBXSMM_INTERNAL_API_DEFINITION
 void libxsmm_generator_gemm_imci_avx512_kernel( libxsmm_generated_code*         io_generated_code,
                                                  const libxsmm_gemm_descriptor* i_xgemm_desc,
-                                                 const char*                     i_arch ) {
+                                                 const char*                    i_arch ) {
   libxsmm_micro_kernel_config l_micro_kernel_config;
   libxsmm_loop_label_tracker l_loop_label_tracker;
   libxsmm_gp_reg_mapping l_gp_reg_mapping;
@@ -241,7 +243,7 @@ void libxsmm_generator_gemm_imci_avx512_kernel( libxsmm_generated_code*
 
   /* define gp register mapping */
   libxsmm_reset_x86_gp_reg_mapping( &l_gp_reg_mapping );
-  /* machting calling convention on Linux */
+  /* matching calling convention on Linux */
   l_gp_reg_mapping.gp_reg_a = LIBXSMM_X86_GP_REG_RDI;
   l_gp_reg_mapping.gp_reg_b = LIBXSMM_X86_GP_REG_RSI;
   l_gp_reg_mapping.gp_reg_c = LIBXSMM_X86_GP_REG_RDX;
