@@ -1,5 +1,5 @@
 /******************************************************************************
-** Copyright (c) 2015-2016, Intel Corporation                                **
+** Copyright (c) 2015-2017, Intel Corporation                                **
 ** All rights reserved.                                                      **
 **                                                                           **
 ** Redistribution and use in source and binary forms, with or without        **
@@ -35,19 +35,22 @@
 #include "generator_spgemm_csc_asparse.h"
 #include "generator_spgemm_csc_bsparse.h"
 #include "generator_spgemm_csr_asparse.h"
+#include "generator_spgemm_csr_asparse_reg.h"
+#include "generator_spgemm_csr_bsparse_soa.h"
+#include "generator_spgemm_csr_asparse_soa.h"
 
 #include <stdlib.h>
 #include <string.h>
 #include <assert.h>
 #include <stdio.h>
 
-
-void libxsmm_generator_spgemm_csc_kernel( libxsmm_generated_code*         io_generated_code,
-                                          const libxsmm_gemm_descriptor*  i_xgemm_desc,
-                                          const char*                     i_arch,
-                                          const unsigned int*             i_row_idx,
-                                          const unsigned int*             i_column_idx,
-                                          const double*                   i_values ) {
+LIBXSMM_INTERNAL_API_DEFINITION
+void libxsmm_generator_spgemm_csc_kernel( libxsmm_generated_code*        io_generated_code,
+                                          const libxsmm_gemm_descriptor* i_xgemm_desc,
+                                          const char*                    i_arch,
+                                          const unsigned int*            i_row_idx,
+                                          const unsigned int*            i_column_idx,
+                                          const double*                  i_values ) {
   /* A matrix is sparse */
   if ( (i_xgemm_desc->lda == 0) && (i_xgemm_desc->ldb > 0) && (i_xgemm_desc->ldc > 0) ) {
     /* check LDB */
@@ -81,12 +84,13 @@ void libxsmm_generator_spgemm_csc_kernel( libxsmm_generated_code*         io_gen
   }
 }
 
-void libxsmm_generator_spgemm_csr_kernel( libxsmm_generated_code*         io_generated_code,
-                                          const libxsmm_gemm_descriptor*  i_xgemm_desc,
-                                          const char*                     i_arch,
-                                          const unsigned int*             i_row_idx,
-                                          const unsigned int*             i_column_idx,
-                                          const double*                   i_values ) {
+LIBXSMM_INTERNAL_API_DEFINITION
+void libxsmm_generator_spgemm_csr_kernel( libxsmm_generated_code*        io_generated_code,
+                                          const libxsmm_gemm_descriptor* i_xgemm_desc,
+                                          const char*                    i_arch,
+                                          const unsigned int*            i_row_idx,
+                                          const unsigned int*            i_column_idx,
+                                          const double*                  i_values ) {
   /* A matrix is sparse */
   if ( (i_xgemm_desc->lda == 0) && (i_xgemm_desc->ldb > 0) && (i_xgemm_desc->ldc > 0) ) {
     /* check LDB */
@@ -122,12 +126,96 @@ void libxsmm_generator_spgemm_csr_kernel( libxsmm_generated_code*         io_gen
   }
 }
 
-void libxsmm_generator_spgemm( const char*                     i_file_out,
-                               const char*                     i_routine_name,
-                               const libxsmm_gemm_descriptor*  i_xgemm_desc,
-                               const char*                     i_arch,
-                               const char*                     i_file_in,
-                               const int                       i_is_csr        ) {
+LIBXSMM_INTERNAL_API_DEFINITION
+void libxsmm_generator_spgemm_csr_reg_kernel( libxsmm_generated_code*        io_generated_code,
+                                              const libxsmm_gemm_descriptor* i_xgemm_desc,
+                                              const char*                    i_arch,
+                                              const unsigned int*            i_row_idx,
+                                              const unsigned int*            i_column_idx,
+                                              const double*                  i_values ) {
+  /* A matrix is sparse */
+  if ( (i_xgemm_desc->lda == 0) && (i_xgemm_desc->ldb > 0) && (i_xgemm_desc->ldc > 0) ) {
+    /* check LDB */
+    if ( i_xgemm_desc->ldb < i_xgemm_desc->n ) {
+      libxsmm_handle_error( io_generated_code, LIBXSMM_ERR_LDB );
+      return;
+    }
+    /* check LDC */
+    if ( i_xgemm_desc->ldc < i_xgemm_desc->n ) {
+      libxsmm_handle_error( io_generated_code, LIBXSMM_ERR_LDC );
+      return;
+    }
+    libxsmm_generator_spgemm_csr_asparse_reg( io_generated_code, i_xgemm_desc, i_arch, i_row_idx, i_column_idx, i_values );
+  /* B matrix is sparse */
+  } else if ( (i_xgemm_desc->lda > 0) && (i_xgemm_desc->ldb == 0) && (i_xgemm_desc->ldc > 0) ) {
+    /* check LDA */
+    if ( i_xgemm_desc->lda < i_xgemm_desc->k ) {
+      libxsmm_handle_error( io_generated_code, LIBXSMM_ERR_LDA );
+      return;
+    }
+    /* check LDC */
+    if ( i_xgemm_desc->ldc < i_xgemm_desc->n ) {
+      libxsmm_handle_error( io_generated_code, LIBXSMM_ERR_LDC );
+      return;
+    }
+    /* something bad happened... */
+    fprintf( stderr, "LIBXSMM ERROR, B sparse for CSR datastructure is not yet available, FATAL ERROR, EXTI!\n");
+    exit(-1);
+  } else {
+    /* something bad happened... */
+    libxsmm_handle_error( io_generated_code, LIBXSMM_ERR_SPGEMM_GEN );
+    return;
+  }
+}
+
+LIBXSMM_INTERNAL_API_DEFINITION
+void libxsmm_generator_spgemm_csr_soa_kernel( libxsmm_generated_code*        io_generated_code,
+                                              const libxsmm_gemm_descriptor* i_xgemm_desc,
+                                              const char*                    i_arch,
+                                              const unsigned int*            i_row_idx,
+                                              const unsigned int*            i_column_idx,
+                                              const double*                  i_values ) {
+  /* A matrix is sparse */
+  if ( (i_xgemm_desc->lda == 0) && (i_xgemm_desc->ldb > 0) && (i_xgemm_desc->ldc > 0) ) {
+    /* check LDB */
+    if ( i_xgemm_desc->ldb < i_xgemm_desc->n ) {
+      libxsmm_handle_error( io_generated_code, LIBXSMM_ERR_LDB );
+      return;
+    }
+    /* check LDC */
+    if ( i_xgemm_desc->ldc < i_xgemm_desc->n ) {
+      libxsmm_handle_error( io_generated_code, LIBXSMM_ERR_LDC );
+      return;
+    }
+    /* something bad happened... */
+    libxsmm_generator_spgemm_csr_asparse_soa( io_generated_code, i_xgemm_desc, i_arch, i_row_idx, i_column_idx, i_values );
+  /* B matrix is sparse */
+  } else if ( (i_xgemm_desc->lda > 0) && (i_xgemm_desc->ldb == 0) && (i_xgemm_desc->ldc > 0) ) {
+    /* check LDA */
+    if ( i_xgemm_desc->lda < i_xgemm_desc->k ) {
+      libxsmm_handle_error( io_generated_code, LIBXSMM_ERR_LDA );
+      return;
+    }
+    /* check LDC */
+    if ( i_xgemm_desc->ldc < i_xgemm_desc->n ) {
+      libxsmm_handle_error( io_generated_code, LIBXSMM_ERR_LDC );
+      return;
+    }
+    libxsmm_generator_spgemm_csr_bsparse_soa( io_generated_code, i_xgemm_desc, i_arch, i_row_idx, i_column_idx, i_values );
+  } else {
+    /* something bad happened... */
+    libxsmm_handle_error( io_generated_code, LIBXSMM_ERR_SPGEMM_GEN );
+    return;
+  }
+}
+
+LIBXSMM_INTERNAL_API_DEFINITION
+void libxsmm_generator_spgemm( const char*                    i_file_out,
+                               const char*                    i_routine_name,
+                               const libxsmm_gemm_descriptor* i_xgemm_desc,
+                               const char*                    i_arch,
+                               const char*                    i_file_in,
+                               const int                      i_is_csr        ) {
   /* CSC/CSR structure */
   unsigned int* l_row_idx = NULL;
   unsigned int* l_column_idx = NULL;
@@ -145,7 +233,11 @@ void libxsmm_generator_spgemm( const char*                     i_file_out,
   l_generated_code.last_error = 0;
 
   /* add signature to code string */
-  libxsmm_mmfunction_signature( &l_generated_code, i_routine_name, i_xgemm_desc );
+  if (i_is_csr == 3) {
+    libxsmm_mmfunction_signature_asparse_reg( &l_generated_code, i_routine_name, i_xgemm_desc );
+  } else {
+    libxsmm_mmfunction_signature( &l_generated_code, i_routine_name, i_xgemm_desc );
+  }
 
   /* check if generate to CSC */
   if ( i_is_csr == 0 ) {
@@ -171,11 +263,11 @@ void libxsmm_generator_spgemm( const char*                     i_file_out,
       }
 
       for ( l_n = 0; l_n < l_row_count+1; l_n++) {
-         printf("%i ", l_column_idx[l_n]);
+         printf("%u ", l_column_idx[l_n]);
       }
       printf("\n");
       for ( l_n = 0; l_n < l_element_count; l_n++) {
-         printf("%i ", l_row_idx[l_n]);
+         printf("%u ", l_row_idx[l_n]);
       }
       printf("\n");
       for ( l_n = 0; l_n < l_element_count; l_n++) {
@@ -227,11 +319,11 @@ void libxsmm_generator_spgemm( const char*                     i_file_out,
       }
 
       for ( l_n = 0; l_n < l_row_count+1; l_n++) {
-         printf("%i ", l_row_idx[l_n]);
+         printf("%u ", l_row_idx[l_n]);
       }
       printf("\n");
       for ( l_n = 0; l_n < l_element_count; l_n++) {
-         printf("%i ", l_column_idx[l_n]);
+         printf("%u ", l_column_idx[l_n]);
       }
       printf("\n");
       for ( l_n = 0; l_n < l_element_count; l_n++) {
@@ -258,8 +350,18 @@ void libxsmm_generator_spgemm( const char*                     i_file_out,
       free( l_tmp );
     }
 #endif
-    /* generate the actual kernel code for current description depending on the architecture */
-    libxsmm_generator_spgemm_csr_kernel( &l_generated_code, i_xgemm_desc, i_arch, l_row_idx, l_column_idx, l_values );
+    if (i_is_csr == 1) {
+      /* generate the actual kernel code for current description depending on the architecture */
+      libxsmm_generator_spgemm_csr_kernel( &l_generated_code, i_xgemm_desc, i_arch, l_row_idx, l_column_idx, l_values );
+    } else if (i_is_csr == 2) {
+      /* generate the actual kernel code for current description depending on the architecture */
+      libxsmm_generator_spgemm_csr_soa_kernel( &l_generated_code, i_xgemm_desc, i_arch, l_row_idx, l_column_idx, l_values );
+    } else if (i_is_csr == 3) {
+      /* generate the actual kernel code for current description depending on the architecture */
+      libxsmm_generator_spgemm_csr_reg_kernel( &l_generated_code, i_xgemm_desc, i_arch, l_row_idx, l_column_idx, l_values );
+    } else {
+      /* shouldn't happen */
+    }
   }
 
   /* close current function */
@@ -287,7 +389,7 @@ void libxsmm_generator_spgemm( const char*                     i_file_out,
   {
     FILE *const l_file_handle = fopen( i_file_out, "a" );
     if ( l_file_handle != NULL ) {
-      fputs( l_generated_code.generated_code, l_file_handle );
+      fputs( (const char*)l_generated_code.generated_code, l_file_handle );
       fclose( l_file_handle );
     } else {
       fprintf(stderr, "LIBXSMM ERROR, libxsmm_generator_spgemm: could not write to into destination source file\n");
