@@ -1,5 +1,5 @@
 /******************************************************************************
-** Copyright (c) 2015-2016, Intel Corporation                                **
+** Copyright (c) 2015-2017, Intel Corporation                                **
 ** All rights reserved.                                                      **
 **                                                                           **
 ** Redistribution and use in source and binary forms, with or without        **
@@ -74,13 +74,13 @@
 #include <stdlib.h>
 #include <string.h>
 
-
-void libxsmm_sparse_csc_asparse_innerloop_scalar( libxsmm_generated_code*         io_generated_code,
-                                                  const libxsmm_gemm_descriptor*  i_xgemm_desc,
-                                                  const unsigned int              i_k,
-                                                  const unsigned int              i_z,
-                                                  const unsigned int*             i_row_idx,
-                                                  const unsigned int*             i_column_idx ) {
+LIBXSMM_INTERNAL_API_DEFINITION
+void libxsmm_sparse_csc_asparse_innerloop_scalar( libxsmm_generated_code*        io_generated_code,
+                                                  const libxsmm_gemm_descriptor* i_xgemm_desc,
+                                                  const unsigned int             i_k,
+                                                  const unsigned int             i_z,
+                                                  const unsigned int*            i_row_idx,
+                                                  const unsigned int*            i_column_idx ) {
   char l_new_code[512];
   int l_max_code_length = 511;
   int l_code_length = 0;
@@ -116,12 +116,13 @@ void libxsmm_sparse_csc_asparse_innerloop_scalar( libxsmm_generated_code*
   }
 }
 
-void libxsmm_sparse_csc_asparse_innerloop_two_vector( libxsmm_generated_code*         io_generated_code,
+LIBXSMM_INTERNAL_API_DEFINITION
+void libxsmm_sparse_csc_asparse_innerloop_two_vector( libxsmm_generated_code*        io_generated_code,
                                                       const libxsmm_gemm_descriptor* i_xgemm_desc,
-                                                      const unsigned int              i_k,
-                                                      const unsigned int              i_z,
-                                                      const unsigned int*             i_row_idx,
-                                                      const unsigned int*             i_column_idx ) {
+                                                      const unsigned int             i_k,
+                                                      const unsigned int             i_z,
+                                                      const unsigned int*            i_row_idx,
+                                                      const unsigned int*            i_column_idx ) {
   char l_new_code[512];
   int l_max_code_length = 511;
   int l_code_length = 0;
@@ -157,13 +158,14 @@ void libxsmm_sparse_csc_asparse_innerloop_two_vector( libxsmm_generated_code*
   }
 }
 
-void libxsmm_sparse_csc_asparse_innerloop_four_vector( libxsmm_generated_code*         io_generated_code,
+LIBXSMM_INTERNAL_API_DEFINITION
+void libxsmm_sparse_csc_asparse_innerloop_four_vector( libxsmm_generated_code*        io_generated_code,
                                                        const libxsmm_gemm_descriptor* i_xgemm_desc,
-                                                       const unsigned int              i_k,
-                                                       const unsigned int              i_z,
-                                                       const unsigned int*             i_row_idx,
-                                                       const unsigned int*             i_column_idx ) {
-  char l_new_code[512]; 
+                                                       const unsigned int             i_k,
+                                                       const unsigned int             i_z,
+                                                       const unsigned int*            i_row_idx,
+                                                       const unsigned int*            i_column_idx ) {
+  char l_new_code[512];
   int l_max_code_length = 511;
   int l_code_length = 0;
 
@@ -209,12 +211,13 @@ void libxsmm_sparse_csc_asparse_innerloop_four_vector( libxsmm_generated_code*
   }
 }
 
-void libxsmm_generator_spgemm_csc_asparse( libxsmm_generated_code*         io_generated_code,
+LIBXSMM_INTERNAL_API_DEFINITION
+void libxsmm_generator_spgemm_csc_asparse( libxsmm_generated_code*        io_generated_code,
                                            const libxsmm_gemm_descriptor* i_xgemm_desc,
-                                           const char*                     i_arch,
-                                           const unsigned int*             i_row_idx,
-                                           const unsigned int*             i_column_idx,
-                                           const double*                   i_values ) {
+                                           const char*                    i_arch,
+                                           const unsigned int*            i_row_idx,
+                                           const unsigned int*            i_column_idx,
+                                           const double*                  i_values ) {
   char l_new_code[512];
   int l_max_code_length = 511;
   int l_code_length = 0;
@@ -245,7 +248,7 @@ void libxsmm_generator_spgemm_csc_asparse( libxsmm_generated_code*         io_ge
   }
 
   /* loop over columns in A, rows in B and fully unroll */
-  for ( l_k = 0; l_k < i_xgemm_desc->k; l_k++ ) {
+  for ( l_k = 0; l_k < (unsigned int)i_xgemm_desc->k; l_k++ ) {
     unsigned int l_column_elements = i_column_idx[l_k + 1] - i_column_idx[l_k];
     unsigned int l_z = 0;
 
@@ -274,17 +277,17 @@ void libxsmm_generator_spgemm_csc_asparse( libxsmm_generated_code*         io_ge
         if ((i_row_idx[i_column_idx[l_k] + l_z] + 1 == i_row_idx[i_column_idx[l_k] + l_z + 1]) &&
             (i_row_idx[i_column_idx[l_k] + l_z] + 2 == i_row_idx[i_column_idx[l_k] + l_z + 2]) &&
             (i_row_idx[i_column_idx[l_k] + l_z] + 3 == i_row_idx[i_column_idx[l_k] + l_z + 3]) &&
-            (i_row_idx[i_column_idx[l_k] + l_z + 3] < i_xgemm_desc->m)) {
+            (i_row_idx[i_column_idx[l_k] + l_z + 3] < (unsigned int)i_xgemm_desc->m)) {
           libxsmm_sparse_csc_asparse_innerloop_four_vector(io_generated_code, i_xgemm_desc, l_k, l_z, i_row_idx, i_column_idx);
           l_z += 3;
         /* check for 128bit vector instruction */
         } else if ((i_row_idx[i_column_idx[l_k] + l_z] + 1 == i_row_idx[i_column_idx[l_k] + l_z + 1]) &&
-                   (i_row_idx[i_column_idx[l_k] + l_z + 1] < i_xgemm_desc->m) ) {
+                   (i_row_idx[i_column_idx[l_k] + l_z + 1] < (unsigned int)i_xgemm_desc->m) ) {
           libxsmm_sparse_csc_asparse_innerloop_two_vector(io_generated_code, i_xgemm_desc, l_k, l_z, i_row_idx, i_column_idx);
           l_z++;
         /* scalare instruction */
         } else {
-          if ( (i_row_idx[i_column_idx[l_k] + l_z] < i_xgemm_desc->m) ) {
+          if ( (i_row_idx[i_column_idx[l_k] + l_z] < (unsigned int)i_xgemm_desc->m) ) {
             libxsmm_sparse_csc_asparse_innerloop_scalar(io_generated_code, i_xgemm_desc, l_k, l_z, i_row_idx, i_column_idx);
           }
         }
@@ -292,18 +295,18 @@ void libxsmm_generator_spgemm_csc_asparse( libxsmm_generated_code*         io_ge
       } else if ( (l_z < (l_column_elements - 1)) && (l_column_elements > 1)) {
         /* check for 128bit vector instruction */
         if ((i_row_idx[i_column_idx[l_k] + l_z] + 1 == i_row_idx[i_column_idx[l_k] + l_z + 1]) &&
-            (i_row_idx[i_column_idx[l_k] + l_z + 1] < i_xgemm_desc->m) ) {
+            (i_row_idx[i_column_idx[l_k] + l_z + 1] < (unsigned int)i_xgemm_desc->m) ) {
           libxsmm_sparse_csc_asparse_innerloop_two_vector(io_generated_code, i_xgemm_desc, l_k, l_z, i_row_idx, i_column_idx);
           l_z++;
         /* scalare instruction */
         } else {
-          if ( (i_row_idx[i_column_idx[l_k] + l_z] < i_xgemm_desc->m) ) {
+          if ( (i_row_idx[i_column_idx[l_k] + l_z] < (unsigned int)i_xgemm_desc->m) ) {
             libxsmm_sparse_csc_asparse_innerloop_scalar(io_generated_code, i_xgemm_desc, l_k, l_z, i_row_idx, i_column_idx);
           }
         }
       /* scalar anayways */
       } else {
-        if ( (i_row_idx[i_column_idx[l_k] + l_z] < i_xgemm_desc->m) ) {
+        if ( (i_row_idx[i_column_idx[l_k] + l_z] < (unsigned int)i_xgemm_desc->m) ) {
           libxsmm_sparse_csc_asparse_innerloop_scalar(io_generated_code, i_xgemm_desc, l_k, l_z, i_row_idx, i_column_idx);
         }
       }
@@ -315,7 +318,7 @@ void libxsmm_generator_spgemm_csc_asparse( libxsmm_generated_code*         io_ge
 
     /* loop over the columns of A */
     for ( l_z = 0; l_z < l_column_elements; l_z++ ) {
-      if ( (i_row_idx[i_column_idx[l_k] + l_z] < i_xgemm_desc->m) ) {
+      if ( (i_row_idx[i_column_idx[l_k] + l_z] < (unsigned int)i_xgemm_desc->m) ) {
         l_code_length = LIBXSMM_SNPRINTF(l_new_code, l_max_code_length, "    C[(l_n*%u)+%u] += A[%u] * B[(l_n*%u)+%u];\n", (unsigned int)i_xgemm_desc->ldc, i_row_idx[i_column_idx[l_k] + l_z], i_column_idx[l_k] + l_z, (unsigned int)i_xgemm_desc->ldb, l_k );
         libxsmm_append_code_as_string( io_generated_code, l_new_code, l_code_length );
         l_flop_count += 2;
