@@ -1,5 +1,5 @@
 /******************************************************************************
-** Copyright (c) 2015-2016, Intel Corporation                                **
+** Copyright (c) 2015-2017, Intel Corporation                                **
 ** All rights reserved.                                                      **
 **                                                                           **
 ** Redistribution and use in source and binary forms, with or without        **
@@ -75,6 +75,7 @@
 #include "generator_common.h"
 #include "generator_spgemm_csc_reader.h"
 
+LIBXSMM_INTERNAL_API_DEFINITION
 void libxsmm_sparse_csc_reader( libxsmm_generated_code* io_generated_code,
                                 const char*             i_csc_file_in,
                                 unsigned int**          o_row_idx,
@@ -143,8 +144,8 @@ void libxsmm_sparse_csc_reader( libxsmm_generated_code* io_generated_code,
         }
       /* now we read the actual content */
       } else {
-        unsigned int l_row, l_column;
-        double l_value;
+        unsigned int l_row = 0, l_column = 0;
+        double l_value = 0;
         /* read a line of content */
         if ( sscanf(l_line, "%u %u %lf", &l_row, &l_column, &l_value) != 3 ) {
           libxsmm_handle_error( io_generated_code, LIBXSMM_ERR_CSC_READ_ELEMS );
