@@ -1,5 +1,5 @@
 /******************************************************************************
-** Copyright (c) 2015-2016, Intel Corporation                                **
+** Copyright (c) 2015-2017, Intel Corporation                                **
 ** All rights reserved.                                                      **
 **                                                                           **
 ** Redistribution and use in source and binary forms, with or without        **
@@ -40,8 +40,9 @@
  * @param io_generated_code pointer to the pointer of the generated code structure
  * @param i_gp_reg_mapping gp register mapping for initialization
  * @param i_arch architecture code was generated for (needed to build clobber)
- * @param i_prefetch prefetch mode which may result in addtional gp reg inits
+ * @param i_prefetch prefetch mode which may result in additional gp reg inits
  */
+LIBXSMM_INTERNAL_API
 void libxsmm_x86_instruction_open_stream( libxsmm_generated_code*       io_generated_code,
                                           const libxsmm_gp_reg_mapping* i_gp_reg_mapping,
                                           const char*                   i_arch,
@@ -53,8 +54,9 @@ void libxsmm_x86_instruction_open_stream( libxsmm_generated_code*       io_gener
  * @param io_generated_code pointer to the pointer of the generated code structure
  * @param i_gp_reg_mapping gp register mapping for clobbering
  * @param i_arch architecture code was generated for (needed to build clobber)
- * @param i_prefetch prefetch mode which may result in addtional gp reg clobbers
+ * @param i_prefetch prefetch mode which may result in additional gp reg clobbers
  */
+LIBXSMM_INTERNAL_API
 void libxsmm_x86_instruction_close_stream( libxsmm_generated_code*       io_generated_code,
                                            const libxsmm_gp_reg_mapping* i_gp_reg_mapping,
                                            const char*                   i_arch,
@@ -72,8 +74,9 @@ void libxsmm_x86_instruction_close_stream( libxsmm_generated_code*       io_gene
  * @param i_vector_name the vector register name prefix (x, y or z)
  * @param i_vec_reg_number_0 the vector register number (xmm/ymm: 0-15, zmm: 0-31)
  * @param i_mask_reg_number the mask register to be used
- * @param i_is_store 0: load semantik, other: store semantik
+ * @param i_is_store 0: load semantic, other: store semantic
  */
+LIBXSMM_INTERNAL_API
 void libxsmm_x86_instruction_vec_move( libxsmm_generated_code* io_generated_code,
                                        const unsigned int      i_instruction_set,
                                        const unsigned int      i_vmove_instr,
@@ -97,6 +100,7 @@ void libxsmm_x86_instruction_vec_move( libxsmm_generated_code* io_generated_code
  * @param i_vec_reg_number_1 the second vector register number (xmm/ymm: 0-15, zmm: 0-31)
  * @param i_vec_reg_number_2 the third vector register number (xmm/ymm: 0-15, zmm: 0-31), if this operand equals LIBXSMM_X86_VEC_REG_UNDEF -> SSE3 code generation
  */
+LIBXSMM_INTERNAL_API
 void libxsmm_x86_instruction_vec_compute_reg( libxsmm_generated_code* io_generated_code,
                                               const unsigned int      i_instruction_set,
                                               const unsigned int      i_vec_instr,
@@ -120,6 +124,7 @@ void libxsmm_x86_instruction_vec_compute_reg( libxsmm_generated_code* io_generat
  * @param i_vec_reg_number_0 the first vector register number (zmm: 0-31)
  * @param i_vec_reg_number_1 the second vector register number (zmm: 0-31)
  */
+LIBXSMM_INTERNAL_API
 void libxsmm_x86_instruction_vec_compute_mem( libxsmm_generated_code* io_generated_code,
                                               const unsigned int      i_instruction_set,
                                               const unsigned int      i_vec_instr,
@@ -143,6 +148,7 @@ void libxsmm_x86_instruction_vec_compute_mem( libxsmm_generated_code* io_generat
  * @param i_vec_reg_number_1 the second vector register number (xmm/ymm: 0-15, zmm: 0-31)
  * @param i_vec_reg_number_2 the third vector register number (xmm/ymm: 0-15, zmm: 0-31), if this operand equals LIBXSMM_X86_VEC_REG_UNDEF -> SSE3 code generation
  */
+LIBXSMM_INTERNAL_API
 void libxsmm_x86_instruction_vec_shuffle_reg( libxsmm_generated_code* io_generated_code,
                                               const unsigned int      i_instruction_set,
                                               const unsigned int      i_vec_instr,
@@ -152,6 +158,34 @@ void libxsmm_x86_instruction_vec_shuffle_reg( libxsmm_generated_code* io_generat
                                               const unsigned int      i_vec_reg_number_2,
                                               const unsigned int      i_shuffle_operand );
 
+/**
+ * Generates shuffle instructions with 2 or 3 vector registers, memory operands are not supported as first operand
+ *
+ * @param io_generated_code pointer to the pointer of the generated code structure
+ * @param i_instruction_set requested instruction set to encode
+ * @param i_vmove_instr actual operation variant (gather/scatter and single/double)
+ * @param i_vector_name the vector register name prefix (x,y or z)
+ * @param i_gp_reg_number the register number (rax=0,rcx=1,rdx=2,rbx=3,rsp=4,rbp=5,rsi=6,rdi=7,r8=8,r9=9,r10=10,r11=11,r12=12,r13=13,r14=14,r15=15) of the base address register
+ * @param i_vec_reg_idx the index vector registers (ymm0-15 AVX2) (zmm0-zmm32 AVX512)
+ * @param i_scale the scaling of the indexes in i_vec_reg_idx
+ * @param i_displacement the offset to the base address
+ * @param i_vec_reg_number the destination(gather)/source(scatter) vec register (xmm/ymm: 0-15, zmm: 0-31)
+ * @param i_mask_reg_number the mask register (xmm/ymm: 0-15 when using AVX2), (k1-k7 when using AVX512)
+ * @param i_is_gather "true" generate a gather instruction, "false" generator a scatter instruction
+ */
+LIBXSMM_INTERNAL_API
+void libxsmm_x86_instruction_vec_move_gathscat( libxsmm_generated_code* io_generated_code,
+                                                const unsigned int      i_instruction_set,
+                                                const unsigned int      i_vmove_instr,
+                                                const char              i_vector_name,
+                                                const unsigned int      i_gp_reg_base,
+                                                const unsigned int      i_vec_reg_idx,
+                                                const unsigned int      i_scale,
+                                                const int               i_displacement,
+                                                const unsigned int      i_vec_reg_number,
+                                                const unsigned int      i_mask_reg_number,
+                                                const unsigned int      i_is_gather );
+
 /* @TODO check if we can merge this alu_imm */
 /**
  * Generates prefetch instructions with displacements, SIB addressing is not
@@ -162,6 +196,7 @@ void libxsmm_x86_instruction_vec_shuffle_reg( libxsmm_generated_code* io_generat
  * @param i_gp_reg_number the register number (rax=0,rcx=1,rdx=2,rbx=3,rsp=4,rbp=5,rsi=6,rdi=7,r8=8,r9=9,r10=10,r11=11,r12=12,r13=13,r14=14,r15=15) of the base address register
  * @param i_displacement the offset to the base address
  */
+LIBXSMM_INTERNAL_API
 void libxsmm_x86_instruction_prefetch( libxsmm_generated_code* io_generated_code,
                                        const unsigned int      i_prefetch_instr,
                                        const unsigned int      i_gp_reg_base,
@@ -177,6 +212,7 @@ void libxsmm_x86_instruction_prefetch( libxsmm_generated_code* io_generated_code
  * @param i_gp_reg_number the register number (rax=0,rcx=1,rdx=2,rbx=3,rsp=4,rbp=5,rsi=6,rdi=7,r8=8,r9=9,r10=10,r11=11,r12=12,r13=13,r14=14,r15=15) of the base address register
  * @param i_immediate the immediate operand
  */
+LIBXSMM_INTERNAL_API
 void libxsmm_x86_instruction_alu_imm( libxsmm_generated_code* io_generated_code,
                                       const unsigned int      i_alu_instr,
                                       const unsigned int      i_gp_reg_number,
@@ -190,6 +226,7 @@ void libxsmm_x86_instruction_alu_imm( libxsmm_generated_code* io_generated_code,
  * @param i_gp_reg_number_src the source register number (rax=0,rcx=1,rdx=2,rbx=3,rsp=4,rbp=5,rsi=6,rdi=7,r8=8,r9=9,r10=10,r11=11,r12=12,r13=13,r14=14,r15=15) of the base address register
  * @param i_gp_reg_number_dest the destination register number (rax=0,rcx=1,rdx=2,rbx=3,rsp=4,rbp=5,rsi=6,rdi=7,r8=8,r9=9,r10=10,r11=11,r12=12,r13=13,r14=14,r15=15) of the base address register
  */
+LIBXSMM_INTERNAL_API
 void libxsmm_x86_instruction_alu_reg( libxsmm_generated_code* io_generated_code,
                                       const unsigned int      i_alu_instr,
                                       const unsigned int      i_gp_reg_number_src,
@@ -201,6 +238,7 @@ void libxsmm_x86_instruction_alu_reg( libxsmm_generated_code* io_generated_code,
  * @param io_generated_code pointer to the pointer of the generated code structure
  * @param i_gp_reg_number the source register number (rax=0,rcx=1,rdx=2,rbx=3,rsp=4,rbp=5,rsi=6,rdi=7,r8=8,r9=9,r10=10,r11=11,r12=12,r13=13,r14=14,r15=15)
  */
+LIBXSMM_INTERNAL_API
 void libxsmm_x86_instruction_push_reg( libxsmm_generated_code* io_generated_code,
                                        const unsigned int      i_gp_reg_number );
 
@@ -210,6 +248,7 @@ void libxsmm_x86_instruction_push_reg( libxsmm_generated_code* io_generated_code
  * @param io_generated_code pointer to the pointer of the generated code structure
  * @param i_gp_reg_number the source register number (rax=0,rcx=1,rdx=2,rbx=3,rsp=4,rbp=5,rsi=6,rdi=7,r8=8,r9=9,r10=10,r11=11,r12=12,r13=13,r14=14,r15=15)
  */
+LIBXSMM_INTERNAL_API
 void libxsmm_x86_instruction_pop_reg( libxsmm_generated_code* io_generated_code,
                                       const unsigned int      i_gp_reg_number );
 
@@ -221,17 +260,35 @@ void libxsmm_x86_instruction_pop_reg( libxsmm_generated_code* io_generated_code,
  * @param i_gp_reg_number the register number (rax=0,rcx=1,rdx=2,rbx=3,rsp=4,rbp=5,rsi=6,rdi=7,r8=8,r9=9,r10=10,r11=11,r12=12,r13=13,r14=14,r15=15) of the base address register
  * @param i_mask_reg_number the register number (k1=1...k7=7)
  */
+LIBXSMM_INTERNAL_API
 void libxsmm_x86_instruction_mask_move( libxsmm_generated_code* io_generated_code,
                                         const unsigned int      i_mask_instr,
                                         const unsigned int      i_gp_reg_number,
                                         const unsigned int      i_mask_reg_number );
 
+/**
+ * Allows for mask move instructions in AVX512
+ *
+ * @param io_generated_code pointer to the pointer of the generated code structure
+ * @param i_mask_instr actual mask compute instruction instruction
+ * @param i_mask_reg_number_src_0 the first operand register number (att syntax) (k1=1...k7=7)
+ * @param i_mask_reg_number_src_1 the second operand register number (att syntax) (k1=1...k7=7)
+ * @param i_mask_reg_number_dest the third operand register number (att syntax) (k1=1...k7=7)
+ */
+LIBXSMM_INTERNAL_API
+void libxsmm_x86_instruction_mask_compute_reg( libxsmm_generated_code* io_generated_code,
+                                               const unsigned int      i_mask_instr,
+                                               const unsigned int      i_mask_reg_number_src_0,
+                                               const unsigned int      i_mask_reg_number_src_1,
+                                               const unsigned int      i_mask_reg_number_dest  );
+
 /**
  * Generates regular all instructions with immediates
  *
  * @param io_generated_code pointer to the pointer of the generated code structure
  * @param io_loop_label_tracker data structure to handle loop labels, nested loops are supported, but not overlapping loops
 */
+LIBXSMM_INTERNAL_API
 void libxsmm_x86_instruction_register_jump_label( libxsmm_generated_code*     io_generated_code,
                                                   libxsmm_loop_label_tracker* io_loop_label_tracker );
 
@@ -242,9 +299,38 @@ void libxsmm_x86_instruction_register_jump_label( libxsmm_generated_code*     io
  * @param i_jmp_instr the particular jump instruction used
  * @param io_loop_label_tracker data structure to handle loop labels will jump to latest registered label
 */
+LIBXSMM_INTERNAL_API
 void libxsmm_x86_instruction_jump_back_to_label( libxsmm_generated_code*     io_generated_code,
                                                  const unsigned int          i_jmp_instr,
                                                  libxsmm_loop_label_tracker* io_loop_label_tracker );
 
+/**
+ * @TODO: clean-up
+ * Opens the inline assembly section / jit stream for convolutions, this is hacked and should be cleaned up
+ *
+ * @param io_generated_code pointer to the pointer of the generated code structure
+ * @param i_arch architecture code was generated for (needed to build clobber)
+ */
+LIBXSMM_INTERNAL_API
+void libxsmm_x86_instruction_open_stream_convolution( libxsmm_generated_code*                   io_generated_code,
+                                                      const unsigned int                        i_gp_reg_input,
+                                                      const unsigned int                        i_gp_reg_weight,
+                                                      const unsigned int                        i_gp_reg_output,
+                                                      const unsigned int                        i_gp_reg_input_pf,
+                                                      const unsigned int                        i_gp_reg_weight_pf,
+                                                      const unsigned int                        i_gp_reg_output_pf,
+                                                      const char*                               i_arch );
+
+/**
+ * @TODO: clean-up
+ * Closes the inline assembly section / jit stream for convolutions, this is hacked and should be cleaned up
+ *
+ * @param io_generated_code pointer to the pointer of the generated code structure
+ * @param i_arch architecture code was generated for (needed to build clobber)
+ */
+LIBXSMM_INTERNAL_API
+void libxsmm_x86_instruction_close_stream_convolution( libxsmm_generated_code*       io_generated_code,
+                                                const char*                   i_arch );
+
 #endif /* GENERATOR_X86_INSTRUCTIONS_H */
 
