@@ -1,5 +1,5 @@
 /******************************************************************************
-** Copyright (c) 2016, Intel Corporation                                     **
+** Copyright (c) 2016-2017, Intel Corporation                                **
 ** All rights reserved.                                                      **
 **                                                                           **
 ** Redistribution and use in source and binary forms, with or without        **
@@ -28,16 +28,17 @@
 ******************************************************************************/
 /* Hans Pabst (Intel Corp.)
 ******************************************************************************/
+#ifndef LIBXSMM_GEMM_DIFF_C
+#define LIBXSMM_GEMM_DIFF_C
+
 #include "libxsmm_gemm_diff.h"
-#include "libxsmm_intrinsics_x86.h"
+#include <libxsmm_intrinsics_x86.h>
 
 #if defined(LIBXSMM_OFFLOAD_TARGET)
 # pragma offload_attribute(push,target(LIBXSMM_OFFLOAD_TARGET))
 #endif
-#include <stdint.h>
 #include <string.h>
 #if !defined(NDEBUG)
-# include <assert.h>
 # include <stdio.h>
 #endif
 #if defined(LIBXSMM_OFFLOAD_TARGET)
@@ -45,85 +46,79 @@
 #endif
 
 /* individually enable intrinsic code paths */
-#if defined(LIBXSMM_MAX_STATIC_TARGET_ARCH)
-# define LIBXSMM_GEMM_DIFF_AVX512_CORE
-# define LIBXSMM_GEMM_DIFF_AVX512_MIC
-# define LIBXSMM_GEMM_DIFF_AVX2
-# define LIBXSMM_GEMM_DIFF_AVX
-# define LIBXSMM_GEMM_DIFF_SSE
-/*# define LIBXSMM_GEMM_DIFF_KNC*/
-#endif
+#define LIBXSMM_GEMM_DIFF_AVX512
+#define LIBXSMM_GEMM_DIFF_AVX2
+#define LIBXSMM_GEMM_DIFF_AVX
+#define LIBXSMM_GEMM_DIFF_SSE
+/*#define LIBXSMM_GEMM_DIFF_KNC*/
 
 
-LIBXSMM_RETARGETABLE LIBXSMM_VISIBILITY_INTERNAL libxsmm_gemm_diff_function internal_gemm_diff_function = libxsmm_gemm_diff_sw;
-LIBXSMM_RETARGETABLE LIBXSMM_VISIBILITY_INTERNAL libxsmm_gemm_diffn_function internal_gemm_diffn_function = libxsmm_gemm_diffn_sw;
+LIBXSMM_EXTERN_C LIBXSMM_RETARGETABLE libxsmm_gemm_diff_function internal_gemm_diff_fn /*= libxsmm_gemm_diff_sw*/;
+LIBXSMM_EXTERN_C LIBXSMM_RETARGETABLE libxsmm_gemm_diffn_function internal_gemm_diffn_fn /*= libxsmm_gemm_diffn_sw*/;
 
 
-LIBXSMM_EXTERN_C LIBXSMM_RETARGETABLE void libxsmm_gemm_diff_init(int target_arch)
+LIBXSMM_GEMM_DIFF_API_DEFINITION void libxsmm_gemm_diff_init(int target_arch)
 {
+  internal_gemm_diff_fn = libxsmm_gemm_diff_sw;
+  internal_gemm_diffn_fn = libxsmm_gemm_diffn_sw;
 #if defined(__MIC__)
   LIBXSMM_UNUSED(target_arch);
-  internal_gemm_diffn_function = libxsmm_gemm_diffn_imci;
-  internal_gemm_diff_function = libxsmm_gemm_diff_imci;
+  internal_gemm_diffn_fn = libxsmm_gemm_diffn_imci;
+  internal_gemm_diff_fn = libxsmm_gemm_diff_imci;
 #else
-  if (LIBXSMM_X86_AVX512_CORE <= target_arch) {
-    internal_gemm_diffn_function = libxsmm_gemm_diffn_avx512;
-    internal_gemm_diff_function = libxsmm_gemm_diff_avx2;
-  }
-  else if (LIBXSMM_X86_AVX512_MIC <= target_arch) {
-    internal_gemm_diffn_function = libxsmm_gemm_diffn_avx512;
-    internal_gemm_diff_function = libxsmm_gemm_diff_avx2;
+  if (LIBXSMM_X86_AVX512 <= target_arch) {
+    internal_gemm_diffn_fn = libxsmm_gemm_diffn_avx512;
+    internal_gemm_diff_fn = libxsmm_gemm_diff_avx2;
   }
   else if (LIBXSMM_X86_AVX2 <= target_arch) {
-    internal_gemm_diffn_function = libxsmm_gemm_diffn_avx2;
-    internal_gemm_diff_function = libxsmm_gemm_diff_avx2;
+    internal_gemm_diffn_fn = libxsmm_gemm_diffn_avx2;
+    internal_gemm_diff_fn = libxsmm_gemm_diff_avx2;
   }
   else if (LIBXSMM_X86_AVX <= target_arch) {
-    internal_gemm_diffn_function = libxsmm_gemm_diffn_avx;
-    internal_gemm_diff_function = libxsmm_gemm_diff_avx;
+    internal_gemm_diffn_fn = libxsmm_gemm_diffn_avx;
+    internal_gemm_diff_fn = libxsmm_gemm_diff_avx;
   }
   else if (LIBXSMM_X86_SSE3 <= target_arch) {
-    internal_gemm_diff_function = libxsmm_gemm_diff_sse;
+    internal_gemm_diff_fn = libxsmm_gemm_diff_sse;
   }
 #endif
+  assert(0 != internal_gemm_diff_fn);
+  assert(0 != internal_gemm_diffn_fn);
 }
 
 
-LIBXSMM_EXTERN_C LIBXSMM_RETARGETABLE void libxsmm_gemm_diff_finalize(void)
+LIBXSMM_GEMM_DIFF_API_DEFINITION void libxsmm_gemm_diff_finalize(void)
 {
-  internal_gemm_diff_function = libxsmm_gemm_diff_sw;
-  internal_gemm_diffn_function = libxsmm_gemm_diffn_sw;
 }
 
 
-LIBXSMM_EXTERN_C LIBXSMM_RETARGETABLE
-unsigned int libxsmm_gemm_diff(const libxsmm_gemm_descriptor* reference, const libxsmm_gemm_descriptor* desc)
+LIBXSMM_GEMM_DIFF_API_DEFINITION unsigned int libxsmm_gemm_diff(const libxsmm_gemm_descriptor* reference, const libxsmm_gemm_descriptor* desc)
 {
   /* attempt to rely on static code path avoids to rely on capability of inlining pointer-based function call */
 #if defined(LIBXSMM_GEMM_DIFF_SW) && (0 != LIBXSMM_GEMM_DIFF_SW)
   return libxsmm_gemm_diff_sw(reference, desc);
 #elif defined(__MIC__)
   return libxsmm_gemm_diff_imci(reference, desc);
-#elif defined(LIBXSMM_STATIC_TARGET_ARCH) && (LIBXSMM_X86_AVX2 <= LIBXSMM_STATIC_TARGET_ARCH)
+#elif (LIBXSMM_X86_AVX2 <= LIBXSMM_STATIC_TARGET_ARCH)
   return libxsmm_gemm_diff_avx2(reference, desc);
-#elif defined(LIBXSMM_STATIC_TARGET_ARCH) && (LIBXSMM_X86_AVX <= LIBXSMM_STATIC_TARGET_ARCH)
+#elif (LIBXSMM_X86_AVX <= LIBXSMM_STATIC_TARGET_ARCH)
   return libxsmm_gemm_diff_avx(reference, desc);
-#elif defined(LIBXSMM_STATIC_TARGET_ARCH) && (LIBXSMM_X86_SSE3 <= LIBXSMM_STATIC_TARGET_ARCH)
+#elif (LIBXSMM_X86_SSE3 <= LIBXSMM_STATIC_TARGET_ARCH)
   return libxsmm_gemm_diff_sse(reference, desc);
 #else /* pointer based function call */
-  assert(0 != internal_gemm_diff_function);
-  return (*internal_gemm_diff_function)(reference, desc);
+  assert(0 != internal_gemm_diff_fn);
+  return internal_gemm_diff_fn(reference, desc);
 #endif
 }
 
 
-LIBXSMM_EXTERN_C LIBXSMM_RETARGETABLE
-unsigned int libxsmm_gemm_diff_sw(const libxsmm_gemm_descriptor* reference, const libxsmm_gemm_descriptor* desc)
+LIBXSMM_GEMM_DIFF_API_DEFINITION unsigned int libxsmm_gemm_diff_sw(const libxsmm_gemm_descriptor* reference, const libxsmm_gemm_descriptor* desc)
 {
 #if defined(LIBXSMM_GEMM_DIFF_SW) && (2 == (LIBXSMM_GEMM_DIFF_SW))
   return 0 != memcmp(reference, desc, LIBXSMM_GEMM_DESCRIPTOR_SIZE);
 #else
-  const unsigned *const ia = (const unsigned int*)reference, *const ib = (const unsigned int*)desc;
+  typedef unsigned int LIBXSMM_MAY_ALIAS uia_type;
+  const uia_type *const ia = (const uia_type*)reference, *const ib = (const uia_type*)desc;
   const unsigned int end = (LIBXSMM_GEMM_DESCRIPTOR_SIZE >> 2/*LOG2(sizeof(int))*/);
   unsigned int result, i;
   assert(0 == LIBXSMM_MOD2(LIBXSMM_GEMM_DESCRIPTOR_SIZE, sizeof(unsigned int)));
@@ -137,17 +132,17 @@ unsigned int libxsmm_gemm_diff_sw(const libxsmm_gemm_descriptor* reference, cons
 }
 
 
-LIBXSMM_EXTERN_C LIBXSMM_RETARGETABLE LIBXSMM_INTRINSICS
+LIBXSMM_GEMM_DIFF_API_DEFINITION LIBXSMM_INTRINSICS(LIBXSMM_X86_SSE3)
 unsigned int libxsmm_gemm_diff_sse(const libxsmm_gemm_descriptor* reference, const libxsmm_gemm_descriptor* desc)
 {
   assert(0 != reference && 0 != desc);
-#if defined(LIBXSMM_GEMM_DIFF_SSE) && (LIBXSMM_X86_SSE3 <= LIBXSMM_MAX_STATIC_TARGET_ARCH)
+#if !defined(LIBXSMM_INTRINSICS_NONE) && !defined(LIBXSMM_INTRINSICS_LEGACY) \
+  && defined(LIBXSMM_GEMM_DIFF_SSE) && (LIBXSMM_X86_SSE3 <= LIBXSMM_MAX_STATIC_TARGET_ARCH)
   assert(0 == LIBXSMM_MOD2(LIBXSMM_GEMM_DESCRIPTOR_SIZE, sizeof(unsigned int)));
-  assert(4 >= LIBXSMM_DIV2(LIBXSMM_GEMM_DESCRIPTOR_SIZE, sizeof(unsigned int)));
 # if (16 == LIBXSMM_GEMM_DESCRIPTOR_SIZE)
   {
-    const __m128i a128 = _mm_lddqu_si128((const __m128i*)reference);
-    const __m128i b128 = _mm_lddqu_si128((const __m128i*)desc);
+    const __m128i a128 = LIBXSMM_INTRINSICS_LDDQU_SI128((const __m128i*)reference);
+    const __m128i b128 = LIBXSMM_INTRINSICS_LDDQU_SI128((const __m128i*)desc);
     const __m128i c128 = _mm_cmpeq_epi8(a128, b128);
     return 0xFFFF != _mm_movemask_epi8(c128);
   }
@@ -156,10 +151,9 @@ unsigned int libxsmm_gemm_diff_sse(const libxsmm_gemm_descriptor* reference, con
 # endif
 #else
 # if !defined(NDEBUG) && defined(LIBXSMM_GEMM_DIFF_SSE) /* library code is expected to be mute */
-  { static LIBXSMM_TLS int once = 0;
-    if (0 == once) {
+  { static int error_once = 0;
+    if (1 == LIBXSMM_ATOMIC_ADD_FETCH(&error_once, 1, LIBXSMM_ATOMIC_RELAXED)) {
       fprintf(stderr, "LIBXSMM: unable to enter SSE code path!\n");
-      once = 1;
     }
   }
 # endif
@@ -168,37 +162,27 @@ unsigned int libxsmm_gemm_diff_sse(const libxsmm_gemm_descriptor* reference, con
 }
 
 
-LIBXSMM_EXTERN_C LIBXSMM_RETARGETABLE LIBXSMM_INTRINSICS
+LIBXSMM_GEMM_DIFF_API_DEFINITION LIBXSMM_INTRINSICS(LIBXSMM_X86_AVX)
 unsigned int libxsmm_gemm_diff_avx(const libxsmm_gemm_descriptor* reference, const libxsmm_gemm_descriptor* desc)
 {
   assert(0 != reference && 0 != desc);
-#if defined(LIBXSMM_GEMM_DIFF_AVX) && (LIBXSMM_X86_AVX <= LIBXSMM_MAX_STATIC_TARGET_ARCH)
+#if !defined(LIBXSMM_INTRINSICS_NONE) && !defined(LIBXSMM_INTRINSICS_LEGACY) \
+  && defined(LIBXSMM_GEMM_DIFF_AVX) && (LIBXSMM_X86_AVX <= LIBXSMM_MAX_STATIC_TARGET_ARCH)
   assert(0 == LIBXSMM_MOD2(LIBXSMM_GEMM_DESCRIPTOR_SIZE, sizeof(unsigned int)));
-  assert(8 >= LIBXSMM_DIV2(LIBXSMM_GEMM_DESCRIPTOR_SIZE, sizeof(unsigned int)));
 # if (28 == LIBXSMM_GEMM_DESCRIPTOR_SIZE)
   {
-    int r0, r1;
-    __m256i a256, b256;
-#   if defined(__CYGWIN__) && !defined(NDEBUG) /* Cygwin/GCC: _mm256_set_epi32 may cause an illegal instruction */
-    const union { int32_t array[8]; __m256i i; } m256 = { /* use literal value rather than yes/no
-      in order to avoid warning about "initializer element is not computable at load time" */
-      { 0x80000000, 0x80000000, 0x80000000, 0x80000000, 0x80000000, 0x80000000, 0x80000000, 0x0 }
-    };
-#   else
     const int yes = 0x80000000, no = 0x0;
-    struct { __m256i i; } m256;
-    m256.i = _mm256_set_epi32(no, yes, yes, yes, yes, yes, yes, yes);
-#   endif
+    const __m256i m256 = _mm256_set_epi32(no, yes, yes, yes, yes, yes, yes, yes);
 #   if defined(LIBXSMM_GEMM_DIFF_MASK_A) || !defined(LIBXSMM_GEMM_DIFF_ZERO_PADDED)
-    a256 = _mm256_castps_si256(_mm256_maskload_ps((const float*)reference, m256.i));
+    const __m256i a256 = _mm256_castps_si256(_mm256_maskload_ps((const float*)reference, m256));
 #   else
-    /*a256 = _mm256_lddqu_si256((const __m256i*)reference);*/
-    a256 = _mm256_loadu_si256((const __m256i*)reference);
+    /*const __m256i a256 = _mm256_lddqu_si256((const __m256i*)reference);*/
+    const __m256i a256 = _mm256_loadu_si256((const __m256i*)reference);
 #   endif
-    b256 = _mm256_castps_si256(_mm256_maskload_ps((const float*)desc, m256.i));
+    const __m256i b256 = _mm256_castps_si256(_mm256_maskload_ps((const float*)desc, m256));
     /* avoid warning about eval. in unspecified order: r0, r1 */
-    r0 = _mm256_testnzc_si256(a256, b256);
-    r1 = _mm256_testnzc_si256(b256, a256);
+    const int r0 = _mm256_testnzc_si256(a256, b256);
+    const int r1 = _mm256_testnzc_si256(b256, a256);
     return r0 | r1;
   }
 # else
@@ -208,10 +192,9 @@ unsigned int libxsmm_gemm_diff_avx(const libxsmm_gemm_descriptor* reference, con
 # endif
 #else
 # if !defined(NDEBUG) && defined(LIBXSMM_GEMM_DIFF_AVX) /* library code is expected to be mute */
-  { static LIBXSMM_TLS int once = 0;
-    if (0 == once) {
+  { static int error_once = 0;
+    if (1 == LIBXSMM_ATOMIC_ADD_FETCH(&error_once, 1, LIBXSMM_ATOMIC_RELAXED)) {
       fprintf(stderr, "LIBXSMM: unable to enter AVX code path!\n");
-      once = 1;
     }
   }
 # endif
@@ -220,24 +203,24 @@ unsigned int libxsmm_gemm_diff_avx(const libxsmm_gemm_descriptor* reference, con
 }
 
 
-LIBXSMM_EXTERN_C LIBXSMM_RETARGETABLE LIBXSMM_INTRINSICS
+LIBXSMM_GEMM_DIFF_API_DEFINITION LIBXSMM_INTRINSICS(LIBXSMM_X86_AVX2)
 unsigned int libxsmm_gemm_diff_avx2(const libxsmm_gemm_descriptor* reference, const libxsmm_gemm_descriptor* desc)
 {
   assert(0 != reference && 0 != desc);
-#if defined(LIBXSMM_GEMM_DIFF_AVX2) && (LIBXSMM_X86_AVX2 <= LIBXSMM_MAX_STATIC_TARGET_ARCH)
+#if !defined(LIBXSMM_INTRINSICS_NONE) && !defined(LIBXSMM_INTRINSICS_LEGACY) \
+  && defined(LIBXSMM_GEMM_DIFF_AVX2) && (LIBXSMM_X86_AVX2 <= LIBXSMM_MAX_STATIC_TARGET_ARCH)
   assert(0 == LIBXSMM_MOD2(LIBXSMM_GEMM_DESCRIPTOR_SIZE, sizeof(unsigned int)));
-  assert(8 >= LIBXSMM_DIV2(LIBXSMM_GEMM_DESCRIPTOR_SIZE, sizeof(unsigned int)));
 # if (28 == LIBXSMM_GEMM_DESCRIPTOR_SIZE)
   {
     const int yes = 0x80000000, no = 0x0;
     const __m256i m256 = _mm256_set_epi32(no, yes, yes, yes, yes, yes, yes, yes);
 #   if defined(LIBXSMM_GEMM_DIFF_MASK_A) || !defined(LIBXSMM_GEMM_DIFF_ZERO_PADDED)
-    const __m256i a256 = _mm256_maskload_epi32((const void*)reference, m256);
+    const __m256i a256 = _mm256_maskload_epi32((const int*)reference, m256);
 #   else
     /*const __m256i a256 = _mm256_lddqu_si256((const __m256i*)reference);*/
     const __m256i a256 = _mm256_loadu_si256((const __m256i*)reference);
 #   endif
-    const __m256i b256 = _mm256_maskload_epi32((const void*)desc, m256);
+    const __m256i b256 = _mm256_maskload_epi32((const int*)desc, m256);
     /* avoid warning about eval. in unspecified order: r0, r1 */
     const int r0 = _mm256_testnzc_si256(a256, b256);
     const int r1 = _mm256_testnzc_si256(b256, a256);
@@ -252,10 +235,9 @@ unsigned int libxsmm_gemm_diff_avx2(const libxsmm_gemm_descriptor* reference, co
 # endif
 #else
 # if !defined(NDEBUG) && defined(LIBXSMM_GEMM_DIFF_AVX2) /* library code is expected to be mute */
-  { static LIBXSMM_TLS int once = 0;
-    if (0 == once) {
+  { static int error_once = 0;
+    if (1 == LIBXSMM_ATOMIC_ADD_FETCH(&error_once, 1, LIBXSMM_ATOMIC_RELAXED)) {
       fprintf(stderr, "LIBXSMM: unable to enter AVX2 code path!\n");
-      once = 1;
     }
   }
 # endif
@@ -264,13 +246,11 @@ unsigned int libxsmm_gemm_diff_avx2(const libxsmm_gemm_descriptor* reference, co
 }
 
 
-LIBXSMM_EXTERN_C LIBXSMM_RETARGETABLE
-unsigned int libxsmm_gemm_diff_imci(const libxsmm_gemm_descriptor* reference, const libxsmm_gemm_descriptor* desc)
+LIBXSMM_GEMM_DIFF_API_DEFINITION unsigned int libxsmm_gemm_diff_imci(const libxsmm_gemm_descriptor* reference, const libxsmm_gemm_descriptor* desc)
 {
   assert(0 != reference && 0 != desc);
 #if defined(__MIC__) && (28 == LIBXSMM_GEMM_DESCRIPTOR_SIZE)
   assert(0 ==  LIBXSMM_MOD2(LIBXSMM_GEMM_DESCRIPTOR_SIZE, sizeof(unsigned int)));
-  assert(16 >= LIBXSMM_DIV2(LIBXSMM_GEMM_DESCRIPTOR_SIZE, sizeof(unsigned int)));
   {
     const __mmask16 mask = (0xFFFF >> (16 - (LIBXSMM_GEMM_DESCRIPTOR_SIZE >> 2/*LOG2(sizeof(int))*/)));
     const __m512i a512 = _mm512_mask_loadunpackhi_epi32(
@@ -287,30 +267,26 @@ unsigned int libxsmm_gemm_diff_imci(const libxsmm_gemm_descriptor* reference, co
 }
 
 
-LIBXSMM_EXTERN_C LIBXSMM_RETARGETABLE
-unsigned int libxsmm_gemm_diffn(const libxsmm_gemm_descriptor* reference, const libxsmm_gemm_descriptor* descs,
+LIBXSMM_GEMM_DIFF_API_DEFINITION unsigned int libxsmm_gemm_diffn(const libxsmm_gemm_descriptor* reference, const libxsmm_gemm_descriptor* descs,
   unsigned int hint, unsigned int ndescs, int nbytes)
 {
   /* attempt to rely on static code path avoids to rely on capability of inlining pointer-based function call */
 #if defined(LIBXSMM_GEMM_DIFF_SW) && (0 != LIBXSMM_GEMM_DIFF_SW)
   return libxsmm_gemm_diffn_sw(reference, descs, hint, ndescs, nbytes);
-#elif defined(LIBXSMM_STATIC_TARGET_ARCH) && (LIBXSMM_X86_AVX512_CORE <= LIBXSMM_STATIC_TARGET_ARCH)
+#elif (LIBXSMM_X86_AVX512 <= LIBXSMM_STATIC_TARGET_ARCH)
   return libxsmm_gemm_diffn_avx512(reference, descs, hint, ndescs, nbytes);
-#elif defined(LIBXSMM_STATIC_TARGET_ARCH) && (LIBXSMM_X86_AVX512_MIC <= LIBXSMM_STATIC_TARGET_ARCH)
-  return libxsmm_gemm_diffn_avx512(reference, descs, hint, ndescs, nbytes);
-#elif defined(LIBXSMM_STATIC_TARGET_ARCH) && (LIBXSMM_X86_AVX2 <= LIBXSMM_STATIC_TARGET_ARCH)
+#elif (LIBXSMM_X86_AVX2 <= LIBXSMM_STATIC_TARGET_ARCH)
   return libxsmm_gemm_diffn_avx2(reference, descs, hint, ndescs, nbytes);
-#elif defined(LIBXSMM_STATIC_TARGET_ARCH) && (LIBXSMM_X86_AVX <= LIBXSMM_STATIC_TARGET_ARCH)
+#elif (LIBXSMM_X86_AVX <= LIBXSMM_STATIC_TARGET_ARCH)
   return libxsmm_gemm_diffn_avx(reference, descs, hint, ndescs, nbytes);
 #else /* pointer based function call */
-  assert(0 != internal_gemm_diffn_function);
-  return (*internal_gemm_diffn_function)(reference, descs, hint, ndescs, nbytes);
+  assert(0 != internal_gemm_diffn_fn);
+  return internal_gemm_diffn_fn(reference, descs, hint, ndescs, nbytes);
 #endif
 }
 
 
-LIBXSMM_EXTERN_C LIBXSMM_RETARGETABLE
-unsigned int libxsmm_gemm_diffn_sw(const libxsmm_gemm_descriptor* reference, const libxsmm_gemm_descriptor* descs,
+LIBXSMM_GEMM_DIFF_API_DEFINITION unsigned int libxsmm_gemm_diffn_sw(const libxsmm_gemm_descriptor* reference, const libxsmm_gemm_descriptor* descs,
   unsigned int hint, unsigned int ndescs, int nbytes)
 {
   const char *const desc = (const char*)descs;
@@ -337,41 +313,34 @@ unsigned int libxsmm_gemm_diffn_sw(const libxsmm_gemm_descriptor* reference, con
 }
 
 
-LIBXSMM_EXTERN_C LIBXSMM_RETARGETABLE LIBXSMM_INTRINSICS
-unsigned int libxsmm_gemm_diffn_avx(const libxsmm_gemm_descriptor* reference, const libxsmm_gemm_descriptor* descs,
+LIBXSMM_GEMM_DIFF_API_DEFINITION LIBXSMM_INTRINSICS(LIBXSMM_X86_AVX)
+unsigned int libxsmm_gemm_diffn_avx(
+  const libxsmm_gemm_descriptor* reference, const libxsmm_gemm_descriptor* descs,
   unsigned int hint, unsigned int ndescs, int nbytes)
 {
-#if defined(LIBXSMM_GEMM_DIFF_AVX) && (LIBXSMM_X86_AVX <= LIBXSMM_MAX_STATIC_TARGET_ARCH)
+#if !defined(LIBXSMM_INTRINSICS_NONE) && !defined(LIBXSMM_INTRINSICS_LEGACY) \
+  && defined(LIBXSMM_GEMM_DIFF_AVX) && (LIBXSMM_X86_AVX <= LIBXSMM_MAX_STATIC_TARGET_ARCH)
   assert(/*is pot*/ndescs == (1u << LIBXSMM_LOG2(ndescs)));
 # if (28 == LIBXSMM_GEMM_DESCRIPTOR_SIZE)
   assert(32 == nbytes); /* padded descriptor array */
   {
     const unsigned int end = hint + ndescs;
     const char *const desc = (const char*)descs;
-    __m256i a256;
-    unsigned int i;
-#   if defined(__CYGWIN__) && !defined(NDEBUG) /* Cygwin/GCC: _mm256_set_epi32 may cause an illegal instruction */
-    const union { int32_t array[8]; __m256i i; } m256 = { /* use literal value rather than yes/no
-      in order to avoid warning about "initializer element is not computable at load time" */
-      { 0x80000000, 0x80000000, 0x80000000, 0x80000000, 0x80000000, 0x80000000, 0x80000000, 0x0 }
-    };
-#   else
     const int yes = 0x80000000, no = 0x0;
-    struct { __m256i i; } m256;
-    m256.i = _mm256_set_epi32(no, yes, yes, yes, yes, yes, yes, yes);
-#   endif
+    unsigned int i;
+    const __m256i m256 = _mm256_set_epi32(no, yes, yes, yes, yes, yes, yes, yes);
 #   if defined(LIBXSMM_GEMM_DIFF_MASK_A) || !defined(LIBXSMM_GEMM_DIFF_ZERO_PADDED)
-    a256 = _mm256_castps_si256(_mm256_maskload_ps((const float*)reference, m256.i));
+    const __m256i a256 = _mm256_castps_si256(_mm256_maskload_ps((const float*)reference, m256));
 #   else
-    /*a256 = _mm256_lddqu_si256((const __m256i*)reference);*/
-    a256 = _mm256_loadu_si256((const __m256i*)reference);
+    /*const __m256i a256 = _mm256_lddqu_si256((const __m256i*)reference);*/
+    const __m256i a256 = _mm256_loadu_si256((const __m256i*)reference);
 #   endif
     for (i = hint; i < end; ++i) {
       const unsigned int j = LIBXSMM_MOD2(i, ndescs); /* wrap around index */
 #   if defined(LIBXSMM_GEMM_DIFF_ZERO_PADDED)
       const __m256i b256 = _mm256_loadu_si256((const __m256i*)(desc + j * nbytes));
 #   else
-      const __m256i b256 = _mm256_castps_si256(_mm256_maskload_ps((const float*)(desc + j * nbytes), m256.i));
+      const __m256i b256 = _mm256_castps_si256(_mm256_maskload_ps((const float*)(desc + j * nbytes), m256));
 #   endif
       if (0 == _mm256_testnzc_si256(a256, b256) && 0 == _mm256_testnzc_si256(b256, a256)) {
         return j;
@@ -389,10 +358,9 @@ unsigned int libxsmm_gemm_diffn_avx(const libxsmm_gemm_descriptor* reference, co
 # endif
 #else
 # if !defined(NDEBUG) && defined(LIBXSMM_GEMM_DIFF_AVX) /* library code is expected to be mute */
-  { static LIBXSMM_TLS int once = 0;
-    if (0 == once) {
+  { static int error_once = 0;
+    if (1 == LIBXSMM_ATOMIC_ADD_FETCH(&error_once, 1, LIBXSMM_ATOMIC_RELAXED)) {
       fprintf(stderr, "LIBXSMM: unable to enter AVX code path!\n");
-      once = 1;
     }
   }
 # endif
@@ -401,11 +369,13 @@ unsigned int libxsmm_gemm_diffn_avx(const libxsmm_gemm_descriptor* reference, co
 }
 
 
-LIBXSMM_EXTERN_C LIBXSMM_RETARGETABLE LIBXSMM_INTRINSICS
-unsigned int libxsmm_gemm_diffn_avx2(const libxsmm_gemm_descriptor* reference, const libxsmm_gemm_descriptor* descs,
+LIBXSMM_GEMM_DIFF_API_DEFINITION LIBXSMM_INTRINSICS(LIBXSMM_X86_AVX2)
+unsigned int libxsmm_gemm_diffn_avx2(
+  const libxsmm_gemm_descriptor* reference, const libxsmm_gemm_descriptor* descs,
   unsigned int hint, unsigned int ndescs, int nbytes)
 {
-#if defined(LIBXSMM_GEMM_DIFF_AVX2) && (LIBXSMM_X86_AVX2 <= LIBXSMM_MAX_STATIC_TARGET_ARCH)
+#if !defined(LIBXSMM_INTRINSICS_NONE) && !defined(LIBXSMM_INTRINSICS_LEGACY) \
+  && defined(LIBXSMM_GEMM_DIFF_AVX2) && (LIBXSMM_X86_AVX2 <= LIBXSMM_MAX_STATIC_TARGET_ARCH)
   assert(/*is pot*/ndescs == (1u << LIBXSMM_LOG2(ndescs)));
 # if (28 == LIBXSMM_GEMM_DESCRIPTOR_SIZE)
   assert(32 == nbytes); /* padded descriptor array */
@@ -416,7 +386,7 @@ unsigned int libxsmm_gemm_diffn_avx2(const libxsmm_gemm_descriptor* reference, c
     const int yes = 0x80000000, no = 0x0;
     const __m256i m256 = _mm256_set_epi32(no, yes, yes, yes, yes, yes, yes, yes);
 #   if defined(LIBXSMM_GEMM_DIFF_MASK_A) || !defined(LIBXSMM_GEMM_DIFF_ZERO_PADDED)
-    const __m256i a256 = _mm256_maskload_epi32((const void*)reference, m256);
+    const __m256i a256 = _mm256_maskload_epi32((const int*)reference, m256);
 #   else
     /*const __m256i a256 = _mm256_lddqu_si256((const __m256i*)reference);*/
     const __m256i a256 = _mm256_loadu_si256((const __m256i*)reference);
@@ -426,7 +396,7 @@ unsigned int libxsmm_gemm_diffn_avx2(const libxsmm_gemm_descriptor* reference, c
 #   if defined(LIBXSMM_GEMM_DIFF_ZERO_PADDED)
       const __m256i b256 = _mm256_loadu_si256((const __m256i*)(desc + j * nbytes));
 #   else
-      const __m256i b256 = _mm256_maskload_epi32((const void*)(desc + j * nbytes), m256);
+      const __m256i b256 = _mm256_maskload_epi32((const int*)(desc + j * nbytes), m256);
 #   endif
       if (0 == _mm256_testnzc_si256(a256, b256) && 0 == _mm256_testnzc_si256(b256, a256)) {
         return j;
@@ -444,10 +414,9 @@ unsigned int libxsmm_gemm_diffn_avx2(const libxsmm_gemm_descriptor* reference, c
 # endif
 #else
 # if !defined(NDEBUG) && defined(LIBXSMM_GEMM_DIFF_AVX2) /* library code is expected to be mute */
-  { static LIBXSMM_TLS int once = 0;
-    if (0 == once) {
+  { static int error_once = 0;
+    if (1 == LIBXSMM_ATOMIC_ADD_FETCH(&error_once, 1, LIBXSMM_ATOMIC_RELAXED)) {
       fprintf(stderr, "LIBXSMM: unable to enter AVX2 code path!\n");
-      once = 1;
     }
   }
 # endif
@@ -456,12 +425,13 @@ unsigned int libxsmm_gemm_diffn_avx2(const libxsmm_gemm_descriptor* reference, c
 }
 
 
-LIBXSMM_EXTERN_C LIBXSMM_RETARGETABLE LIBXSMM_INTRINSICS
-unsigned int libxsmm_gemm_diffn_avx512(const libxsmm_gemm_descriptor* reference, const libxsmm_gemm_descriptor* descs,
+LIBXSMM_GEMM_DIFF_API_DEFINITION LIBXSMM_INTRINSICS(LIBXSMM_X86_AVX512)
+unsigned int libxsmm_gemm_diffn_avx512(
+  const libxsmm_gemm_descriptor* reference, const libxsmm_gemm_descriptor* descs,
   unsigned int hint, unsigned int ndescs, int nbytes)
 {
-#if (defined(LIBXSMM_GEMM_DIFF_AVX512_MIC)  && LIBXSMM_X86_AVX512_MIC  <= LIBXSMM_MAX_STATIC_TARGET_ARCH) || \
-    (defined(LIBXSMM_GEMM_DIFF_AVX512_CORE) && LIBXSMM_X86_AVX512_CORE <= LIBXSMM_MAX_STATIC_TARGET_ARCH)
+#if !defined(LIBXSMM_INTRINSICS_NONE) && !defined(LIBXSMM_INTRINSICS_INCOMPLETE_AVX512) \
+  && defined(LIBXSMM_GEMM_DIFF_AVX512) && (LIBXSMM_X86_AVX512 <= LIBXSMM_MAX_STATIC_TARGET_ARCH)
   assert(/*is pot*/ndescs == (1 << LIBXSMM_LOG2(ndescs)));
 # if (28 == LIBXSMM_GEMM_DESCRIPTOR_SIZE)
   assert(32 == nbytes); /* padded descriptor array */
@@ -473,7 +443,7 @@ unsigned int libxsmm_gemm_diffn_avx512(const libxsmm_gemm_descriptor* reference,
     const int yes = 0x80000000, no = 0x0;
     const __m256i m256 = _mm256_set_epi32(no, yes, yes, yes, yes, yes, yes, yes);
 #   if defined(LIBXSMM_GEMM_DIFF_MASK_A)
-    const __m256i a256 = _mm256_maskload_epi32((const void*)reference, m256);
+    const __m256i a256 = _mm256_maskload_epi32((const int*)reference, m256);
 #   else
     /* SKX: consider _mm256_maskz_expandloadu_epi32 */
     const __m256i a256 = _mm256_loadu_si256((const __m256i*)reference);
@@ -501,11 +471,10 @@ unsigned int libxsmm_gemm_diffn_avx512(const libxsmm_gemm_descriptor* reference,
   return libxsmm_gemm_diffn_avx2(reference, descs, hint, ndescs, nbytes);
 # endif
 #else
-# if !defined(NDEBUG) && (defined(LIBXSMM_GEMM_DIFF_AVX512_MIC) || defined(LIBXSMM_GEMM_DIFF_AVX512_CORE))
-  { static LIBXSMM_TLS int once = 0;
-    if (0 == once) {
+# if !defined(NDEBUG) && defined(LIBXSMM_GEMM_DIFF_AVX512)
+  { static int error_once = 0;
+    if (1 == LIBXSMM_ATOMIC_ADD_FETCH(&error_once, 1, LIBXSMM_ATOMIC_RELAXED)) {
       fprintf(stderr, "LIBXSMM: unable to enter AVX-512 code path!\n");
-      once = 1;
     }
   }
 # endif
@@ -514,8 +483,7 @@ unsigned int libxsmm_gemm_diffn_avx512(const libxsmm_gemm_descriptor* reference,
 }
 
 
-LIBXSMM_EXTERN_C LIBXSMM_RETARGETABLE
-unsigned int libxsmm_gemm_diffn_imci(const libxsmm_gemm_descriptor* reference, const libxsmm_gemm_descriptor* descs,
+LIBXSMM_GEMM_DIFF_API_DEFINITION unsigned int libxsmm_gemm_diffn_imci(const libxsmm_gemm_descriptor* reference, const libxsmm_gemm_descriptor* descs,
   unsigned int hint, unsigned int ndescs, int nbytes)
 {
 #if defined(LIBXSMM_GEMM_DIFF_KNC) && defined(__MIC__) && (28 == LIBXSMM_GEMM_DESCRIPTOR_SIZE)
@@ -555,3 +523,4 @@ unsigned int libxsmm_gemm_diffn_imci(const libxsmm_gemm_descriptor* reference, c
 #endif
 }
 
+#endif /*LIBXSMM_GEMM_DIFF_C*/
