@@ -1,5 +1,5 @@
 /******************************************************************************
-** Copyright (c) 2016, Intel Corporation                                     **
+** Copyright (c) 2016-2017, Intel Corporation                                **
 ** All rights reserved.                                                      **
 **                                                                           **
 ** Redistribution and use in source and binary forms, with or without        **
@@ -29,10 +29,16 @@
 /* Hans Pabst (Intel Corp.)
 ******************************************************************************/
 #include "libxsmm_trace.h"
+#include <libxsmm_sync.h>
+
+#if !defined(LIBXSMM_TRACE_DLINFO)
+/*# define LIBXSMM_TRACE_DLINFO*/
+#endif
 
 #if defined(LIBXSMM_OFFLOAD_TARGET)
 # pragma offload_attribute(push,target(LIBXSMM_OFFLOAD_TARGET))
 #endif
+#include <assert.h>
 #include <stdlib.h>
 # if (!defined(_BSD_SOURCE) || 0 == _BSD_SOURCE) && (!defined(_SVID_SOURCE) || 0 == _SVID_SOURCE) && \
      (!defined(_XOPEN_SOURCE) || !defined(_XOPEN_SOURCE_EXTENDED) || 500 > _XOPEN_SOURCE) && \
@@ -41,30 +47,32 @@
 int mkstemp(char* filename_template);
 # endif
 #include <string.h>
-#include <assert.h>
 #include <stdio.h>
 #if !defined(NDEBUG)
 # include <errno.h>
 #endif
 #if defined(_WIN32) || defined(__CYGWIN__)
 # include <Windows.h>
+# if defined(_MSC_VER)
+#   pragma warning(push)
+#   pragma warning(disable: 4091)
+# endif
 # include <DbgHelp.h>
+# if defined(_MSC_VER)
+#   pragma warning(pop)
+# endif
 #else
 # include <execinfo.h>
 # include <sys/stat.h>
 # include <sys/mman.h>
 # include <unistd.h>
 # include <fcntl.h>
-# if defined(_REENTRANT)
-#   include <pthread.h>
+# if defined(LIBXSMM_TRACE_DLINFO)
+#   include <dlfcn.h>
 # endif
 # if defined(__APPLE__) && defined(__MACH__)
-/* taken from libtransmission fdlimit.c */
-LIBXSMM_INLINE
-#if defined(__GNUC__)
-LIBXSMM_ATTRIBUTE(no_instrument_function)
-#endif
-LIBXSMM_RETARGETABLE int posix_fallocate(int fd, off_t offset, off_t length)
+/* taken from "libtransmission" fdlimit.c */
+LIBXSMM_INLINE LIBXSMM_RETARGETABLE int posix_fallocate(int fd, off_t offset, off_t length)
 {
   fstore_t fst;
   fst.fst_flags = F_ALLOCATECONTIG;
@@ -92,35 +100,21 @@ int posix_fallocate(int, off_t, off_t);
 # undef LIBXSMM_TRACE_SYMBOLSIZE
 # define LIBXSMM_TRACE_SYMBOLSIZE 256
 #endif
-#if defined(__GNUC__)
-# if !defined(LIBXSMM_TRACE_GCCATOMICS)
-#   if (LIBXSMM_VERSION3(4, 7, 4) <= LIBXSMM_VERSION3(__GNUC__, __GNUC_MINOR__, __GNUC_PATCHLEVEL__))
-#     define LIBXSMM_TRACE_GCCATOMICS 1
-#   else
-#     define LIBXSMM_TRACE_GCCATOMICS 0
-#   endif
-# endif
-#endif
-
 
 #if defined(_WIN32) || defined(__CYGWIN__)
 # define LIBXSMM_TRACE_MINDEPTH 5
-LIBXSMM_RETARGETABLE LIBXSMM_VISIBILITY_INTERNAL LIBXSMM_ALIGNED(volatile LONG internal_trace_initialized, 32) = -1;
+LIBXSMM_EXTERN_C LIBXSMM_RETARGETABLE volatile LONG internal_trace_initialized /*= -1*/;
 #else
 # define LIBXSMM_TRACE_MINDEPTH 4
-LIBXSMM_RETARGETABLE LIBXSMM_VISIBILITY_INTERNAL LIBXSMM_ALIGNED(int internal_trace_initialized, 32) = -1;
-LIBXSMM_RETARGETABLE LIBXSMM_VISIBILITY_INTERNAL pthread_key_t internal_trace_key = 0;
-
-LIBXSMM_INLINE
-#if defined(__GNUC__)
-LIBXSMM_ATTRIBUTE(no_instrument_function)
+LIBXSMM_EXTERN_C LIBXSMM_RETARGETABLE int internal_trace_initialized /*= -1*/;
+#if !defined(LIBXSMM_NO_SYNC)
+LIBXSMM_EXTERN_C LIBXSMM_RETARGETABLE pthread_key_t internal_trace_key /*= 0*/;
 #endif
-LIBXSMM_RETARGETABLE void internal_delete(void* value)
+LIBXSMM_INLINE LIBXSMM_RETARGETABLE void internal_delete(void* value)
 {
   int fd;
   assert(value);
   fd = *((int*)value);
-
 #if defined(NDEBUG)
   munmap(value, LIBXSMM_TRACE_SYMBOLSIZE);
 #else /* library code is expected to be mute */
@@ -130,7 +124,6 @@ LIBXSMM_RETARGETABLE void internal_delete(void* value)
       strerror(error), error, value);
   }
 #endif
-
   if (0 <= fd) {
     close(fd);
   }
@@ -142,87 +135,64 @@ LIBXSMM_RETARGETABLE void internal_delete(void* value)
 }
 #endif /*!defined(_WIN32) && !defined(__CYGWIN__)*/
 
-LIBXSMM_RETARGETABLE LIBXSMM_VISIBILITY_INTERNAL int internal_trace_threadid = -1/*all*/;
-LIBXSMM_RETARGETABLE LIBXSMM_VISIBILITY_INTERNAL int internal_trace_mindepth =  0/*all*/;
-LIBXSMM_RETARGETABLE LIBXSMM_VISIBILITY_INTERNAL int internal_trace_maxnsyms = -1/*all*/;
+
+LIBXSMM_EXTERN_C LIBXSMM_RETARGETABLE int internal_trace_mindepth /*=  0*/;
+LIBXSMM_EXTERN_C LIBXSMM_RETARGETABLE int internal_trace_threadid /*= -1*/;
+LIBXSMM_EXTERN_C LIBXSMM_RETARGETABLE int internal_trace_maxnsyms /*= -1*/;
 
 
-LIBXSMM_EXTERN_C
+LIBXSMM_API
 #if defined(__GNUC__)
 LIBXSMM_ATTRIBUTE(no_instrument_function)
 #endif
-LIBXSMM_RETARGETABLE int libxsmm_trace_init(
-  int filter_threadid, int filter_mindepth, int filter_maxnsyms)
+int libxsmm_trace_init(int filter_threadid, int filter_mindepth, int filter_maxnsyms);
+
+LIBXSMM_API_DEFINITION int libxsmm_trace_init(int filter_threadid, int filter_mindepth, int filter_maxnsyms)
 {
-  int result;
-#if !defined(__TRACE)
-  result = EXIT_FAILURE;
-#elif defined(_WIN32) || defined(__CYGWIN__)
-  SymSetOptions(SYMOPT_DEFERRED_LOADS | SYMOPT_UNDNAME);
-  result = FALSE != SymInitialize(GetCurrentProcess(), NULL, TRUE)
-    ? EXIT_SUCCESS
-    : GetLastError();
-#else
-  result = pthread_key_create(&internal_trace_key, internal_delete);
-#endif
-  if (EXIT_SUCCESS == result) {
-    internal_trace_threadid = filter_threadid;
-    internal_trace_mindepth = filter_mindepth;
-    internal_trace_maxnsyms = filter_maxnsyms;
-#if (defined(_REENTRANT) || defined(_OPENMP)) && defined(LIBXSMM_TRACE_GCCATOMICS)
-# if (0 != LIBXSMM_TRACE_GCCATOMICS)
-    __atomic_store_n(&internal_trace_initialized, 0, __ATOMIC_SEQ_CST);
-# else
-    __sync_and_and_fetch(&internal_trace_initialized, 0);
+  int result = EXIT_SUCCESS;
+  internal_trace_initialized = -1; /* disabled */
+#if defined(__TRACE)
+  if (0 != filter_maxnsyms) { /* enabled */
+# if defined(_WIN32) || defined(__CYGWIN__)
+    SymSetOptions(SYMOPT_DEFERRED_LOADS | SYMOPT_UNDNAME);
+    result = (FALSE != SymInitialize(GetCurrentProcess(), NULL, TRUE) ? EXIT_SUCCESS : GetLastError());
+# elif !defined(LIBXSMM_NO_SYNC)
+    result = pthread_key_create(&internal_trace_key, internal_delete);
 # endif
-#elif (defined(_REENTRANT) || defined(_OPENMP)) && defined(_WIN32)
-    internal_trace_initialized = 0; /*TODO*/
-#else
-    internal_trace_initialized = 0;
-#endif
+    if (EXIT_SUCCESS == result) {
+      internal_trace_threadid = filter_threadid;
+      internal_trace_maxnsyms = filter_maxnsyms;
+      internal_trace_mindepth = filter_mindepth;
+      internal_trace_initialized = 0; /* enabled */
+    }
   }
+#else
+  LIBXSMM_UNUSED(filter_threadid);
+  LIBXSMM_UNUSED(filter_mindepth);
+  LIBXSMM_UNUSED(filter_maxnsyms);
+#endif
   return result;
 }
 
 
-LIBXSMM_EXTERN_C
+LIBXSMM_API
 #if defined(__GNUC__)
 LIBXSMM_ATTRIBUTE(no_instrument_function)
 #endif
-LIBXSMM_RETARGETABLE int libxsmm_trace_finalize(void)
+int libxsmm_trace_finalize(void);
+
+LIBXSMM_API_DEFINITION int libxsmm_trace_finalize(void)
 {
   int result;
 #if !defined(__TRACE)
   result = EXIT_FAILURE;
 #else
-# if (defined(_REENTRANT) || defined(_OPENMP)) && defined(LIBXSMM_TRACE_GCCATOMICS)
-#   if (0 != LIBXSMM_TRACE_GCCATOMICS)
-  const int initialized = __atomic_load_n(&internal_trace_initialized, __ATOMIC_RELAXED);
-#   else
-  const int initialized = __sync_or_and_fetch(&internal_trace_initialized, 0);
-#   endif
-# elif (defined(_REENTRANT) || defined(_OPENMP)) && defined(_WIN32)
-  const int initialized = internal_trace_initialized; /*TODO*/
-# else
-  const int initialized = internal_trace_initialized;
-# endif
+  const int initialized = LIBXSMM_ATOMIC_LOAD(&internal_trace_initialized, LIBXSMM_ATOMIC_RELAXED);
   if (0 == initialized) {
-# if (defined(_REENTRANT) || defined(_OPENMP)) && defined(LIBXSMM_TRACE_GCCATOMICS)
-#   if (0 != LIBXSMM_TRACE_GCCATOMICS)
-    __atomic_store_n(&internal_trace_initialized, -1, __ATOMIC_SEQ_CST);
-#   else
-    __sync_or_and_fetch(&internal_trace_initialized, -1);
-#   endif
-# elif (defined(_REENTRANT) || defined(_OPENMP)) && defined(_WIN32)
-    internal_trace_initialized = -1; /*TODO*/
-# else
-    internal_trace_initialized = -1;
-# endif
+    LIBXSMM_ATOMIC_STORE(&internal_trace_initialized, -1/*disable*/, LIBXSMM_ATOMIC_SEQ_CST);
 # if defined(_WIN32) || defined(__CYGWIN__)
-    result = FALSE != SymCleanup(GetCurrentProcess())
-      ? EXIT_SUCCESS
-      : GetLastError();
-# else
+    result = (FALSE != SymCleanup(GetCurrentProcess()) ? EXIT_SUCCESS : GetLastError());
+# elif !defined(LIBXSMM_NO_SYNC)
     result = pthread_key_delete(internal_trace_key);
 # endif
   }
@@ -234,18 +204,19 @@ LIBXSMM_RETARGETABLE int libxsmm_trace_finalize(void)
 }
 
 
-LIBXSMM_EXTERN_C
+LIBXSMM_API
+#if defined(__GNUC__)
+LIBXSMM_ATTRIBUTE(no_instrument_function)
+#endif
+const char* libxsmm_trace_info(unsigned int* depth, unsigned int* threadid, const int* filter_threadid, const int* filter_mindepth, const int* filter_maxnsyms);
+
+LIBXSMM_API_DEFINITION
 #if defined(_WIN32)
 /*TODO: no inline*/
 #elif defined(__GNUC__)
-LIBXSMM_ATTRIBUTE(noinline)
-LIBXSMM_ATTRIBUTE(no_instrument_function)
+/*LIBXSMM_ATTRIBUTE(noinline)*/
 #endif
-LIBXSMM_RETARGETABLE const char* libxsmm_trace_info(
-  unsigned int* depth, unsigned int* threadid,
-  const int* filter_threadid,
-  const int* filter_mindepth,
-  const int* filter_maxnsyms)
+const char* libxsmm_trace_info(unsigned int* depth, unsigned int* threadid, const int* filter_threadid, const int* filter_mindepth, const int* filter_maxnsyms)
 {
   const char *fname = NULL;
 #if defined(__TRACE)
@@ -263,17 +234,7 @@ LIBXSMM_RETARGETABLE const char* libxsmm_trace_info(
 # if defined(__GNUC__)
     __asm__("");
 # endif
-# if (defined(_REENTRANT) || defined(_OPENMP)) && defined(LIBXSMM_TRACE_GCCATOMICS)
-#   if (0 != LIBXSMM_TRACE_GCCATOMICS)
-    i = __atomic_load_n(&internal_trace_initialized, __ATOMIC_RELAXED);
-#   else
-    i = __sync_or_and_fetch(&internal_trace_initialized, 0);
-#   endif
-# elif (defined(_REENTRANT) || defined(_OPENMP)) && defined(_WIN32)
-    i = internal_trace_initialized; /*TODO*/
-# else
-    i = internal_trace_initialized;
-# endif
+    i = LIBXSMM_ATOMIC_LOAD(&internal_trace_initialized, LIBXSMM_ATOMIC_RELAXED);
     if (0 <= i) { /* do nothing if not yet initialized */
       const int mindepth = filter_mindepth ? *filter_mindepth : internal_trace_mindepth;
       const int maxnsyms = filter_maxnsyms ? *filter_maxnsyms : internal_trace_maxnsyms;
@@ -301,17 +262,7 @@ LIBXSMM_RETARGETABLE const char* libxsmm_trace_info(
             abs_tid = (0 <= tid ? tid : -tid);
           }
           else {
-# if (defined(_REENTRANT) || defined(_OPENMP)) && defined(LIBXSMM_TRACE_GCCATOMICS)
-#   if (0 != LIBXSMM_TRACE_GCCATOMICS)
-            abs_tid = __atomic_add_fetch(&internal_trace_initialized, 1, __ATOMIC_RELAXED);
-#   else
-            abs_tid = __sync_add_and_fetch(&internal_trace_initialized, 1);
-#   endif
-# elif (defined(_REENTRANT) || defined(_OPENMP)) && defined(_WIN32)
-            abs_tid = _InterlockedIncrement(&internal_trace_initialized);
-# else
-            abs_tid = ++internal_trace_initialized;
-# endif
+            abs_tid = LIBXSMM_ATOMIC_ADD_FETCH(&internal_trace_initialized, 1, LIBXSMM_ATOMIC_RELAXED);
             /* use sign bit to flag enabled fallback for symbol resolution */
             tid = -abs_tid;
           }
@@ -333,8 +284,14 @@ LIBXSMM_RETARGETABLE const char* libxsmm_trace_info(
             if (threadid) *threadid = abs_tid - 1;
           }
 # else
-          char *const raw_value = (char*)pthread_getspecific(internal_trace_key), *value = 0;
+#   if defined(LIBXSMM_NO_SYNC)
+          static char raw_c;
+          char *const raw_value = &raw_c;
+#   else
+          char *const raw_value = (char*)pthread_getspecific(internal_trace_key);
+#   endif
           int* ivalue = 0, fd = -1;
+          char* value = 0;
 
           if (raw_value) {
             ivalue = (int*)raw_value;
@@ -363,22 +320,17 @@ LIBXSMM_RETARGETABLE const char* libxsmm_trace_info(
               if (MAP_FAILED != buffer) {
                 int check = -1;
                 ivalue = (int*)buffer;
-                ivalue[0] = fd; /* valid fd for internal_delete */
+                ivalue[0] = fd; /* valid file descriptor for internal_delete */
 
-                if (0 == pthread_setspecific(internal_trace_key, buffer)
-                  && (sizeof(int) * 1) == read(fd, &check, sizeof(int))
+                if (
+#   if !defined(LIBXSMM_NO_SYNC)
+                  0 == pthread_setspecific(internal_trace_key, buffer) &&
+#   endif
+                     (sizeof(int) * 1) == read(fd, &check, sizeof(int))
                   && (sizeof(int) * 2) == lseek(fd, sizeof(int), SEEK_CUR)
                   && check == fd)
                 {
-# if (defined(_REENTRANT) || defined(_OPENMP)) && defined(LIBXSMM_TRACE_GCCATOMICS)
-#   if (0 != LIBXSMM_TRACE_GCCATOMICS)
-                  abs_tid = __atomic_add_fetch(&internal_trace_initialized, 1, __ATOMIC_RELAXED);
-#   else
-                  abs_tid = __sync_add_and_fetch(&internal_trace_initialized, 1);
-#   endif
-# else
-                  abs_tid = ++internal_trace_initialized;
-# endif
+                  abs_tid = LIBXSMM_ATOMIC_ADD_FETCH(&internal_trace_initialized, 1, LIBXSMM_ATOMIC_RELAXED);
                   assert(0 < abs_tid);
                   /* use sign bit to flag enabled fallback for symbol resolution */
                   ivalue[1] = -abs_tid;
@@ -435,11 +387,6 @@ LIBXSMM_RETARGETABLE const char* libxsmm_trace_info(
           }
 # endif
         }
-# if !defined(NDEBUG) /* library code is expected to be mute */
-        else {
-          fprintf(stderr, "LIBXSMM: failed to capture stack trace\n");
-        }
-# endif
       }
     }
 
@@ -456,15 +403,13 @@ LIBXSMM_RETARGETABLE const char* libxsmm_trace_info(
 }
 
 
-LIBXSMM_EXTERN_C
+LIBXSMM_API
 #if defined(__GNUC__)
 LIBXSMM_ATTRIBUTE(no_instrument_function)
 #endif
-LIBXSMM_RETARGETABLE void libxsmm_trace(
-  FILE* stream, unsigned int depth,
-  const int* filter_threadid,
-  const int* filter_mindepth,
-  const int* filter_maxnsyms)
+void libxsmm_trace(FILE* stream, unsigned int depth, const int* filter_threadid, const int* filter_mindepth, const int* filter_maxnsyms);
+
+LIBXSMM_API_DEFINITION void libxsmm_trace(FILE* stream, unsigned int depth, const int* filter_threadid, const int* filter_mindepth, const int* filter_maxnsyms)
 {
 #if defined(__TRACE)
   unsigned int depth1 = depth + 1, threadid;
@@ -474,7 +419,7 @@ LIBXSMM_RETARGETABLE void libxsmm_trace(
   if (name && *name) { /* implies actual other results to be valid */
     const int depth0 = LIBXSMM_MAX(filter_mindepth ? *filter_mindepth : internal_trace_mindepth, 0);
     assert(0 != stream/*otherwise fprintf handle the error*/);
-    if ((0 == filter_threadid && 0 > internal_trace_threadid) || (filter_threadid && 0 > *filter_threadid)) {
+    if ((0 == filter_threadid && 0 > internal_trace_threadid) || (0 != filter_threadid && 0 > *filter_threadid)) {
       fprintf(stream, "%*s%s@%u\n", (int)(depth1 - depth0), "", name, threadid);
     }
     else {
@@ -491,15 +436,36 @@ LIBXSMM_RETARGETABLE void libxsmm_trace(
 
 
 #if defined(__GNUC__)
-LIBXSMM_EXTERN_C LIBXSMM_RETARGETABLE LIBXSMM_ATTRIBUTE(no_instrument_function) void __cyg_profile_func_enter(void* this_fn, void* call_site);
-LIBXSMM_EXTERN_C LIBXSMM_RETARGETABLE LIBXSMM_ATTRIBUTE(no_instrument_function) void __cyg_profile_func_enter(void* this_fn, void* call_site)
+
+#if !defined(LIBXSMM_BUILD) && defined(__APPLE__) && defined(__MACH__)
+LIBXSMM_PRAGMA_OPTIMIZE_OFF
+#endif
+
+LIBXSMM_EXTERN_C
+#if defined(__cplusplus) || defined(LIBXSMM_BUILD)
+LIBXSMM_API
+#else
+static
+#endif
+LIBXSMM_ATTRIBUTE(no_instrument_function) void __cyg_profile_func_enter(void* this_fn, void* call_site);
+
+LIBXSMM_EXTERN_C
+#if defined(__cplusplus) || defined(LIBXSMM_BUILD)
+LIBXSMM_API_DEFINITION
+#else
+static
+#endif
+void __cyg_profile_func_enter(void* this_fn, void* call_site)
 {
 #if defined(__TRACE)
-# if 1
+# if !defined(LIBXSMM_TRACE_DLINFO)
   LIBXSMM_UNUSED(this_fn); LIBXSMM_UNUSED(call_site); /* suppress warning */
   libxsmm_trace(stderr, 1/*no need for parent (0) but parent of parent (1)*/,
     /* inherit global settings from libxsmm_trace_init */
     NULL, NULL, NULL);
+# else
+# if 1
+  Dl_info info;
 # else
   struct {
       const char *dli_fname;
@@ -509,11 +475,12 @@ LIBXSMM_EXTERN_C LIBXSMM_RETARGETABLE LIBXSMM_ATTRIBUTE(no_instrument_function)
                                  lower than addr */
       void       *dli_saddr;
   } info;
-  if (dladdr(this_fn, &info)) {
-    if (info.dli_sname) {
+# endif
+  if (0 != dladdr(this_fn, (Dl_info*)&info)) {
+    if (0 != info.dli_sname) {
       fprintf(stderr, "%s\n", info.dli_sname);
     }
-    else if (info.dli_saddr) {
+    else if (0 != info.dli_saddr) {
       fprintf(stderr, "0x%llx\n", (unsigned long long)info.dli_saddr);
     }
   }
@@ -523,10 +490,29 @@ LIBXSMM_EXTERN_C LIBXSMM_RETARGETABLE LIBXSMM_ATTRIBUTE(no_instrument_function)
 #endif
 }
 
-LIBXSMM_EXTERN_C LIBXSMM_RETARGETABLE LIBXSMM_ATTRIBUTE(no_instrument_function) void __cyg_profile_func_exit(void* this_fn, void* call_site);
-LIBXSMM_EXTERN_C LIBXSMM_RETARGETABLE LIBXSMM_ATTRIBUTE(no_instrument_function) void __cyg_profile_func_exit(void* this_fn, void* call_site)
+
+LIBXSMM_EXTERN_C
+#if defined(__cplusplus) || defined(LIBXSMM_BUILD)
+LIBXSMM_API
+#else
+static
+#endif
+LIBXSMM_ATTRIBUTE(no_instrument_function) void __cyg_profile_func_exit(void* this_fn, void* call_site);
+
+LIBXSMM_EXTERN_C
+#if defined(__cplusplus) || defined(LIBXSMM_BUILD)
+LIBXSMM_API_DEFINITION
+#else
+static
+#endif
+void __cyg_profile_func_exit(void* this_fn, void* call_site)
 {
   LIBXSMM_UNUSED(this_fn); LIBXSMM_UNUSED(call_site); /* suppress warning */
 }
+
+#if !defined(LIBXSMM_BUILD) && defined(__APPLE__) && defined(__MACH__)
+LIBXSMM_PRAGMA_OPTIMIZE_ON
+#endif
+
 #endif /*defined(__GNUC__)*/
 
